package appl.data;

import java.io.Serializable;

/**
 * A class that automatically loads a specified object and returns it, as soon
 * as the {@link #load()} method is invoked. It is the counterpart of {@link DataUnloader}.
 * 
 * @author Dominik Appl
 * @see DataUnloader
 */
public interface DataLoader extends Serializable {
	
	/**
	 * On invocation the data is loaded and returned
	 * 
	 * @return the newly loaded object
	 * 
	 * @throws LoadingException if for some reason the data could not
	 * be loaded
	 */
	public Object load() throws LoadingException;
	
	/**
	 * @return some general information about the loading class 
	 * (like what the class exactly loads)
	 * 
	 */
	public String getLoadInfo();

}
