package appl.data;

import schmitzm.data.WritableGrid;
import appl.util.RasterMetaData;
import edu.bonn.xulu.plugin.io.grid.WritableGridFactory;

/**
 * This is an implementation of a loader class which loads data with a
 * {@link WritableGridFactory}. It is used for Late Loading.
 * 
 * @author Dominik Appl
 * @see WritableGridLLProxy
 */
public class FactoryLoader implements DataLoader {

	private WritableGridFactory fac;

	private RasterMetaData metaData;

	private FactoryLoader() {
	}

	/**
	 * @param factory
	 *            the factory which is used for instantiation
	 * @param metaData
	 *            the meta data is required for grid instantiation
	 */
	public FactoryLoader(WritableGridFactory factory, RasterMetaData metaData) {
		fac = factory;
		this.metaData = metaData;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see appl.data.DataLoader#load()
	 */
	public WritableGrid load() {
		return fac.newInstance(metaData.getDataType(), metaData.getWidth(),
				metaData.getHeight(), metaData.getMinX(), metaData.getMinY(),
				metaData.getX(), metaData.getY(), metaData.getRealWidth(),
				metaData.getRealHeight(), metaData
						.getCoordinateReferenceSystem());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see appl.data.DataLoader#getLoadInfo()
	 */
	public String getLoadInfo() {
		return "FactoryLoader|" + fac + "|" + metaData;
	}

}
