package appl.data;

import edu.bonn.xulu.appl.XuluRegistry;
import edu.bonn.xulu.io.ImportFactory;

/**
 * This is an implementation of a loader class which loads data with an
 * importer. It is for Late Loading.
 * 
 * @author Dominik Appl
 * @see WritableGridLLProxy
 * @author Dominik Appl
 */
public class ImportFactoryLoader implements DataLoader {

	private ImportFactory fac;

	private Object inputPara;

	private XuluRegistry reg;

	/**
	 * not usable
	 */
	private ImportFactoryLoader() {
	}

	/**
	 * @param importFac
	 *            the import factory to be used for loading
	 * @param inputPara
	 *            the input object (usually a file)
	 * @param reg
	 *            the {@link XuluRegistry}
	 */
	public ImportFactoryLoader(ImportFactory importFac, Object inputPara,
			XuluRegistry reg) {
		fac = importFac;
		this.inputPara = inputPara;
		this.reg = reg;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see appl.data.DataLoader#load()
	 */
	public Object load() throws LoadingException {
		try {
			return fac.importObject(inputPara, reg);
		} catch (Exception e) {
			throw new LoadingException(
					"Could not load Grid with Importfactory <"
							+ fac.getClass().getName() + ">" + " and "
							+ inputPara, inputPara);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see appl.data.DataLoader#getLoadInfo()
	 */
	public String getLoadInfo() {
		return "ImportFactoryLoader|" + fac + "|" + inputPara;
	}

}
