package appl.data;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import schmitzm.data.WritableGrid;

/**
 * @author Dominik Appl
 */
public class SerializedDataLoader implements DataLoader {

	protected File loadFile;

	private boolean del_after_loading;

	private SerializedDataLoader() {
	}

	public SerializedDataLoader(File loadFile,
			boolean DELETE_SOURCE_AFTER_LOADING) {
		this.loadFile = loadFile;
		del_after_loading = DELETE_SOURCE_AFTER_LOADING;
	}

	public SerializedDataLoader(File loadFile) {
		this(loadFile, false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see appl.data.DataLoader#load()
	 */
	public Object load() throws LoadingException {
		Object loadedObject = null;
		try {
			BufferedInputStream fis = new BufferedInputStream(
					new FileInputStream(loadFile));
			ObjectInputStream ois = new ObjectInputStream(fis);
			loadedObject = ois.readObject();
			ois.close();
			fis.close();
			// remove File from Filesystem
			if (del_after_loading)
				loadFile.delete();

		} catch (FileNotFoundException e) {
			throw new LoadingException("Could not find temporary unload-file:"
					+ loadFile, loadFile);

		} catch (Exception e) {
			e.printStackTrace();
			throw new LoadingException("Could not load from temporary file "
					+ loadFile, loadFile);

		}
		return loadedObject;

	}

	public static void SerializeToFile(Object o, File destFile)
			throws LoadingException {
		try {
			destFile.createNewFile();
			BufferedOutputStream fos = new BufferedOutputStream(
					new FileOutputStream(destFile));
			ObjectOutputStream oos = new ObjectOutputStream(fos);
			oos.writeObject(o);
			oos.close();
			fos.close();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see appl.data.DataLoader#getLoadInfo()
	 */
	public String getLoadInfo() {
		return "SerializedDataLoader|" + loadFile.getPath();
	}

}
