package appl.data;

import java.awt.Rectangle;

import schmitzm.data.WritableGrid;
import schmitzm.data.WritableGridArray;
import appl.util.RasterMetaData;
import edu.bonn.xulu.appl.XuluRegistry;
import edu.bonn.xulu.io.ImportFactory;
import edu.bonn.xulu.plugin.io.grid.WritableGridFactory;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

/**
 * This is an a late loading implementation for {@link WritableGrid WritableGrids}.
 * 
 * @author Dominik Appl
 */
public class WritableGridLLProxy extends LateLoadingProxy implements
		WritableGrid {

	protected RasterMetaData metaData;

	public WritableGridLLProxy(ImportFactory importFac,
			RasterMetaData metaData, Object inputPara, XuluRegistry reg) {
		super(new ImportFactoryLoader(importFac, inputPara, reg));
		this.metaData = metaData;
	}

	public WritableGridLLProxy(WritableGridFactory targetFactory,
			RasterMetaData metaData) {
		super(new FactoryLoader(targetFactory, metaData));
		this.metaData = metaData;
	}

	public WritableGridLLProxy(RasterMetaData metaData) {
		super(new WritableGridArrayLoader(metaData));
		baseObject = null;
		loaded = false;
		this.metaData = metaData;
	}

	protected WritableGridLLProxy(RasterMetaData metaData, DataLoader loader) {
		super(loader);
		loaded = false;
		this.metaData = metaData;
	}

	public double convertRasterToReal(int cell, int dim) {
		if (!loaded)
			tryLoadingGrid();
		;
		return ((WritableGrid) baseObject).convertRasterToReal(cell, dim);
	}

	public int convertRealToRaster(double coord, int dim) {
		if (!loaded)
			tryLoadingGrid();
		return ((WritableGrid) baseObject).convertRealToRaster(coord, dim);
	}

	/** @see WritableGrid#dispose() */
	public void dispose() {
		if (baseObject != null)
			((WritableGrid) baseObject).dispose();
	}

	public Object getGridSample(double... coord) {
		if (!loaded)
			tryLoadingGrid();
		return ((WritableGrid) baseObject).getGridSample();
	}

	public byte getGridSampleAsByte(double... coord) {
		if (!loaded)
			tryLoadingGrid();
		return ((WritableGrid) baseObject).getGridSampleAsByte(coord);
	}

	public double getGridSampleAsDouble(double... coord) {
		if (!loaded)
			tryLoadingGrid();
		return ((WritableGrid) baseObject).getGridSampleAsDouble(coord);
	}

	public float getGridSampleAsFloat(double... coord) {
		if (!loaded)
			tryLoadingGrid();
		return ((WritableGrid) baseObject).getGridSampleAsFloat(coord);
	}

	public int getGridSampleAsInt(double... coord) {
		if (!loaded)
			tryLoadingGrid();
		return ((WritableGrid) baseObject).getGridSampleAsInt(coord);
	}

	public long getGridSampleAsLong(double... coord) {
		if (!loaded)
			tryLoadingGrid();
		return ((WritableGrid) baseObject).getGridSampleAsLong(coord);
	}

	public short getGridSampleAsShort(double... coord) {
		if (!loaded)
			tryLoadingGrid();
		return ((WritableGrid) baseObject).getGridSampleAsShort(coord);
	}

	public CoordinateReferenceSystem getCoordinateReferenceSystem() {
		return metaData.getCoordinateReferenceSystem();
	}

	public int getHeight() {
		return metaData.getHeight();
	}

	public int getMinX() {
		return metaData.getMinX();

	}

	public int getMinY() {
		return metaData.getMinY();
	}

	public Object getRasterSample(int... cell) {
		if (!loaded)
			tryLoadingGrid();
		return ((WritableGrid) baseObject).getRasterSample(cell);
	}

	public byte getRasterSampleAsByte(int... cell) {
		if (!loaded)
			tryLoadingGrid();
		return ((WritableGrid) baseObject).getRasterSampleAsByte(cell);
	}

	public double getRasterSampleAsDouble(int... cell) {
		if (!loaded)
			tryLoadingGrid();
		return ((WritableGrid) baseObject).getRasterSampleAsDouble(cell);
	}

	public float getRasterSampleAsFloat(int... cell) {
		if (!loaded)
			tryLoadingGrid();
		return ((WritableGrid) baseObject).getRasterSampleAsFloat(cell);
	}

	public int getRasterSampleAsInt(int... cell) {
		if (!loaded)
			tryLoadingGrid();
		return ((WritableGrid) baseObject).getRasterSampleAsInt(cell);
	}

	public long getRasterSampleAsLong(int... cell) {
		if (!loaded)
			tryLoadingGrid();
		return ((WritableGrid) baseObject).getRasterSampleAsLong(cell);
	}

	public short getRasterSampleAsShort(int... cell) {
		if (!loaded)
			tryLoadingGrid();
		return ((WritableGrid) baseObject).getRasterSampleAsShort(cell);
	}

	public double getRealHeight() {
		return metaData.getRealHeight();
	}

	public double getRealWidth() {
		return metaData.getRealWidth();
	}

	public int getSampleType() {
		return metaData.getDataType();
	}

	/** Returns the Width of the Grid */
	public int getWidth() {
		return metaData.getWidth();
	}

	public double getX() {
		return metaData.getX();
	}

	public double getY() {
		return metaData.getY();
	}

	public void setGridSample(Object value, double... coord) {
		if (!loaded)
			tryLoadingGrid();
		((WritableGrid) baseObject).setGridSample(value, coord);
	}

	public void setRasterSample(Object value, int... cell) {
		if (!loaded)
			tryLoadingGrid();
		((WritableGrid) baseObject).setRasterSample(value, cell);

	}

	/**
	 * Trys to load the grid and gives out an error to the LOG, if the loading
	 * fails.
	 */
	protected void tryLoadingGrid() {
		try {
			super.loadData();
		} catch (LoadingException e) {
			LOG.error("Error while Loading Grid " + this + " from Source "
					+ e.getLoadSource());
			e.printStackTrace();
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see schmitzm.data.WritableGrid#getCellHeight()
	 */
	public double getCellHeight() {
		// TODO Auto-generated method stub
		return metaData.getCellHeight();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see schmitzm.data.WritableGrid#getCellWidth()
	 */
	public double getCellWidth() {
		// TODO Auto-generated method stub
		return metaData.getCellWidth();
	}

	/**
	 * This class supports late loading
	 * 
	 * @see appl.data.LateLoadable#unloadData()
	 * @return true;
	 * @see appl.data.LateLoadable#isLateLoadable()
	 */
	public boolean isLateLoadable() {
		return true;
	}
}
