package appl.parallel;

import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Vector;

import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.gui.ModelControlFrame;

import appl.parallel.spmd.SPMDTask;
import appl.parallel.spmd.split.SinglePartitionInfo;
import appl.parallel.spmd.split.SplitMap;
import appl.parallel.spmd.split.SplittableResource;

/**
 * This is the base class for remote computing resources, which could be
 * displayed e.g. in the {@link ModelControlFrame} of the
 * {@link XuluModellingPlatform}. It provides very basic functionality like
 * connecting, disconnecting or pinging. Also it provides a flag saying if the
 * resource is available for computation (or e.g. in use).
 *
 * @author Dominik Appl
 */
public interface ComputingResource extends java.rmi.Remote {

	/**
	 * Should return Information about the ComputingResource. This Information
	 * is primary used to display infos to the user.
	 */
	public ComputingResourceProperties getResourceInformation()
			throws RemoteException;

	/**
	 * Can be used to ping the object
	 *
	 * @param o
	 *            the object to ping with
	 * @return the same object
	 * @throws RemoteException
	 */
	public Object ping(Object... o) throws RemoteException;

	/**
	 * tries to connect to the resource
	 *
	 * @return true for success
	 * @throws RemoteException
	 */
	public boolean connect() throws RemoteException;

	/**
	 * disconnects from the resource
	 *
	 * @throws RemoteException
	 */
	public void disconnect() throws RemoteException;

	/**
	 * @return true, if the resource is available for computation (may return
	 *         false, if the resource is used by another client)
	 * @throws RemoteException
	 */
	public boolean isAvailable() throws RemoteException;
}
