package appl.parallel.client;

import java.awt.Rectangle;
import java.rmi.Remote;
import java.rmi.RemoteException;

import appl.parallel.event.CommEvent;
import appl.parallel.event.CommEventSink;
import appl.parallel.server.PartitionDataServer;
import appl.parallel.spmd.split.DataPartition;
import appl.parallel.spmd.split.SplittableResource;

/**
 * A {@link DataServer} shares resources over a network. For
 * {@link SplittableResource}s it allows also the retrieval of
 * {@link DataPartition}s. The data is retrieved using a unique id identifying
 * the resource.
 *
 * @author Dominik Appl
 */
public interface DataServer extends Remote {

	/**
	 * gets the data with the id of a SplittableRessource
	 *
	 * @param id
	 *            the {@link SplittableResource#getRootID() ID} of a the base
	 *            data
	 * @param bounds
	 *            the bounds of the partition to retrieve (using global
	 *            coordinates)
	 * @return the partition
	 * @throws RemoteException
	 */
	public DataPartition getPartition(int id, Rectangle bounds)
			throws RemoteException;

	/**
	 * sets the data with the id of a SplittableRessource
	 *
	 * @param id
	 *            the {@link SplittableResource#getRootID() ID} of a the base
	 *            data
	 * @param bounds
	 *            the location where the partition is to be updated (using
	 *            global coordinates)
	 * @param updateData
	 *            the updateData (which may only a partitial update, depending
	 *            on the bounds set with the last parameter)
	 * @throws RemoteException
	 */
	public void updatePartition(int id, DataPartition updateData,
			Rectangle bounds) throws RemoteException;

	/**
	 * Returns the whole Partition with the given ID
	 *
	 * @param id
	 *            the id of the partition
	 * @return the partition
	 * @throws RemoteException
	 *             if the connection to the server fails
	 */
	public DataPartition getData(int id) throws RemoteException;

	/**
	 * Adds a Partition to the server
	 *
	 * @param partition
	 *            the partition to add
	 * @throws RemoteException
	 */
	public void addData(DataPartition partition) throws RemoteException;

	/**
	 * Removes the partition with the specified id.
	 *
	 * @param id
	 *            id of the partition to remove
	 * @throws RemoteException
	 */
	public void removeData(int id) throws RemoteException;

}
