package appl.parallel.data;

import java.awt.Rectangle;

import appl.data.LoadingException;
import appl.parallel.client.ClientDataServer;
import appl.parallel.spmd.split.DataPartition;

/**
 * Responsible for loading and unloading data partitions
 * 
 * @author Dominik Appl
 */
public interface PartitionDataHandler extends DataLoadHandler, Cloneable {

	/**
	 * sets the bounds of the data to be unloaded so that not all data is
	 * unloaded to the destination. This is for example useful if there is a
	 * neighborhood region which should not be uploaded to the destination.
	 * 
	 * @param bounds
	 */
	public void setUnloadBounds(Rectangle bounds);

	public void setBasePartition(DataPartition data);

	public void setRootID(int rootID);

	/**
	 * @param rootID
	 *            the id of the data
	 * @param partitionBounds
	 *            the bounds of the partition to be retrieved on server side
	 * @param unloadBounds
	 *            the bounds of the partition which is to be uploaded to the
	 *            client after calculation
	 * @return the new instance
	 */
	public PartitionDataHandler newInstance(int rootID,
			ClientDataServer spmdClient, Rectangle partitionBounds,
			Rectangle unloadBounds);

	public DataPartition load() throws LoadingException;

	public PartitionDataHandler clone();

	/**
	 * sets a local spmd client which MAY be used by the handler for local
	 * access
	 * 
	 * @param localSPMDClient
	 */
	public void setSPMDClient(ClientDataServer localSPMDClient);

}
