package appl.parallel.data.splittable;

import java.io.File;

import appl.data.WritableGridLLProxy;
import appl.ext.XuluConfig;
import appl.util.RasterUtil;

import schmitzm.data.WritableGrid;

import edu.bonn.xulu.appl.XuluRegistry;
import edu.bonn.xulu.io.AbstractFactory;
import edu.bonn.xulu.io.ImportFactory;

import edu.bonn.xulu.plugin.data.grid.MultiGrid;
import edu.bonn.xulu.plugin.data.grid.GridList;
import edu.bonn.xulu.plugin.io.grid.WritableGridFactory;
import edu.bonn.xulu.plugin.io.grid.array.WritableGridArrayFactory_ArcInfoAsciiGrid;
//import edu.bonn.xulu.plugin.io.grid.gt.GridListFactory_ArcInfoAsciiGrid;

// nur fuer Doku
import schmitzm.data.WritableGridArray;
import edu.bonn.xulu.io.InstantiationFactory;

/**
 * Diese Factory importiert und exportiert Instanzen des Datentyps
 * {@link MultiGrid} aus/in Dateien im ArcInfo-ASCII-Grid-Format.
 * Beim Import liefert die Factory eine Standard-Arrays basierendes Raster-Liste
 * 
 * 
 * @author Dominik Appl
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class MultiGridFactory_ArcInfoAsciiGrid extends GridListFactory_ArcInfoAsciiGrid {
  private static final ImportFactory importFac = new WritableGridArrayFactory_ArcInfoAsciiGrid();

  ////////////////////////////////////////////////////////////////////////
  ///////////////     Implementierung von ImportFactory     //////////////
  ////////////////////////////////////////////////////////////////////////
  /**
   * Liefert den Datentyp, den die Factory erzeugt.
   * @return immer {@link MultiGrid MultiGrid.class}
   */
  public Class getImportType() {
    return MultiGrid.class;
  }

  /**
   * Importiert Raster aus Dateien im ArcInfoAsciiGrid-Format
   * und fuegt diese in einem {@link MultiGrid} zusammen.
   * @param input Eingabe-Quelle (muss ein {@link File File[]} sein!)
   * @param reg   Instanz der Xulu-Registry, ueber die eine Factory ermittelt
   *              wird, die eine Standard-Instanz von {@link MultiGrid} erzeugt, in
   *              die das Objekt importiert wird
   * @exception UnsupportedOperationException falls als Eingabe-Quelle keine
   *            Dateien angegeben werden
   */
  public MultiGrid importObject(Object input, XuluRegistry reg) throws Exception {
    AbstractFactory.checkImportSourceObject(this,input,getImportSourceType(),true);

    // MultiGrid erst erzeugen, wenn ein Beispiel-Raster vorliegt
    MultiGrid mg = null;
    // Raster importieren und in GridList einfuegen
    for (int i=0; i<((Object[])input).length; i++) {
    	SplittableLLProxyGrid grid = new SplittableLLProxyGrid
                                (importFac,
                                 RasterUtil.getRasterMetaData_from_ArcGridASCII_File(((File[])input)[i]), 
                                 ((File[])input)[i], reg );
      //read and apply settings from the registry
      String unloadFolder = XuluConfig.getXuluConfig().getProperty("Datatypes.proxy.unloadFolder");
      boolean unloading = XuluConfig.getXuluConfig().getBooleanProperty("Datatypes.proxy.enableUnloading");
      if(unloadFolder!=null)
      	grid.setUnloadDir(unloadFolder);
      grid.setUnloading(unloading);
      // das erste Grid dient dem MultiGrid als Vorlage
      if ( mg == null )
        mg = new MultiGrid(grid,getWritableGridFactory());

      mg.addGrid(grid);
    }
    return mg;
  }

  /**
   * Liefert eine Instanz von {@link SplittableGridLLFactory}, die auf Standard-Arrays
   * basierende Instanzen von {@link WritableGrid} erzeugt. Mit diesen
   * Instanzen wird das SingleGrid gefuellt.
   */
  protected WritableGridFactory getWritableGridFactory() {
    return new SplittableGridLLFactory();
  }

}
