package appl.parallel.data.splittable;

import java.awt.geom.Rectangle2D;
import java.awt.image.DataBuffer;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

import appl.data.WritableGridLLProxy;
import appl.util.RasterMetaData;
import schmitzm.data.WritableGrid;
import edu.bonn.xulu.plugin.io.grid.WritableGridFactory;
import edu.bonn.xulu.plugin.io.grid.array.WritableGridArrayFactory;

// nur fuer Doku
import edu.bonn.xulu.data.XuluObject;
import edu.bonn.xulu.io.Factory;
import edu.bonn.xulu.plugin.data.grid.SingleGrid;

/**
 * Diese Factory erzeugt Standard-Instanzen des Datentyps {@link SplittableLLProxyGrid}.<br>
 * <b>Bemerke:</b><br>
 * Dieser stellt <b>kein</b> {@linkplain XuluObject Xulu-Objekt} dar, kann also
 * nicht direkt im Xulu-Datenpool gespeichert werden. Hierzu ist der Datentyp
 * {@link SingleGrid} (bzw. eine entsprechende Factory) zu verwenden!
 * @author Dominik Appl
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class SplittableGridLLFactory extends WritableGridFactory {


    WritableGridFactory  targetFactory = new WritableGridArrayFactory();


  /**
   * Liefert den Datentyp, den die Factory erzeugt.
   * @return immer {@link SplittableLLProxyGrid SplittableLLProxyGrid.class}
   */
  public Class getInstanceType() {
    return SplittableLLProxyGrid.class;
  }

  /**
   * Erzeugt ein neues Grid.
   * @param type    Datentyp der gespeicherten Objekte
   * @param widthc  Breite in Zellen
   * @param heightc Hoehe in Zellen
   * @param minX    Index der ersten Zelle in X-Richtung
   * @param minY    Index der ersten Zelle in Y-Richtung
   * @param x       Georeferenz Latitude (Suedliche/Untere Kante)
   * @param y       Georeferenz Longitute (Westliche/Linke Kante)
   * @param width   Breite
   * @param height  Hoehe
   */
  public SplittableLLProxyGrid newInstance(int type, int widthc, int heightc, int minX, int minY, double x, double y, double width, double height, CoordinateReferenceSystem crs) {


	  return new SplittableLLProxyGrid(targetFactory,new RasterMetaData(
			  											type,widthc,heightc,
			  											minX,minY,x,y,width,
			  											height, crs));
    }
}
