package appl.parallel.data.xulugridfile.factories;

import java.io.File;
import java.io.FileInputStream;

import appl.data.WritableGridLLProxy;
import appl.ext.XuluConfig;
import appl.parallel.data.xulugridfile.XuluGridFile;
import appl.parallel.data.xulugridfile.XuluGridFileConverter;
import appl.parallel.data.xulugridfile.XuluWritableGridFile;
import appl.util.RasterUtil;

import schmitzm.data.WritableGrid;

import edu.bonn.xulu.appl.XuluRegistry;
import edu.bonn.xulu.io.AbstractFactory;
import edu.bonn.xulu.io.ImportFactory;

import edu.bonn.xulu.plugin.data.grid.MultiGrid;
import edu.bonn.xulu.plugin.data.grid.GridList;
import edu.bonn.xulu.plugin.io.grid.WritableGridFactory;
import edu.bonn.xulu.plugin.io.grid.array.WritableGridArrayFactory_ArcInfoAsciiGrid;
//import edu.bonn.xulu.plugin.io.grid.gt.GridListFactory_ArcInfoAsciiGrid;

// nur fuer Doku
import schmitzm.data.WritableGridArray;
import edu.bonn.xulu.io.InstantiationFactory;

/**
 * Diese Factory importiert und exportiert Instanzen des Datentyps
 * {@link MultiGrid} aus/in Dateien im ArcInfo-ASCII-Grid-Format.
 * Beim Import liefert die Factory eine Standard-Arrays basierendes Raster-Liste
 * ({@link WritableGridArray}).
 * Entsprechend koennen beim Export nur {@link MultiGrid MultiGrids}
 * verarbeitet werden, die {@link WritableGridArray}-Instanzen in ihrer
 * Raster-Property besitzen.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class MultiGridFactory_ArcInfoAsciiGrid extends GridListFactory_ArcInfoAsciiGrid {
  private static final ImportFactory importFac = new WritableGridArrayFactory_ArcInfoAsciiGrid();

  ////////////////////////////////////////////////////////////////////////
  ///////////////     Implementierung von ImportFactory     //////////////
  ////////////////////////////////////////////////////////////////////////
  /**
   * Liefert den Datentyp, den die Factory erzeugt.
   * @return immer {@link MultiGrid MultiGrid.class}
   */
  public Class getImportType() {
    return MultiGrid.class;
  }

  /**
   * Importiert Raster aus Dateien im ArcInfoAsciiGrid-Format
   * und fuegt diese in einem {@link MultiGrid} zusammen.
   * @param input Eingabe-Quelle (muss ein {@link File File[]} sein!)
   * @param reg   Instanz der Xulu-Registry, ueber die eine Factory ermittelt
   *              wird, die eine Standard-Instanz von {@link MultiGrid} erzeugt, in
   *              die das Objekt importiert wird
   * @exception UnsupportedOperationException falls als Eingabe-Quelle keine
   *            Dateien angegeben werden
   */
  public MultiGrid importObject(Object input, XuluRegistry reg) throws Exception {
    AbstractFactory.checkImportSourceObject(this,input,getImportSourceType(),true);

    // MultiGrid erst erzeugen, wenn ein Beispiel-Raster vorliegt
    MultiGrid mg = null;
    // Raster importieren und in GridList einfuegen
    for (int i=0; i<((Object[])input).length; i++) {
    	File inputFile = (File) ((Object[]) input)[i];
        File outputFile = new File(inputFile.getAbsoluteFile()+".xgrid");
        //check if already a gridFile (should not be..)
        XuluGridFile xuluGridFile;
        if(inputFile.getAbsolutePath().endsWith(".xgrid"))
        	xuluGridFile = new XuluGridFile(inputFile);
        else
        {	
        //check if a the import should be converted even if a xulugridfile with the same name exists
        boolean alwaysreload = XuluConfig.getXuluConfig().getBooleanProperty("Datatypes.xulugrid.alwaysconvert");		
        //only convert if nessecary
        if(alwaysreload || (!outputFile.exists())){
        	 //the ArcInfoGrid must be converted to XuluGridFile
            XuluGridFileConverter converter = new XuluGridFileConverter(inputFile,outputFile);
            int dataType = RasterUtil.getRasterMetaData_from_ArcGridASCII_File(inputFile).getDataType();
            xuluGridFile = converter.convertArcInfoToXuluGridFile(dataType);
        }
        else
        	xuluGridFile = new XuluGridFile(outputFile);
        }
        WritableGrid grid = new XuluWritableGridFile(xuluGridFile);
    	
    	
    	
    	
      // das erste Grid dient dem MultiGrid als Vorlage
      if ( mg == null )
        mg = new MultiGrid(grid,getWritableGridFactory());

      mg.addGrid(grid);
    }
    return mg;
  }

  /**
   * Liefert eine Instanz von {@link XuluGridFactory}, die auf Standard-Arrays
   * basierende Instanzen von {@link WritableGrid} erzeugt. Mit diesen
   * Instanzen wird das SingleGrid gefuellt.
   */
  protected WritableGridFactory getWritableGridFactory() {
    return new XuluGridFactory();
  }

}
