package appl.parallel.data.xulugridfile.factories;

import java.awt.geom.Rectangle2D;
import java.awt.image.DataBuffer;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

import appl.data.WritableGridLLProxy;
import appl.ext.XuluConfig;
import appl.parallel.data.xulugridfile.XuluGridFile;
import appl.parallel.data.xulugridfile.XuluWritableGridFile;
import appl.util.RasterMetaData;
import schmitzm.data.WritableGrid;
import edu.bonn.xulu.plugin.io.grid.WritableGridFactory;
import edu.bonn.xulu.plugin.io.grid.array.WritableGridArrayFactory;

// nur fuer Doku
import edu.bonn.xulu.data.XuluObject;
import edu.bonn.xulu.io.Factory;
import edu.bonn.xulu.plugin.data.grid.SingleGrid;

/**
 *Creates instances of the memory saving data type {@link XuluGridFile} for the use in the Xulu
 *Plattform.
 *
 * @author Dominik Appl
 * @version 1.0
 */
public class XuluGridFactory extends WritableGridFactory {

	/**
	 * Liefert den Datentyp, den die Factory erzeugt.
	 * @return immer {@link XuluWritableGridFile XuluWritableGridFile.class}
	 */
	public Class getInstanceType() {
		return XuluWritableGridFile.class;
	}

	/**
	 * Creates a new grid under the directory given under
	 * @param type    Datentyp der gespeicherten Objekte
	 * @param widthc  Breite in Zellen
	 * @param heightc Hoehe in Zellen
	 * @param minX    Index der ersten Zelle in X-Richtung
	 * @param minY    Index der ersten Zelle in Y-Richtung
	 * @param x       Georeferenz Latitude (Suedliche/Untere Kante)
	 * @param y       Georeferenz Longitute (Westliche/Linke Kante)
	 * @param width   Breite
	 * @param height  Hoehe
	 */
	public WritableGridLLProxy newInstance(int type, int widthc, int heightc,
			int minX, int minY, double x, double y, double width,
			double height, CoordinateReferenceSystem crs) {

		String directory = XuluConfig.getXuluConfig().getProperty(
				"Datatypes.xulugrid.creationFolder");
		if (directory == null)
			directory = "Temp";

		return new XuluWritableGridFile(new RasterMetaData(type, widthc,
				heightc, minX, minY, x, y, width, height, crs), directory);
	}
}
