package appl.parallel.event;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import appl.parallel.client.RemoteEventHandler;
import appl.parallel.client.RemoteExecutionController;
import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.AbstractXuluPlugin;
import edu.bonn.xulu.appl.XuluPlugin;

/**
 * A simple {@link XuluPlugin} that outputs received {@link TimeEvent}s 
 * and {@link TransferEvent}s to the console.
 * 
 * @author Dominik Appl
 */
public class SimpleConsoleMonitor extends AbstractXuluPlugin implements
		TimeMonitor, TransferMonitor {

	protected final Logger LOG = LogManager
			.getLogger(this.getClass().getName());

	RemoteEventHandler eventProxy;

	public SimpleConsoleMonitor() {
		super(false);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see appl.parallel.event.TimeMonitor#receiveTimeEvent(appl.parallel.event.TimeEvent)
	 */
	public void receiveTimeEvent(TimeEvent t) {
		System.out.println(t);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see appl.parallel.event.TransferMonitor#receiveTransferEvent(appl.parallel.event.TransferEvent)
	 */
	public void receiveTransferEvent(TransferEvent t) {
		System.out.println(t);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.bonn.xulu.appl.XuluPlugin#execute(edu.bonn.xulu.XuluModellingPlatform)
	 */
	public void execute(XuluModellingPlatform appl) {
		super.execute(appl);
		RemoteExecutionController controller = RemoteExecutionController
				.getRemoteExecutionController(appl);
		if (controller == null) {
			LOG
					.error("Could not find RemoteExecutionController! -> Could not start SimpleConsoleMonitor");
			return;
		}
		eventProxy = controller.getEventProxy();
		eventProxy.addTimeEventListener(this);
		eventProxy.addTransferEventListener(this);
		started = true;
	}

	@Override
	public void stop() {
		eventProxy.removeTimeMonitor(this);
		eventProxy.removeTransferMonitor(this);
		// TODO Auto-generated method stub
		super.stop();
	}
}
