package appl.parallel.gui;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;

import schmitzm.lang.WorkingThread;
import schmitzm.lang.WorkingThreadAdapter;

import appl.parallel.client.RemoteExecutionController;
import appl.parallel.model.AbstractParallelStepModel;
import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.model.XuluModel;
import edu.bonn.xulu.plugin.gui.ModelControlContainer;
import edu.bonn.xulu.plugin.gui.ModelControlFrame_Basic;

/**
 * Introduces a new TAB into the {@link ModelControlFrame_Basic}
 * for controlling the parallel execution
 *
 * @author Dominik Appl
 */
public class ModelControlFrame_parallel extends ModelControlFrame_Tabbed {

	protected ParallelControlPanelEngine spcEngine;

	private boolean parallelPaneInitalized;

	/**
	 * @param appl
	 * @param model
	 */
	public ModelControlFrame_parallel(XuluModellingPlatform appl,
			XuluModel model) {
		super(appl, model);
		this.pack();
	}

	/**
	 * does basically the same as
	 * {@link ModelControlFrame_Basic#initControlContainer()}
	 */
	protected void initControlContainer() {
		spcEngine = new ParallelControlPanelEngine(appl);
		// ####### Initialize controlcomponent #######
		controlContainer = new ModelControlContainer_parallel(model, spcEngine,
				appl);

		// ******************** COPIED FROM SUPERCLASS
		// **************************

		// Bei Init-Aktion muessen zunaechst die Modell-Ressourcen mit den
		// ausgewaehlten
		// Datenpool-Objekten verknuepft werden; bevor das Modell initialisiert
		// wird
		controlContainer.addButtonActionListener(
				ModelControlContainer.BUTTON_INIT, new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						setModelResources();
					}
				});
		// Waehrend das Modell laeuft, soll nichts an den Ressourcen veraendet
		// werden
		controlContainer.addThreadListener(new WorkingThreadAdapter() {
			public void threadStarted(WorkingThread thread) {
				contentContainer.setEnabled(false);
				reloadButton.setEnabled(false);
			}

			public void threadStopped(WorkingThread thread) {
				contentContainer.setEnabled(true);
				reloadButton.setEnabled(true);
			}
		});
		contentPane.add(controlContainer, new GridBagConstraints(0, 0, 1, 1,
				1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(5, 5, 5, 5), 0, 0));
	}

	@Override
	protected void initContentPane() {
		super.initContentPane();
//		System.out.println(model.getClass().getName());
		if (model instanceof AbstractParallelStepModel) {
			initializeParallelPane();
		} else
			parallelPaneInitalized = false;
	}

	private boolean isParallelPaneInitalized() {
		return parallelPaneInitalized;
	}

	@Override
	protected ModelControlFrame_parallel newInstance(XuluModel newModel) {
		ModelControlFrame_parallel frame_parallel = new ModelControlFrame_parallel(
				appl, newModel);
		// initialize the parallel pane if it was initialized before
		if (this.isParallelPaneInitalized()
				&& !frame_parallel.isParallelPaneInitalized()) {
			if (newModel.getClass().getName()
					.equals(model.getClass().getName())) {
				frame_parallel.initializeParallelPane();
			}
		}
		return frame_parallel;
	}

	private void initializeParallelPane() {
		super.tabbedPane.addTab("Parallel Control", spcEngine.getPanel());
		// The boolean parallelPaneInitalized is required because after
		// reloading
		// with a custom classloader, like the preferred classloader, the
		// 'instanceof' will no longer work properly with the reloaded class.
		// (instanceof will return false)
		parallelPaneInitalized = true;
	}

}
