package appl.parallel.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import appl.ext.XuluConfig;
import appl.parallel.ComputingResource;
import appl.parallel.ComputingResourceContainer;
import appl.parallel.ComputingResourceProperties;
import appl.parallel.client.RemoteExecutionController;
import appl.parallel.client.ResourceChangeListener;
import appl.util.NonEditableTableModel;
import schmitzm.data.event.ObjectEvent;
import schmitzm.data.event.ObjectListener;

import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.ModelControlManager;
import edu.bonn.xulu.appl.XuluPlugin;
import edu.bonn.xulu.data.XuluDataException;
import edu.bonn.xulu.gui.ModelContentManagerContainer;
import edu.bonn.xulu.gui.ModelControlFrame;
import edu.bonn.xulu.gui.XuluGUIMessages;
import edu.bonn.xulu.model.ModelContentManager;
import edu.bonn.xulu.model.XuluModel;
import edu.bonn.xulu.plugin.gui.ModelControlContainer;
import edu.bonn.xulu.plugin.gui.ModelControlFrame_Basic;

/**
 * This class controls the {@link ParallelControlPanel}.
 *
 * @author Dominik Appl
 */
public class ParallelControlPanelEngine implements ActionListener,
		ListSelectionListener, TableModelListener, ResourceChangeListener {

	ParallelControlPanel panel;

	XuluModellingPlatform xulu;

	// get logger for this class
	protected final Logger LOG = LogManager
			.getLogger(this.getClass().getName());

	private DefaultTableModel resTableModel;

	private JTable resTable;

	private Vector<ComputingResourceContainer> resources;

	private RemoteExecutionController remoteExecutionController;

	/**
	 * @param xulu
	 *            the active modeling platform
	 */
	public ParallelControlPanelEngine(XuluModellingPlatform xulu) {
		this.xulu = xulu;
		this.panel = new ParallelControlPanel();
		init();
	}

	/**
	 * @param xulu
	 *            the active modeling platform
	 * @param panel
	 *            the panel to integrate
	 */
	public ParallelControlPanelEngine(XuluModellingPlatform xulu,
			ParallelControlPanel panel) {
		this.xulu = xulu;
		this.panel = panel;
		init();
	}

	/**
	 * The init method must be called after all Components of the GUI are
	 * initialized.
	 */
	void init() {
		remoteExecutionController = RemoteExecutionController
				.getRemoteExecutionController(xulu);
		if (remoteExecutionController != null)
			remoteExecutionController.addResourceChangeListener(this);
		resTable = panel.computingResourcesList;

		// add listeners
		resTable.getSelectionModel().addListSelectionListener(this);
		panel.refreshButton.addActionListener(this);
		panel.selectAllButton.addActionListener(this);

		resTableModel = new NonEditableTableModel(new Object[] { "Name",
				"Rating" }, new boolean[] { false, true }, 1);
		resTable.setModel(resTableModel);
		resTable.getModel().addTableModelListener(this);
		selectAvailableResources();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		// the source of the event.
		Object src = e.getSource();
		if (src == panel.refreshButton) {
			handleRefresh();
		} else if (src == panel.selectAllButton) {
			selectAvailableResources();
		}

	}

	/**
	 * handles the refresh button
	 */
	private void handleRefresh() {
		if (remoteExecutionController != null)
			remoteExecutionController.refreshResources();
		else
			JOptionPane
					.showMessageDialog(
							getPanel(),
							"Error: no discovery possible. The RemoteExecutionController-Plugin can " +
							"not be found. The plugin is required for parallel " +
							"computation","Error",JOptionPane.ERROR_MESSAGE);
	}

	private Vector<String> getVectorOfSelectedNames() {

		Object[] selectedComputingResources = getSelectedResourceContainers();

		// write selected names in vector
		Vector<String> names = new Vector<String>();
		for (int i = 0; i < selectedComputingResources.length; i++) {
			names
					.add(((ComputingResourceContainer) selectedComputingResources[i])
							.getInformation().getName());
		}
		return names;
	}

	private void setSelectedElements(Vector<String> names) {
		for (int i = 0; i < resTableModel.getRowCount(); i++) {
			ComputingResourceContainer container = (ComputingResourceContainer) resTableModel
					.getValueAt(i, 0);
			if (container == null)
				return;
			String name = ((ComputingResourceContainer) resTableModel
					.getValueAt(i, 0)).getInformation().getName();
			if (names.contains(name))
				resTable.getSelectionModel().addSelectionInterval(i, i);
		}
	}

	private void selectAvailableResources() {
		// write available selected names in vector
		Vector<String> names = new Vector<String>();
		for (int i = 0; i < resTableModel.getRowCount(); i++) {
			ComputingResourceContainer container = ((ComputingResourceContainer) resTableModel
					.getValueAt(i, 0));
			if (container == null)
				return;
			ComputingResourceProperties info = container.getInformation();
			if (info.isAvailable())
				names.add(info.getName());

		}
		setSelectedElements(names);
	}

	/**
	 * @return the first current selected ComputingResourceContainer from the
	 *          resouce list
	 */
	private ComputingResourceContainer getSelectedResourceContainer() {
		int selectedRow = resTable.getSelectedRow();
		if (selectedRow == -1)
			return null;
		return (ComputingResourceContainer) resTableModel.getValueAt(
				selectedRow, 0);
	}

	/**
	 * @return all current selected ComputingResourceContainer from the resouce
	 *         list
	 */
	ComputingResourceContainer[] getSelectedResourceContainers() {
		int[] selRows = panel.computingResourcesList.getSelectedRows();
		Object[] selectedValues = new Object[selRows.length];
		for (int i = 0; i < selRows.length; i++) {
			selectedValues[i] = resTableModel.getValueAt(selRows[i], 0);
			if (selectedValues[i] == null)
				return new ComputingResourceContainer[0];
		}
		ComputingResourceContainer[] selectedContainers = new ComputingResourceContainer[selectedValues.length];
		for (int i = 0; i < selectedValues.length; i++)
			selectedContainers[i] = (ComputingResourceContainer) selectedValues[i];
		return selectedContainers;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
	 */
	public void valueChanged(ListSelectionEvent e) {
		Object src = e.getSource();
		if (src == resTable.getSelectionModel()) {
			ComputingResourceContainer selValue = getSelectedResourceContainer();
			if (selValue == null)
				return;
			panel.propertyTable.setPropertyData(selValue.getInformation()
					.getProperties());
		}
	}

	/**
	 * Listens for user changes of the rating
	 *
	 * @see javax.swing.event.TableModelListener#tableChanged(javax.swing.event.TableModelEvent)
	 */
	public void tableChanged(TableModelEvent e) {
		// enter the new rating information to the configuration
		XuluConfig config = XuluConfig.getXuluConfig();
		if (e.getType() == TableModelEvent.UPDATE) {
			int changedRow = e.getFirstRow();
			// only the 2nd col is editable
			String newRating = resTableModel.getValueAt(changedRow, 1)
					.toString();
			try {
				int rating = Integer.valueOf(newRating);
				if (rating < 0) {
					resTableModel.setValueAt(0, changedRow, 1);
					throw new UnsupportedOperationException(
							"Rating must be > 0");
				}

			} catch (Exception f) {
				resTableModel.setValueAt(0, changedRow, 1);
				throw new UnsupportedOperationException(
						"Rating must be an integer!");
			}
			String computerName = resTableModel.getValueAt(changedRow, 0)
					.toString();
			String prefix = "PerformanceRating";
			String key = prefix + "." + computerName;
			config.setProperty(key, newRating);
		}
	}

	/**
	 * @return the panel
	 */
	public ParallelControlPanel getPanel() {
		return this.panel;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see appl.parallel.client.ResourceChangeListener#updateResources(java.util.Vector)
	 */
	public void updateResources(
			Vector<ComputingResourceContainer> currentResources) {
		// get the selected values (and restore the selection after list
		// initializing)
		Vector names = getVectorOfSelectedNames();

		// clear list
		resTableModel.setRowCount(0);
		for (ComputingResourceContainer res : currentResources) {
			// init the rating from the configuration
			XuluConfig config = XuluConfig.getXuluConfig();
			String computerName = res.getInformation().getName();
			String prefix = "PerformanceRating";
			String key = prefix + "." + computerName;
			String rating = config.getProperty(key, false);
			// if no rating found use the selfrating of the machine
			if (rating == null || rating.equals("0"))
				rating = res.getInformation().getProperty("Rating");
			if (rating == null)
				rating = "0"; // '0' means average
			resTableModel.addRow(new Object[] { res, rating });
		}

		// restore selection
		this.setSelectedElements(names);
	}
}
