package appl.parallel.model;

import appl.parallel.spmd.AdvancedSPMDClientController;
import appl.parallel.spmd.AdvancedSPMDClientInterface;
import appl.parallel.spmd.SPMDClientInterface;
import edu.bonn.xulu.model.AbstractStepModel;
import edu.bonn.xulu.model.ModelContentManager;
import edu.bonn.xulu.model.event.ModelStepFinishedEvent;
import edu.bonn.xulu.model.event.ModelStepStartedEvent;

/**
 * This class is used instead of the {@link AbstractStepModel} for implementation
 * of parallel algorithms. It provides access to the {@link SPMDClientInterface}.
 * 
 * @author Dominik Appl
 */
public abstract class AbstractParallelStepModel extends AbstractStepModel
		implements ParallelStepModel {

	SPMDClientInterface SPMDController;

	/* (non-Javadoc)
	 * @see appl.parallel.model.ParallelStepModel#getSPMDController()
	 */
	public SPMDClientInterface getSPMDController() {
		if (SPMDController == null) {
			throw new UnsupportedOperationException(
					"SPMDController is null!"
							+ "You cannot use the SPMDController before the performInit-method");
		}
		return SPMDController;
	}

	/**
	 * @return the Advanced SPMD controller which provides access to advanced parallel functionality
	 * like preloading or multithreading
	 */
	public AdvancedSPMDClientInterface getAdvancedSPMDController() {
		return (AdvancedSPMDClientController) getSPMDController();
	}

	/**
	 * A new instance. See {@link AbstractStepModel#AbstractStepModel(ModelContentManager)} for details
	 * 
	 */
	public AbstractParallelStepModel(ModelContentManager contentManager) {
		super(contentManager);
	}

	/* (non-Javadoc)
	 * @see appl.parallel.model.ParallelStepModel#setSPMDController(appl.parallel.spmd.SPMDClientController)
	 */
	public void setSPMDController(SPMDClientInterface controller) {
		this.SPMDController = controller;
	}

}
