package appl.parallel.server;

import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Vector;

import appl.parallel.ComputingResource;
import appl.parallel.spmd.SPMDClientController;
import appl.parallel.spmd.SPMDTask;
import appl.parallel.spmd.split.SinglePartitionInfo;
import appl.parallel.spmd.split.SplitMap;

/**
 * The resources participating in a SPMD computation need to implement this
 * interface.
 * 
 * @author Dominik Appl
 */
public interface SPMDResource extends ComputingResource {

	/**
	 * Executes the given Task.
	 * 
	 * @param SPMDTaskName
	 *            name of the taskclass to be executed
	 * @param referenceID
	 *            the ID of the Grid which should be the reference resource
	 * @param parameters
	 *            parameters for the given Task
	 * @return an object array containing the results for possibly multiple
	 *         executions (when using multithreading with multi-core cpus)
	 * @throws RemoteException
	 *             if the connection fails
	 * @see SPMDClientController#setReferenceResource(Object)
	 */

	public Object[] runSPMDModelTask(String SPMDTaskName, int referenceID,
			Object... parameters) throws RemoteException;

	/**
	 * @param IPs
	 *            the IP addresses of all other participating servers. They MUST
	 *            have the same index as their according partition in the
	 *            {@link SplitMap} has.
	 * @return creates a new {@link PartitionDataServer} on this resource and
	 *         returns it
	 */
	public PartitionDataServer createDataServer(String[] IPs)
			throws RemoteException;

}
