package appl.parallel.server;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;
import java.net.SocketException;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import appl.ext.XuluConfig;
import appl.parallel.services.MulticastDiscoveryService;

/**
 * Waits for multicast messages from clients and responds properly.
 * 
 * @see MulticastDiscoveryService
 * @author Dominik Appl
 */
public class ServerMulticastReceiver extends Thread {

	private final Logger LOG = LogManager.getLogger(this.getClass().getName());

	private static final String helloFromXuluMessage = "Hello Servers";

	private static final String helloFromServerMessage = "Hello Xulu from XuluServer";

	boolean exit = false;

	MulticastSocket socket;

	int multicastPort = XuluConfig.getXuluConfig().getIntProperty(
			"XuluServer.multicastport");

	/**
	 * @param socket
	 *            Socket which is used for listening. Set a timeout for the
	 *            socket via {@link MulticastSocket#setSoTimeout(int)} so that
	 *            the thread can be stopped (because of the blocked reading)
	 */
	public ServerMulticastReceiver(MulticastSocket socket) {
		this.socket = socket;
		// try {
		// socket.setSoTimeout(5000);
		// } catch (SocketException e) {
		// //do not printout the timeout
		// }
	}

	/**
	 * Safe method to stop the Thread. Use this method instead of {@link Thread}
	 */
	public void stopThread() {
		exit = true;
	}

	public void run() {
		System.out.println("Listening on port " + socket.getLocalPort()
				+ " for Xulu hello messages!");
		while (!exit) {
			try {
				// receive buffer
				byte[] buffer = new byte[256];
				DatagramPacket datagram = new DatagramPacket(buffer,
						buffer.length);
				socket.receive(datagram);

				// get right message format (cut out null bytes)
				byte[] result = new byte[datagram.getLength()];
				System.arraycopy(datagram.getData(), 0, result, 0, datagram
						.getLength());

				// check message:
				if (new String(result).equals(helloFromXuluMessage)) {
					// LOG.info("received hello from Client " +
					// datagram.getAddress());
					LOG.info("received hello from Client "
							+ datagram.getAddress());

					// get the IP again and send the answer
					String IP = XuluConfig.getXuluConfig().getProperty(
							"XuluServer.multicastgroup");
					DatagramPacket answer = new DatagramPacket(
							helloFromServerMessage.getBytes(),
							helloFromServerMessage.length(), InetAddress
									.getByName(IP), multicastPort);
					socket.send(answer);
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (Exception e) {
				if (!(e instanceof InterruptedException))
					e.printStackTrace();
			}
		}
	}
}
