package appl.parallel.server;

import java.io.Serializable;
import java.lang.management.ManagementFactory;
import java.lang.management.MemoryPoolMXBean;
import java.lang.management.MemoryUsage;
import java.lang.management.ThreadMXBean;
import java.util.List;
import java.util.Timer;

import org.apache.log4j.lf5.util.Resource;

import appl.parallel.ComputingResourceProperties;
import appl.parallel.SimpleResourceProperties;

/**
 * Constructs information out of a XuluServer Object. Unfortunately there is no way
 * to measure CPU load (for this you would need to have access to the OS core). This could
 * be done with the use of the JNI, but that is low level programming and would be OS
 * dependent.
 *  
 * Must be invoked on server side (which should be clear, anyway!).<br>
 * 
 * Sets the following properties:
 * <br><br>
 * Name <br>
 * isAvailable<br>
 * Uptime<br><br>
 * 
 * and of course the Properties the superclass sets<br>
 * @author Dominik Appl
 */
public class XuluServerProperties extends SimpleResourceProperties implements
		Serializable {

	public XuluServerProperties(XuluServer server, String name) {
		super();
		if (name != null)
			properties.setProperty("Name", name);
		properties.setProperty("isAvailable", (!(server.isClientConnected()))
				+ "");

		//format uptime
		int uptimeInSeconds = server.getUptime() / 1000;
		int hours, minutes, seconds;
		hours = uptimeInSeconds / 3600;
		uptimeInSeconds = uptimeInSeconds - (hours * 3600);
		minutes = uptimeInSeconds / 60;
		uptimeInSeconds = uptimeInSeconds - (minutes * 60);
		seconds = uptimeInSeconds;
		String formatedTime = hours + "h " + minutes + "m " + seconds + "s";
		properties.setProperty("Uptime", formatedTime);
		properties.setProperty("Rating", server.getRating() + "");
		properties.setProperty("Port", server.getRegistryPort() + "");
	}

}