package appl.parallel.spmd.split;

/**
 * A {@link PartitionInfo} object contains information about the partition, which is
 * normally transfered a server. It contains a {@link SplitMap} for the
 * partition, the position inside the {@link SplitMap}, the name of the base
 * resource (the mother resource) and a method which supports cloning the
 * information.
 * 
 * @see SinglePartitionInfo
 * 
 * @author Dominik Appl
 */
public interface PartitionInfo extends Cloneable {

	/**
	 * @return the splitMap for this partition
	 */
	public abstract SplitMap getSplitMap();

	/**
	 * @return the splitMapPos position inside the {@link SplitMap}
	 */
	public abstract int getSplitMapPos();

	/**
	 * @return the baseResourceName the name of the base resource (mother resource). The name
	 * is string which is the user provides for identification an retrieval on server side. 
	 */
	public abstract String getBaseResourceName();

	/**
	 * This method can especially be used by multi data methods to create a new instance of this 
	 * data type
	 * 
	 * @param rootID the id of the base resource for the NEW copy of the {@link PartitionInfo}
	 * 
	 * @return the new {@link PartitionInfo}
	 */
	public abstract PartitionInfo clone(int rootID);

}