package appl.parallel.spmd.split;

import java.awt.Rectangle;

import org.w3c.dom.css.Rect;

import appl.data.DataLoader;

/**
 * A resource that is splittable must provide some information to work with {@link SplitMap}s.
 * Important is the with and the height of the resource to be splitted.
 *
 *
 * @author Dominik Appl
 */
public interface SplittableResource extends DataPartition {
	/**
	 * Should return a unique ID for this resource. Standard implementation
	 * would be returning the {@link Object#hashCode() hashcode} of the object.
	 *
	 * @return a ID which is unique for this client
	 *
	 */
	public int getRootID();

	/**
	 * Returns the loader which is responible for loading the data. This loader should be
	 * independet from the current system. This means that, e.g. for a loader which
	 * loads a grid from filesystem, that all paths should be relative (and not
	 * absolute), so that a possibly remote resource can load the data from its local
	 * filesystem, if the data is stored in the same position relative to the home
	 * directory.
	 *
	 * @return a loader for this resource.
	 */
	public DataLoader getLocalLoader();

	/**
	 * @return the splittable is splitted according to its splitHeight (example: the height of a raster)
	 * */
	public int getSplitHeight();

	/**
	 * @return the splittable is splitted according to its splitWidth  (example: the width of a raster or the length of an array)
	 * */
	public int getSplitWidth();
}
