package appl.parallel.starter;

import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * An interface for programs which start themself other programs as separate
 * processes. It is possible to start/stop and restart the encapsulated process.
 * 
 * @author Dominik Appl
 */

public interface Starter extends Remote {
	/**
	 * Starts the process
	 * 
	 * @throws RemoteException
	 */
	public void start() throws RemoteException;

	/**
	 * stops the process
	 * 
	 * @throws RemoteException
	 */
	public void stop() throws RemoteException;

	/**
	 * restarts the process
	 * 
	 * @throws RemoteException
	 */
	public void restart() throws RemoteException;

	/**
	 * @return true, if the process is running
	 * @throws RemoteException
	 */
	public boolean isRunning() throws RemoteException;
}
