package appl.parallel.starter.client;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.rmi.RemoteException;
import java.util.Vector;

import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JList;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import appl.parallel.client.RemoteExecutionController;
import appl.parallel.services.HostnameDiscoveryService;
import appl.parallel.starter.Starter;
import appl.parallel.starter.server.XuluServerStarter;

/**
 * This is the Controller for the {@link XuluStarterControllerFrame}, which is
 * the user interface for the {@link XuluServerStarter}. All events are handled
 * here. <br>
 * 
 * The controller updates a list of {@link Starter}s it finds in the network.
 * At the moment is uses the {@link HostnameDiscoveryService} to discover
 * starters.
 * 
 * @see HostnameDiscoveryService#getStarterContainers()
 * @author Dominik Appl
 */
public class XuluStarterController implements ActionListener {

	Logger LOG = LogManager.getLogger(this.getClass().getName());

	private XuluStarterClientPanel GUI;

	private HostnameDiscoveryService discovery;

	private Vector<StarterContainer> currentData;

	/**
	 * 
	 */
	public XuluStarterController() {
		GUI = new XuluStarterClientPanel();
		// start simpleDiscovery service
		discovery = new HostnameDiscoveryService();
		discovery.startService();
		updateList();
		initGUI();
	}

	private void initGUI() {
		GUI.startButton.addActionListener(this);
		GUI.stopButton.addActionListener(this);
		GUI.refreshButton.addActionListener(this);
		GUI.restartButton.addActionListener(this);
	}

	public XuluStarterClientPanel getPanel() {
		return GUI;
	}

	/**
	 * 
	 */
	private void updateList() {
		currentData = discovery.getStarterContainers();
		GUI.serverList.setCellRenderer(new MyCellRenderer(currentData));
		// DefaultListModel serverListModel = new DefaultListModel();
		// GUI.serverList.setModel(serverListModel);
		GUI.serverList.setListData(currentData);
	}

	private Starter getSelectedStarter() {
		Object value = GUI.serverList.getSelectedValue();
		if (value != null) {
			return ((StarterContainer) value).getStarter();
		}
		return null;
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == GUI.refreshButton) {
			updateList();
		}
		Starter selectedStarter = getSelectedStarter();
		// if nothing selected: return
		if (selectedStarter == null)
			return;
		try {
			if (e.getSource() == GUI.startButton) {
				selectedStarter.start();
				updateList();
			}
			if (e.getSource() == GUI.stopButton) {
				selectedStarter.stop();
				updateList();
			}
			if (e.getSource() == GUI.restartButton) {
				selectedStarter.restart();
				updateList();
			}
		} catch (RemoteException e1) {
			LOG
					.error(
							"Could not access selected Starter. Remote Exeception occured.",
							e1);
		}
	}

	class MyCellRenderer extends DefaultListCellRenderer {
		private final Vector<StarterContainer> containers;

		public MyCellRenderer(Vector<StarterContainer> containers) {
			this.containers = containers;
			// TODO Auto-generated constructor stub
		}

		@Override
		public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			// TODO Auto-generated method stub
			Component listCellRendererComponent = super
					.getListCellRendererComponent(list, value, index,
							isSelected, cellHasFocus);
			if (!isSelected) {
				if (containers.get(index).isServerRunning())
					listCellRendererComponent.setBackground(Color.green);
				else
					listCellRendererComponent.setBackground(Color.red);
			}
			return listCellRendererComponent;
		}

	}
}
