package appl.parallel.test;

import java.awt.Rectangle;
import java.io.File;
import java.io.FileInputStream;
import java.io.Serializable;
import java.util.Random;
import java.util.Vector;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Layout;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import edu.bonn.xulu.io.ImportFactory;
import edu.bonn.xulu.plugin.io.grid.array.WritableGridArrayFactory;

import schmitzm.data.WritableGrid;
import schmitzm.data.WritableGridArray;
import schmitzm.data.WritableGridRaster;
import schmitzm.geotools.io.GeoImportUtil;

import appl.parallel.ComputingResourceContainer;
import appl.parallel.client.RemoteEventHandler;
import appl.parallel.client.ClientDataServer;
import appl.parallel.data.splittable.SplittableLLProxyGrid;
import appl.parallel.services.GlobalDiscoveryService;
import appl.parallel.spmd.AbstractSPMDTask;
import appl.parallel.spmd.SPMDClientController;
import appl.parallel.spmd.SPMDTask;
import appl.parallel.spmd.SPMDServerController;
import appl.parallel.spmd.split.SplittableGrid;
import appl.util.RasterMetaData;
import appl.util.RasterUtil;

/**
 * Generated code for the test suite <b>SPMDTest</b> located at
 * <i>/XuluSVN/javasrc/appl/parallel/test/SPMDTest.testsuite</i>. This class
 * tests all the SPMD functionality, using only XULU/V and nearly nothing of
 * XULU
 */
public class SPMDTest extends TestCase implements Serializable {

	transient private final Logger LOG = LogManager.getLogger(this.getClass()
			.getName());

	transient private ClientDataServer client;
	transient private Vector<ComputingResourceContainer> computingResources;
	transient private SPMDClientController clientController;

	transient SplittableGrid gridInput;
	transient SplittableGrid remoteInput;
	transient SplittableGrid localOutput;
	transient SplittableGrid remoteOutput;

	/**
	 * Constructor for SPMDTest.
	 * 
	 * @param name
	 */
	public SPMDTest(String name) {
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>SPMDTest</b>
	 * definition.
	 */
	public static Test suite() {
		TestSuite sPMDTest = new TestSuite("SPMDTest");
//		sPMDTest.setArbiter(DefaultTestArbiter.INSTANCE).setId(
//				"E873D66A70977ED2A98AAC10B54511DB");
//
//		sPMDTest.addTest(new SPMDTest("testSPMD").setId(
//				"E873D66A70977ED2E5E8ED20B54511DB").setTestInvocationId(
//				"E873D66A70977ED2EF7E3C00B54511DB"));
		return sPMDTest;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		PropertyConfigurator.configure("xululog4j.cfg");
		computingResources = new GlobalDiscoveryService()
				.getRemoteResources();

		for (ComputingResourceContainer computingRes : computingResources) {
			System.out.println("DiscoveredRessource " + computingRes);
		}
		LOG.info("Discovery finished...");
		RemoteEventHandler handler = new RemoteEventHandler();
		client = new ClientDataServer(handler);
		clientController = new SPMDClientController(computingResources,null, client,handler);
		// ////////// Initialise data \\\\\\\\\\\\\
		WritableGridRaster baseGrid = GeoImportUtil
				.readGridRasterFromArcInfoASCII(new File(
						"../Xulu-Data/minigrid2.arc")); // small data
//		 "../Xulu-Data/large_clue_data/sc1gr0.0")); //large data
		gridInput = new SplittableLLProxyGrid(new WritableGridArrayFactory(),
				new RasterMetaData(baseGrid));
		localOutput = new SplittableLLProxyGrid(new WritableGridArrayFactory(),
				new RasterMetaData(baseGrid));
		remoteInput = new SplittableLLProxyGrid(new WritableGridArrayFactory(),
				new RasterMetaData(baseGrid));
		remoteOutput = new SplittableLLProxyGrid(
				new WritableGridArrayFactory(), new RasterMetaData(baseGrid));
		RasterUtil.copyInto(baseGrid, gridInput);
		RasterUtil.copyInto(baseGrid, remoteInput);
		//RasterUtil.copyInto(baseGrid, localOutput);
		//RasterUtil.copyInto(baseGrid, remoteOutput);
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		client.close();
		clientController.close();
		// GlobalDiscoveryService.getInstance().stopServices();
	}

	/**
	 * TestSPMD
	 * 
	 * @throws Exception
	 */
	public void testSPMD() throws Exception {
		int steps = 3;
		int neighborhoodRange = 6;
		System.out.println("Starting step 1: ");
				
		/***********************************************************************
		 * START LOCAL ALGORITHM ******************** The simple Algorithm will
		 * calculate the average of the neighborhood und write the output to the
		 * coresponing cell in the output cell
		 **********************************************************************/

		Rectangle partition = new Rectangle(0, 0, gridInput.getWidth(),
				gridInput.getHeight());
		if (gridInput.getWidth() < 20)
			RasterUtil.printGrid(gridInput, 5, 1, "Input");
		
		for (int step = 0; step < steps; step++) {
//			 the overallsum is simply the sum over all elements
			float overallSum = 0f;
			long localStartTime = System.currentTimeMillis();
			for (int y = 0; y < gridInput.getHeight(); y++)
				for (int x = 0; x < gridInput.getWidth(); x++) {
					float tmp = gridInput.getRasterSampleAsFloat(x, y);
					if (!Float.isNaN(tmp))
						overallSum += tmp;
					// the local sum is simply the sum over all elements in the
					// neighborhood
					float localSum = 0;
					// number of cells over which the sum is calculated
					int noOfCells = 0;
					// for each cell: calculate the sum of all neighbors
					for (int y2 = y - neighborhoodRange; y2 <= y
							+ neighborhoodRange; y2++)
						for (int x2 = x - neighborhoodRange; x2 <= x
								+ neighborhoodRange; x2++)
							// check if the coordinates are valid (inside the
							// grid and not NaN)
							if (partition.contains(x2, y2))
								if (!Float.isNaN(gridInput
										.getRasterSampleAsFloat(x2, y2))) {
									localSum += gridInput
											.getRasterSampleAsFloat(x2, y2);
									noOfCells++;
								}

					localOutput.setRasterSample(localSum / (float) (noOfCells),
							x, y);

				}

			System.out.print("Finished step " + (step+1) + " in "
					+ ((System.currentTimeMillis() - localStartTime)) + " ms");
			localStartTime = System.currentTimeMillis();
			System.out.println(" with sum (local calculated): " + overallSum);
			//copy values into results from output to input (for next step)
			RasterUtil.copyInto(localOutput, gridInput);
		 }
		 if(gridInput.getWidth()<20)
		 RasterUtil.printGrid(localOutput, 5, 1, "Local Output");

		/***********************************************************************
		 * START REMOTE ALGORITHM
		 *  ******************** All the same..now remotely********************/
		 //set back the grid to the original input
		 
		 System.out
				.println("******************* starting remote execution *****************");
		long remoteStartTime = System.currentTimeMillis();
		clientController.setNeighborhoodRange(neighborhoodRange);
		clientController.addToSplitControl(remoteInput, "inputGrid");
		clientController.addToSplitControl(remoteOutput, "outputGrid");
		System.out.println("After "
				+ ((System.currentTimeMillis() - remoteStartTime))
				+ "ms: finished with adding to splitcontroll");
		
		//********************** START STEP CONTROL *********************
		for (int step = 0; step < steps; step++) {
			if(step > 0){
				long updateStartTime = System.currentTimeMillis();
				clientController.updateNeighborhood(remoteInput);
				System.out.println("Update took:" + (System.currentTimeMillis()-updateStartTime));
			}
			try {
				Object[] sum =clientController.runSPMDModelTask(
						new AverageNeighborhoodTestTask(), neighborhoodRange);
//				add results
				float overallSum = 0;
				for (int i = 0; i < sum.length; i++)
					overallSum += (Float) sum[i];
				System.out.println("finished step "
						+ (step+1) + " in "
						+ ((System.currentTimeMillis() - remoteStartTime))
						+ "ms " + " with sum "+
						+ overallSum);
				remoteStartTime = System.currentTimeMillis();
			} catch (Throwable e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		//***************** END STEP CONTROL *****************************
		
		//After execution:
		// merge partitions and check if the local and remotely calculated
		// versions are equal
		remoteStartTime = System.currentTimeMillis();
		clientController.mergePartition(remoteOutput);
		System.out.println("Merging took another "
				+ ((System.currentTimeMillis() - remoteStartTime))
				+ "ms");
		// Thread.currentThread().sleep(10000);
		 if(remoteInput.getWidth()<20)
			 RasterUtil.printGrid(remoteOutput, 5, 1, "Remote Output");
		assertTrue(RasterUtil.checkEqual(remoteOutput, localOutput, true));
	}
}
