package appl.parallel.test;

import java.awt.Rectangle;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import edu.bonn.xulu.plugin.io.grid.array.WritableGridArrayFactory_ArcInfoAsciiGrid;

import schmitzm.data.WritableGrid;
import schmitzm.data.WritableGridArray;
import schmitzm.data.WritableGridRaster;
import schmitzm.geotools.io.GeoExportUtil;
import schmitzm.geotools.io.GeoImportUtil;
import appl.parallel.data.xulugridfile.XuluGridFile;
import appl.parallel.spmd.split.WritableGridPartition;
import appl.util.RasterMetaData;
import appl.util.RasterUtil;

/**
 * Generated code for the test suite <b>XuluGridTestCase</b> located at
 * <i>/Xulu-latest/javasrc/appl/test/XuluGridTestCase.testsuite</i>.
 *
 * Extensivly tests the XuluGridFile for errors
 */
public class XuluGridTestCase extends TestCase {
	 XuluGridFile gridFile;
	
	/**
	 * Constructor for XuluGridTestCase.
	 * @param name
	 */
	public XuluGridTestCase(String name) {
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>XuluGridTestCase</b>
	 * definition.
	 */
	public static Test suite() {
		TestSuite xuluGridTestCase = new TestSuite("XuluGridTestCase");
//		xuluGridTestCase.setArbiter(DefaultTestArbiter.INSTANCE).setId(
//				"D14CC6AE1994BD92F864B0C0A54011DB");
//	
//		xuluGridTestCase.addTest(new XuluGridTestCase("testReadingPartitialGrid")
//				.setId("D14CC6AE1994BD92132B3E60A54111DB").setTestInvocationId(
//						"D14CC6AE1994BD9269E4C820A54111DB"));
//	
//		xuluGridTestCase.addTest(new XuluGridTestCase("testMartin").setId(
//				"D14CC6AE1994BD92FC1EA54AA62111DB").setTestInvocationId(
//				"D14CC6AE1994BD9262B92000A62211DB"));
		return xuluGridTestCase;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		if (gridFile!=null) 
			gridFile.close();	
	}

    /**
	* testReadingPartitial Grid
	* @throws Exception
	*/
	public void testReadingPartitialGrid()
	throws Exception
	{
		
		//base file: all assertions are checked against this grid
		WritableGridRaster baseGrid = GeoImportUtil.readGridRasterFromArcInfoASCII(new File("../Xulu-Data/minigrid.arc"));
		printSmall(baseGrid);
		GeoExportUtil.writeGridRasterToArcInfoASCII(baseGrid, new File("BaseGridExport_miniGrid_export1"));
        
		// write a XuluGridFile / check it against baseGrid 
        File file = new File("XuluGridFile_miniGrid.xgrid");
        XuluGridFile.writeToXuluGridFile(baseGrid,file);
        // fill the grid2 with any Data
        WritableGrid grid2 = GeoImportUtil.readGridRasterFromArcInfoASCII(new File("../Xulu-Data/SimpleGrid.arc"));
        assertFalse(RasterUtil.checkEqual(baseGrid,grid2,true));
        XuluGridFile.readIntoGridFromXuluGridFile(grid2, file);
        assertTrue(RasterUtil.checkEqual(baseGrid,grid2,true));
        
        //export und reimport an check it again (this caused some hard to trace errors in the past, even if it should not.)
        GeoExportUtil.writeGridRasterToArcInfoASCII((WritableGridRaster)grid2, new File("BaseGridExport_miniGrid_export2"));
        grid2 = GeoImportUtil.readGridRasterFromArcInfoASCII(new File("BaseGridExport_miniGrid_export2"));
        assertTrue(RasterUtil.checkEqual(baseGrid,grid2,true));
        
        //initialize a new XuluGrid 
        gridFile = new XuluGridFile(new File("XuluGridFile_miniGrid.xgrid"),"r");
        RasterMetaData meta = gridFile.getMetaData();
        //get the whole gridFile using the partition method and check it
        WritableGrid wholeGrid = gridFile.getPartitialGrid2D(new Rectangle(0, 0, meta.getWidth(), meta.getHeight()));
        assertTrue(RasterUtil.checkEqual(baseGrid,wholeGrid,true));
        assertTrue(RasterUtil.checkEqual(grid2,wholeGrid,true));
        //write all data from wholeGrid into grid2, then export grid2
        for (int y = meta.getMinY(); y < meta.getHeight()- meta.getMinY(); y++)
        		for (int x = meta.getMinX(); x < meta.getWidth() - meta.getMinY(); x++)
        			grid2.setRasterSample(wholeGrid.getRasterSample(x,y),x,y);
        GeoExportUtil.writeGridRasterToArcInfoASCII(baseGrid, new File("XuluGrid2FromWholeGrid_test"));
        	        
        
        //new WritableGridArrayFactory_ArcInfoAsciiGrid().exportObject(wholeGrid, new BufferedOutputStream(new FileOutputStream("XuluGridTest_ARCINFO2")));
        
//        for (y = 1; y < metadata.getheight(); y++)
//            for (x = 1; x < metadata.getwidth(); x++) {
        //get left half of the gridFile
        WritableGrid leftHalf = gridFile.getPartitialGrid2D(new Rectangle(0, 0, (meta.getWidth()/2), meta.getHeight()));
        new WritableGridArrayFactory_ArcInfoAsciiGrid().exportObject(leftHalf, new BufferedOutputStream(new FileOutputStream("LeftHalf_XULUGRID")));
        
        //########################### no gridfile test ###############################
        XuluGridFile gridfile = new XuluGridFile(new File("XuluGridFile_miniGrid.xgrid"),"rw");
        WritableGridPartition partition = gridfile.getPartitialGrid2D(new Rectangle(0,0,5,5));
        printSmall(partition);
         partition = gridfile.getPartitialGrid2D(new Rectangle(0,0,4,4));
        printSmall(partition);
         partition = gridfile.getPartitialGrid2D(new Rectangle(1,1,8,8));
        printSmall(partition);
         partition = gridfile.getPartitialGrid2D(new Rectangle(5,5,5,5));
        printSmall(partition);
         partition = gridfile.getPartitialGrid2D(new Rectangle(1,1,2,2));
        printSmall(partition);
        
}
	/**
	 * prints only small grids
	 * 
	 * @param grid
	 */
	public void printSmall(WritableGrid grid){
		if(grid.getWidth()<20 && grid.getHeight()<20)
			RasterUtil.printGrid(grid,5,1,"output");
	}

public void testMartin()
	throws Exception
	{
		WritableGridRaster baseGrid = GeoImportUtil.readGridRasterFromArcInfoASCII(new File("../Xulu-Data/standard_clue_data/CLUE-Datensatz1/sc1gr0.23"));
		WritableGridArray testGrid = new WritableGridArray.Float(baseGrid.getMinX(),baseGrid.getMinY(),baseGrid.getWidth(),baseGrid.getHeight(),baseGrid.getX(),baseGrid.getY(),baseGrid.getRealWidth(),baseGrid.getRealHeight(),null,null);
		for (int y = baseGrid.getMinY(); y < baseGrid.getHeight()- baseGrid.getMinY(); y++)
    		for (int x = baseGrid.getMinX(); x < baseGrid.getWidth() - baseGrid.getMinY(); x++)
    			testGrid.setRasterSample(baseGrid.getRasterSample(x,y),x,y);
       GeoExportUtil.writeGridRasterToArcInfoASCII(baseGrid, new File("Test_baseGrid"));
       new WritableGridArrayFactory_ArcInfoAsciiGrid().exportObject(testGrid,new FileOutputStream("Test_testGrid2"));
	}
}
