package appl.util;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.URL;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import net.jini.loader.pref.PreferredClassLoader;

/**
 * General utility class. See method description for details.
 * 
 * @author Dominik Appl
 */
public class GeneralUtil {

	public static Logger LOG = LogManager.getLogger("appl.util.GeneralUtil");

	/**
	 * Loads a class from the specified URL and returns an instance of this class.
	 * @param location the URL of the rootDirectory/networkLocation
	 * @param classname the fully qualified name of the class (the class must have a 
	 * 
	 * @return the instance of the class or <code>null</code> if loading fails
	 */
	public static Object loadClassFromURL(URL location, String classname) {
		try {
			URL[] locations = { location };
			PreferredClassLoader classLoader = new PreferredClassLoader(
					locations, Thread.currentThread().getContextClassLoader(),
					null, false);
			LOG.debug("Try to load " + classname.getClass().getName()
					+ " from " + location);
			Class classObject = classLoader.loadClass(classname);
			return classObject.newInstance();
		} catch (Exception e) {
			LOG.error("Could not load class for " + classname + " from "
					+ location, e);
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Serializes the given object into the specified file. All exceptions are printed to console.
	 * @param o the object
	 * @param destFile the file to write to
	 */
	public static void SerializeToFile(Object o, File destFile) {
		try {
			destFile.createNewFile();
			BufferedOutputStream fos = new BufferedOutputStream(
					new FileOutputStream(destFile));
			ObjectOutputStream oos = new ObjectOutputStream(fos);
			oos.writeObject(o);
			oos.close();
			fos.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Reads the first object out of the file
	 * @param loadFile the file to read from
	 * @return the object
	 */
	public static Object readSerializedFrom(File loadFile) {
		Object loadedObject = null;
		try {
			BufferedInputStream fis = new BufferedInputStream(
					new FileInputStream(loadFile));
			ObjectInputStream ois = new ObjectInputStream(fis);
			loadedObject = ois.readObject();
			ois.close();
			fis.close();
			// remove File from Filesystem
		} catch (FileNotFoundException e) {
			LOG.error(e);
			e.printStackTrace();

		} catch (Exception e) {
			LOG.error(e);
			e.printStackTrace();
		}
		return loadedObject;
	}
}
