package de.skrueger.xulu.plugin.gnur;

import org.rosuda.JRI.REXP;
import org.rosuda.JRI.Rengine;

import schmitzm.data.property.ScalarProperty;
import schmitzm.data.property.ValueProperty;

/**
 * Link between datapool and R-Engine 
 * @author stefan
 */

public class RVar {
	/**
	 * Soll diese Datenstruktur in die R Engine importiert und-/oder aus der 
	 * Engine heraus exportiert werden?  
	 */
	private Boolean exp = true;
	private Boolean imp = true;

	private ValueProperty prop = null;
	private String RVarName = null;

	public RVar ( ValueProperty prop) throws PropertytypeNotConvertableToRVar {
		if (prop instanceof ScalarProperty) {
			this.prop = prop;
			RVarName = prop.getName().replaceAll(" ", "_");
		} else {
			throw new PropertytypeNotConvertableToRVar( prop );
		}
	}
	
	public Boolean isExport() {
		return exp;
	}
	
	public Boolean isImport() {
		return imp;
	}

	
	/**
	 * @return the name of the variabl ein the R environment
	 */
	public String getRName() {
		return RVarName;
	}
	

	/**
	 * Set the name that will reference the R-variable inside the RVar.
	 * TODO: Check for doubles, name validity
	 * @param rname
	 */
	public void setRName(String rname) {
		RVarName = rname.replace(" ", "_");
	}

	/**
	 * @return the name of the object in the datapool
	 */
	public String getName() {
		return prop.getName();
	}

	public boolean toR(Rengine re) {
		REXP x;
		
		if (!isImport()) return false;
		
		if (prop instanceof ScalarProperty) {
			Integer val = prop.getOneTimeReadAccess().getValueAsInt();
			String expr = getRName() + " <- " + val;
			re.eval(expr);
			return true;
		} else {
			System.out.println("Can't import this var!"+getName());
			return false;
		}
	}
	

	public boolean fromR(Rengine re) {
		REXP x;
		
		if (!isExport()) return false;
		
		if (prop instanceof ScalarProperty) {
			String expr = getRName();
			REXP eval = re.eval(expr);
			System.out.println(eval);
			Double val = eval.asDouble();
			prop.getOneTimeWriteAccess().setValue(val.intValue());
			System.out.println(getName()+" now has value "+val.intValue());
			return true;
		} else {
			System.out.println("Can't export this var!"+getName());
			return false;
		}
	}
	
}
