/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.appl;

import java.util.Vector;

import edu.bonn.xulu.model.XuluModel;
import edu.bonn.xulu.appl.AbstractManager;
import edu.bonn.xulu.gui.ModelControlFrame;

// nur fuer Doku
import schmitzm.data.event.ObjectListener;
import schmitzm.data.event.AbstractObjectTraceable;

/**
 * Der ModelManager verwaltet alle in der Xulu-Modelling-Platform aktiven
 * Modell-Instanzen durch eine Liste der zugehoerigen {@linkplain ModelControlFrame Kontrollfenster}.<br>
 * Der Manager fungiert als {@link AbstractObjectTraceable},
 * so dass {@link ObjectListener} angeschlossen werden
 * koennen, welche automatisch durch ein {@link AbstractManager.ChangeEvent}
 * informiert werden, wenn sich der Inhalt des Managers aendert.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class ModelControlManager extends AbstractManager {
  /**
   * Erzeugt einen neuen (leeren) ModelManager.
   */
  public ModelControlManager() {
    super(false);
    this.content = new Vector<ModelControlFrame>();
  }

  /**
   * Prueft, ob eine Komponente im ModelManager verwaltet werden
   * kann.
   * @param comp Komponente
   * @exception UnsupportedOperationException falls es sich bei der Komponente
   *            nicht um eine Instanz von {@link ModelControlFrame} handelt
   */
  public void checkComponentAndError(AbstractManager.ContentType comp) {
    if (! (comp instanceof ModelControlFrame))
      throw new UnsupportedOperationException(getClass().getSimpleName().concat(
          " can not manage instances of ").concat(comp.getClass().getName()));
  }

  /**
   * Liefert alle registrierte Modell-Kontroll-Fenster als Array.
   * @return leerer <code>ModelControlFrame[]</code>-Array falls keine Instanzen
   *         im Manager registriert sind.
   */
  public ModelControlFrame[] getAll() {
    return (ModelControlFrame[])content.toArray(new ModelControlFrame[0]);
  }
}
