/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.appl;

import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.VisualisationUpdateListener;

/**
 * Dieses Interface bildet die Grundlage fuer verschiedenste Visualisierungstools.
 * <b>Wichtig:</b><br>
 * Damit eine Implementierung dieser Klasse in der Xulu-Modelling-Platform
 * verwendet werden kann, <u>muss</u> sie einen Standard-Konstruktor
 * implementieren!!
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public interface VisualisationTool extends AbstractManager.ContentType {
  /**
   * Initialisiert das Visualisierungstools. Sollte nur einmal und unmittelbar
   * nach der Instanziierung aufgerufen werden.
   * @param appl Xulu-Instanz fuer die das Visualisierungstool erzeugt wurde 
   */
  public void initialize(XuluModellingPlatform appl);

  /**
   * Zeigt oder verbirgt das Fenster des Visualisierungstools.
   */
  public void setVisible(boolean visible);

  /**
   * Prueft, ob das Fenster des Visualisierungstools aktuell angezeigt wird.
   */
  public boolean isVisible();

  /**
   * Prueft, ob ein Objekt durch das Tool visualisiert werden kann.
   */
  public boolean isVisualisable(Object obj);

  /**
   * Prueft, ob Instanzen einer Klasse durch das Tool visualisiert werden kann.
   */
  public boolean isVisualisable(Class c);

  /**
   * Visualisiert ein Objekt. Beinhaltet das Visualisierungstool mehrere Fenster,
   * so muss es ueber einen Anwender-Dialog selbst erfragen, in welchem
   * Fenster/Layer das Objekt dargestellt werden soll.
   * @param obj  darzustellendes Objekt
   * @param desc Erlaeuterung, die im Vis.Tool zu dem Objekt angezeigt werden kann
   * @return <code>false</code>, falls das Objekt nicht dargestellt werden kann
   */
  public boolean add(Object obj, String desc);

  /**
   * Entfernt die Visualisierung eines Objekts.
   * @param obj  zu entfernendes Objekt
   * @return <code>false</code>, falls das Objekt nicht entfernt werden kann, z.B.
   *         weil es nicht (mehr) dargestellt wird.
   */
  public boolean remove(Object obj);

  /**
   * Aktualisiert die Visualisierung eins Objekts.
   * @param obj  zu aktualisierendes Objekt
   * @return <code>false</code>, falls das Objekt nicht aktualisiert werden kann, z.B.
   *         weil es nicht (mehr) dargestellt wird.
   */
  public boolean update(Object obj);

  /**
   * Schliesst das Visualisierungstool und alle darin enthaltenen Fenster/Layer.
   */
  public void close();

  /**
   * Liefert eine Beschreibung fuer die Instanz des Visualisierungstools.
   */
  public String getTitle();

  /**
   * Liefert eine Beschreibung fuer die Instanz des Visualisierungstools.
   */
  public void setTitle(String desc);
}
