/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.gui;

import java.io.File;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JFileChooser;
import java.util.Vector;

import schmitzm.io.FileOutputStream;
import schmitzm.io.FileInputStream;

import schmitzm.lang.NamedObject;

import schmitzm.data.property.Property;
import schmitzm.data.property.AccessViolationException;
import schmitzm.data.event.ObjectListener;
import schmitzm.data.event.ObjectEvent;
import schmitzm.data.event.ObjectTraceable;
import schmitzm.data.event.GeneralObjectChangeEvent;

import schmitzm.swing.SwingWorker;
import schmitzm.swing.menu.ActionStructure;
import schmitzm.swing.menu.ObjectSubMenu;
import schmitzm.swing.menu.ObjectMenuItem;

import schmitzm.lang.AlreadyHandledException;

import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.XuluRegistry;
import edu.bonn.xulu.appl.XuluConstants;
import edu.bonn.xulu.appl.VisualisationTool;
import edu.bonn.xulu.appl.VisualisationManager;
import edu.bonn.xulu.appl.VisualisationUpdateListener;
import edu.bonn.xulu.io.IODefinition;
import edu.bonn.xulu.io.InstantiationFactory;
import edu.bonn.xulu.io.ImportFactory;
import edu.bonn.xulu.io.ExportFactory;
import edu.bonn.xulu.io.FactoryCanceledException;
import edu.bonn.xulu.data.XuluObject;
import edu.bonn.xulu.data.XuluDataException;

/**
 * Diese Klasse stellt eine Menuestruktur fuer die Verwaltung des Datenpool
 * zur Verfuegung.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class DataPoolMenu extends ObjectSubMenu implements XuluConstants {
  /** Aktion steht fuer die Auswahl des "Neu"-Untermenues. */
  public static final int ACTION_NEW       = 1;
  /** Aktion steht fuer die Auswahl des "Import"-Untermenues. */
  public static final int ACTION_IMPORT    = 2;
  /** Aktion steht fuer die Auswahl des "Export"-Untermenues. */
  public static final int ACTION_EXPORT    = 4;
  /** Aktion steht fuer die Auswahl des "Display"-Untermenues. */
  public static final int ACTION_DISPLAY   = 8;
  /** Aktion steht fuer die Auswahl des Menu-Punkt "Display | Aktualisieren". */
  public static final int ACTION_ACTUALIZE = 16;
  /** Aktion steht fuer die Auswahl des Menu-Punkt "Kopieren". */
  public static final int ACTION_COPY = 32;
  /** Aktion steht fuer die Auswahl des Menu-Punkt "Löschen". */
  public static final int ACTION_DELETE = 64;
  /** Aktion steht fuer die Auswahl des Menu-Punkt "Import | Neu importieren" und
   *  "Export | Neu exportieren". */
  public static final int ACTION_RECENTIO = 128;

  private XuluModellingPlatform appl = null;

  /**
   * Erzeugt ein neues Aktionsmenue fuer den Xulu-Datenpool.
   * @param appl          Instanz der Xulu-Applikation
   * @param dataPoolFrame DatenPool-Fenster auf dem das Menu aufgerufen wird
   *                      (wird benoetigt, um die darin selektieren Objekte zu
   *                      referenzieren, auf denen die Menu-Aktionen ausgefuehrt
   *                      werden sollen)
   */
  public DataPoolMenu(final XuluModellingPlatform appl, DataPoolFrame dataPoolFrame) {
    super(XULUGUI_RES.getString("Menu_DataPool"),ACTION_NONE,null,new DataPoolMenuListener(appl,dataPoolFrame),false);

    add( new DataPoolSubMenu_New(this,itemActionListener,appl.getRegistry(),true) );
    add( new ObjectMenuItem(XULUGUI_RES.getString("Menu_DataPool_Del"),itemActionListener,ACTION_DELETE,this) );
    add( new DataPoolSubMenu_Import(this,itemActionListener,appl.getRegistry(), true));
    add( new DataPoolSubMenu_Export(this,itemActionListener,appl.getRegistry(),true) );
    addSeparator();
    add( new DataPoolSubMenu_Display(this,itemActionListener,appl.getRegistry(),appl.getVisualisationManager(),true) );


    ObjectListener updateListener = new ObjectListener() {
      public void performObjectEvent(ObjectEvent e) {
//        if ( e instanceof XuluRegistry.ChangeEvent ||
//             e instanceof VisualisationManager.ChangeEvent )
        if ( e.getSource().contains( appl.getRegistry() ) ||
             e.getSource().contains( appl.getVisualisationManager() ) )
          reorganize();
    }};

    this.appl = appl;
    appl.getRegistry().addObjectListener( updateListener );
    appl.getVisualisationManager().addObjectListener( updateListener );
  }

  public void reorganize() {
    super.reorganize();
  }

  /////////////////////////////////////////////////////////////////////
  // Untermenue zur Darstellung von Klassennamen
  /////////////////////////////////////////////////////////////////////
  /**
   * Diese Klasse stellt ein allgemeines Untermenue fuer die Auflistung von
   * Objekten dar. Angezeigt wird der jeweilige Klassenname. Den einzelnen
   * Menue-Eintraegen wird kein eigener ActionCode zugeordnet,
   * sondern lediglich {@link ActionStructure#ACTION_NONE}.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  public static class DataPoolSubMenu_ObjectClasses extends ObjectSubMenu {
    /** Speichert die angezeigten Objekte. */
    protected Object[] object = null;

    /**
     * Erzeugt ein Untermenue.
     * @param name               Anzeigename des Menues
     * @param actionCode         ActionCode fuer das Menue (nicht fuer die Eintraege!)
     * @param parent             Übergeordnete Aktions-Struktur
     * @param itemActionListener wird den Menuepunkten zugeordnet und verarbeitet
     *                           deren Auswahl
     * @param object             Objekte die dargestellt werden
     * @param reorganize         gibt, an ob das Menu bereits zur Instanziierung
     *                           komplett (mit allen Untermenues) aufgebaut wird
     */
    public DataPoolSubMenu_ObjectClasses(String name, int actionCode, ActionStructure parent, ActionListener itemActionListener, Object[] object, boolean reorganize) {
      super(name, actionCode, parent, itemActionListener, false);
      this.object = object;
      if (reorganize)
        reorganize();
    }

    /**
     * Reorganisiert das "Display"-Untermenue. Es werden alle
     * Visualisierungstool-Instanzen geloescht und auf Basis des
     * Visualisation-Managers neu eingefuegt. Danach werden
     * alle (etwaigen) Untermenues ("Neu") reorganisiert.
     */
    public void reorganize() {
      // Index des ersten und zweiten Seperators finden
      int firstSepIdx = -1;
      int secondSepIdx = -1;
      for (int i = 0; i < getItemCount(); i++)
        if ( getItem(i) == null ) {
          if ( firstSepIdx < 0 )
            firstSepIdx = i;
          else {
            secondSepIdx = i;
            break;
          }
        }
      // Wenn keine Seperator vorhanden, werden alle Menue-Eintraege
      // geloescht
      if ( firstSepIdx < 0 )
        firstSepIdx = -1;
      if ( secondSepIdx < 0 )
        secondSepIdx = getItemCount();

      // erst alle Klassen-Eintraege zwischen dem ersten und zweiten
      // Seperator loeschen
      // Beachte: von hinten nach vorne loeschen!!
      //          Alles zwischen den Seperatorn
      for (int i = secondSepIdx-1; i > firstSepIdx; i--)
        if ( getItem(i) instanceof ObjectMenuItem )
          remove(i);

      // Klassen vorne neu einfuegen
      for (int i = 0; i < object.length; i++) {
        String name = object[i].getClass().getSimpleName();
        if ( object[i] instanceof Class )
          name = ((Class)object[i]).getSimpleName();
        else if ( object[i] instanceof NamedObject ) {
          NamedObject namedObject = (NamedObject)object[i];
          if ( namedObject != null && namedObject.getName() != null && !namedObject.getName().equals("") )
            name = namedObject.getName();
        } 
        ObjectMenuItem item = new ObjectMenuItem(name,object[i],ActionStructure.ACTION_NONE,this);
        item.addActionListener( itemActionListener );
        insert(item, firstSepIdx+1+i);
      }

      // SubMenus reorganisieren
      super.reorganize();
    }
  }

  /////////////////////////////////////////////////////////////////////
  // Untermenue "Neu"
  /////////////////////////////////////////////////////////////////////
  /**
   * Diese Klasse stellt das Untermenue "Neu" des Datenpool-Menues dar.
   * Es beinhaltet eine Liste mit den in der Xulu-Registry registrierten
   * Datentypen und ein {@link DataPoolMenu.DataPoolSubMenu_Import Import-Untermenue}.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  public static class DataPoolSubMenu_New extends DataPoolSubMenu_ObjectClasses {
    private XuluRegistry registry = null;
    private ObjectMenuItem copyItem = null;

    /**
     * Erzeugt ein "Neu"-Untermenue.
     * @param parent             Übergeordnete Aktions-Struktur
     * @param itemActionListener wird den Menuepunkten zugeordnet und verarbeitet
     *                           deren Auswahl
     * @param registry           Xulu-Registry als Datenbasis
     * @param reorganize         gibt, an ob das Menu bereits zur Instanziierung
     *                           komplett (mit allen Untermenues) aufgebaut wird
     */
    public DataPoolSubMenu_New(ActionStructure parent, ActionListener itemActionListener, final XuluRegistry registry, boolean reorganize) {
      super(XULUGUI_RES.getString("Menu_DataPool_New"),ACTION_NEW,parent,itemActionListener,registry.getDataTypes(),false);
      this.registry = registry;
      ObjectMenuItem item = new ObjectMenuItem(XULUGUI_RES.getString("Menu_DataPool_New_CopyStr"),null,ACTION_COPY,this);
      item.addActionListener(itemActionListener);
      add(item);
      addSeparator();
//      addSeparator();
//      add(new DataPoolSubMenu_Import(this,itemActionListener,registry, false));
      if (reorganize)
        reorganize();
    }

    /**
     * Ermittelt die Datentypen aus der Registry neu und regorganisiert dann
     * das Menu.
     */
    public void reorganize() {
      this.object = registry.getDataTypes();
      super.reorganize();
    }
  }

  /////////////////////////////////////////////////////////////////////
  // Untermenue "Import"
  /////////////////////////////////////////////////////////////////////

  /**
   * Diese Klasse stellt ein Untermenue "Import" dar.
   * Es beinhaltet eine Liste mit den in der Xulu-Registry registrierten
   * Import-Factorys.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  public static class DataPoolSubMenu_Import extends DataPoolSubMenu_ObjectClasses {
    private XuluRegistry registry = null;

    /**
     * Erzeugt ein "Import"-Untermenue.
     * @param parent             Übergeordnete Aktions-Struktur
     * @param itemActionListener wird den Menuepunkten zugeordnet und verarbeitet
     *                           deren Auswahl
     * @param registry           Xulu-Registry als Datenbasis
     * @param reorganize         gibt, an ob das Menu bereits zur Instanziierung
     *                           komplett (mit allen Untermenues) aufgebaut wird
     */
    public DataPoolSubMenu_Import(ActionStructure parent, ActionListener itemActionListener, XuluRegistry registry, boolean reorganize) {
      super(XULUGUI_RES.getString("Menu_DataPool_Import"), ACTION_IMPORT, parent, itemActionListener, registry.getImportFactories(), false);
      this.registry = registry;
      add(new DataPoolSubMenu_RecentIOs(this,XULUGUI_RES.getString("Menu_DataPool_RecentImport"),itemActionListener,registry.getRecentImports(), false));
      addSeparator();
      if (reorganize)
        reorganize();
    }

    /**
     * Ermittelt die ImportFactorys aus der Registry neu und regorganisiert dann
     * das Menu.
     */
    public void reorganize() {
      this.object = registry.getImportFactories();
      super.reorganize();
    }
  }

  /////////////////////////////////////////////////////////////////////
  // Untermenue "Export"
  /////////////////////////////////////////////////////////////////////
  /**
   * Diese Klasse stellt ein Untermenue "Export" dar.
   * Es beinhaltet eine Liste mit den in der Xulu-Registry registrierten
   * Export-Factorys.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  public static class DataPoolSubMenu_Export extends DataPoolSubMenu_ObjectClasses {
    private XuluRegistry registry = null;

    /**
     * Erzeugt ein "Export"-Untermenue.
     * @param parent             Übergeordnete Aktions-Struktur
     * @param itemActionListener wird den Menuepunkten zugeordnet und verarbeitet
     *                           deren Auswahl
     * @param registry           Xulu-Registry als Datenbasis
     * @param reorganize         gibt, an ob das Menu bereits zur Instanziierung
     *                           komplett (mit allen Untermenues) aufgebaut wird
     */
    public DataPoolSubMenu_Export(ActionStructure parent, ActionListener itemActionListener, XuluRegistry registry, boolean reorganize) {
      super(XULUGUI_RES.getString("Menu_DataPool_Export"), ACTION_EXPORT, parent, itemActionListener, registry.getExportFactories(), false);
      this.registry = registry;
//      add(new DataPoolSubMenu_RecentIOs(this,XULUGUI_RES.getString("Menu_DataPool_RecentExport"),itemActionListener,registry.getRecentExports(), false));
//      addSeparator();
      if (reorganize)
        reorganize();
    }

    /**
     * Ermittelt die ExportFactorys aus der Registry neu und regorganisiert dann
     * das Menu.
     */
    public void reorganize() {
      this.object = registry.getExportFactories();
      super.reorganize();
    }
  }

  /////////////////////////////////////////////////////////////////////
  // Untermenue "Display"
  /////////////////////////////////////////////////////////////////////
  /**
   * Diese Klasse stellt ein Untermenue "Display" dar.
   * Es beinhaltet eine Liste mit den im VisualisationManager instanziierten
   * Tools, sowie (in einem Untermenue "Neu") alle in Xulu-Registry registrierten
   * Visualisierungstool-Klassen.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  public static class DataPoolSubMenu_Display extends ObjectSubMenu {
    private VisualisationManager visManager;

    /**
     * Erzeugt ein "Display"-Untermenue.
     * @param parent             Übergeordnete Aktions-Struktur
     * @param itemActionListener wird den Menuepunkten zugeordnet und verarbeitet
     *                           deren Auswahl
     * @param registry           Xulu-Registry als Datenbasis
     * @param reorganize         gibt, an ob das Menu bereits zur Instanziierung
     *                           komplett (mit allen Untermenues) aufgebaut wird
     */
    public DataPoolSubMenu_Display(ActionStructure parent, ActionListener itemActionListener, final XuluRegistry registry, VisualisationManager visManager, boolean reorganize) {
      super(XULUGUI_RES.getString("Menu_DataPool_Display"), ACTION_DISPLAY, parent, itemActionListener, false);
      this.visManager = visManager;
      // Menuepunkt "Aktualisieren"
      ObjectMenuItem item = new ObjectMenuItem(XULUGUI_RES.getString("Menu_DataPool_Display_Actualise"),null,ACTION_ACTUALIZE,this);
      add(item);
      item.addActionListener(itemActionListener);
      // Seperator vor und nach den aktiven Visualisierungstools
      insertSeparator(1);
      insertSeparator(2);
      add(new DataPoolSubMenu_ObjectClasses(XULUGUI_RES.getString("Menu_DataPool_Display_New"),ACTION_NEW,this,itemActionListener,registry.getVisualisationTools(), false) {
        public void reorganize() {
          this.object = registry.getVisualisationTools();
          super.reorganize();
        }
      });
      if (reorganize)
        reorganize();
    }

    /**
     * Reorganisiert das "Display"-Untermenue. Es werden alle
     * Visualisierungstool-Instanzen geloescht und auf Basis des
     * Visualisation-Managers neu eingefuegt. Danach werden
     * alle (etwaigen) Untermenues ("Neu") reorganisiert.
     */
    public void reorganize() {
        // erst alle Menue-Eintraege fuer VisualisationTool-Instanzen loeschen
        for (int i = 0; i < getItemCount(); i++)
          if ( getItem(i) instanceof ObjectMenuItem && ((ObjectMenuItem)getItem(i)).getObject() instanceof VisualisationTool )
            remove(i--);

        // VisualisationTool-Instanzen HINTER (!) erstem Seperator einfuegen
        VisualisationTool[] openTool = (VisualisationTool[])visManager.getAll();
        for (int i = 0; i < openTool.length; i++) {
          ObjectMenuItem item = new ObjectMenuItem(openTool[i].getTitle(),openTool[i],ActionStructure.ACTION_NONE,this);
          item.addActionListener( itemActionListener );
          insert(item, i+2); // +2 weil darueber "Aktualisieren" und der Seperator
        }

      // SubMenus reorganisieren
      super.reorganize();
    }
  }

  /////////////////////////////////////////////////////////////////////
  // Untermenue "Letzte Dateien"
  /////////////////////////////////////////////////////////////////////

  /**
   * Diese Klasse stellt ein Untermenue fuer die letzten Import- oder
   * Export-Operationen dar.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  public static class DataPoolSubMenu_RecentIOs extends ObjectSubMenu {
    private Vector<? extends IODefinition> recentIOs = new Vector<IODefinition>();

    /**
     * Erzeugt ein neues Untermenue.
     * @param parent             Übergeordnete Aktions-Struktur
     * @param name               Bezeichnung fuer das Menue
     * @param itemActionListener wird den Menuepunkten zugeordnet und verarbeitet
     *                           deren Auswahl
     * @param recentIOs          Liste der letzten I/O-Operationen, die im Menue
     *                           dargestellt werden
     * @param reorganize         gibt, an ob das Menu bereits zur Instanziierung
     *                           komplett (mit allen Untermenues) aufgebaut wird
     */
    public DataPoolSubMenu_RecentIOs(ActionStructure parent, String name, ActionListener itemActionListener, Vector<? extends IODefinition> recentIOs, boolean reorganize) {
      super(name, ACTION_RECENTIO, parent, itemActionListener, false);
      this.recentIOs = recentIOs;
      if (reorganize)
        reorganize();
    }

    /**
     * Reorganisiert das "Letzte Dateien"-Untermenue. Es werden alle Eintraege
     * geloescht und auf Basis der Datei-Liste neu eingefuegt.
     */
    public void reorganize() {
      // erst alle Menue-Eintraege loeschen
      removeAll();

      // Alle Imports (mit Factory) einfuegen (der aelteste Import steht
      // im ersten Element!)
      for (int i = recentIOs.size()-1; i>=0; i--) {
        IODefinition ioDef    = recentIOs.elementAt(i);
        Object       ioObj    = ioDef.getIOObject();
        // Namen des Menue-Eintrags zusammensetzen
        // -> Factory-Name + Liste der Quell- bzw. Ziel-Objekte
        Object[]     ioObjArr = ( ioObj instanceof Object[] ) ? (Object[])ioObj : new Object[] { ioObj };
        String       itemName = ioDef.getFactory().getName() + " >> ";
        for (int j=0; j<ioObjArr.length; j++) {
          if ( j > 0 )
            itemName += "; ";
          ioObj = ioObjArr[j];
          if ( ioObj instanceof File )
            itemName += ((File)ioObj).getName();
          else
            itemName += ioObj.toString();
        }
        // Menue-Eintrag soll max. 50 Zeichen lang sein
        if ( itemName.length() > 75 )
          itemName = itemName.substring(0,75) + " ... ";

        // Menue-Eintrag erzeugen
        ObjectMenuItem item = new ObjectMenuItem(itemName, ioDef, ActionStructure.ACTION_NONE, this);
        item.addActionListener(itemActionListener);
        add(item);
      }

      // SubMenus reorganisieren
      super.reorganize();
    }
  }

  /////////////////////////////////////////////////////////////////////
  // Listener fuer die Menues, der die entspr. Funktionen implementiert
  /////////////////////////////////////////////////////////////////////

  /**
   * Diese Klasse stellt einen Listener dar, der die Aktionen, die im
   * <code>DataPoolMenu</code> angewaehlt wurden, interpretiert und
   * durchfuehrt.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  private static class DataPoolMenuListener implements ActionListener {
    private DataPoolFrame         dataPoolFrame = null;
    private XuluModellingPlatform appl = null;

    /**
     * Erzeugt einen Listener fuer das DatenPool-Menue.
     * @param appl    XuluModellingPlatform-Instanz in der die Aktionen
     *                ausgefuehrt werden
     * @param parent  DatenPool-Frame, der das Menue aufruft (benoetigt um
     *                selektierte Objekte zu referenzieren, auf die die Aktionen
     *                ausgefuehrt werden sollen)
     */
    public DataPoolMenuListener(XuluModellingPlatform appl, DataPoolFrame parentFrame) {
      this.appl          = appl;
      this.dataPoolFrame = parentFrame;
    }

    /**
     * Verarbeitet die jeweilige Menu-Auswahl und reagiert auf etwaige Fehler
     * mit einer Meldung.
     */
    public void actionPerformed(ActionEvent e) {
      if (e == null || e.getSource() == null ||
          ! (e.getSource() instanceof ObjectMenuItem))
        return;

      ObjectMenuItem item = (ObjectMenuItem) e.getSource();
      Object menuObject = item.getObject();
      int menuAction = item.getCompleteActionCode();


      try {
        Object newDataPoolObject = null;
        // wenn NEU ohne IMPORT >> MenuObject enthaelt DatenTyp
        //          ohne COPY   >> 1) DefaultFactory aus TypeMapping
        //                         2) Checken, ob XuluObject erzeugt wird
        //                         3) Neues Objekt erzeugen und DP hinzufuegen
        if ( item.compareActions(menuAction, DataPoolMenu.ACTION_NEW) &&
            !item.compareActions(menuAction, DataPoolMenu.ACTION_IMPORT) &&
            !item.compareActions(menuAction, DataPoolMenu.ACTION_COPY) &&
            !item.compareActions(menuAction, DataPoolMenu.ACTION_DISPLAY)) {
          InstantiationFactory fac = appl.getRegistry().getDefaultFactory((Class)menuObject);
          if (fac == null)
            throw new XuluDataException("No TypeMapping found for ".concat(
                menuObject.getClass().getName()));
          if (!XuluObject.class.isAssignableFrom(fac.getInstanceType()))
            throw new XuluDataException("Factory does not create an instance of ".
                                        concat(XuluObject.class.getName()));
          newDataPoolObject = fac.newInstance(true);
          appl.getDataPool().addObject( (XuluObject) newDataPoolObject);
        }

        // wenn IMPORT >> MenuObject enthaelt ImportDefinition
        //             >> 1) Checken, ob XuluObject erzeugt wird
        //                2) Dateiname abfragen
        //                3) Neues Objekt erzeugen und DP hinzufuegen
        if ( item.compareActions(menuAction, DataPoolMenu.ACTION_IMPORT) &&
            !item.compareActions(menuAction, DataPoolMenu.ACTION_RECENTIO)) {
          ImportFactory fac = (ImportFactory) menuObject;
          XuluGUIUtil.importObjectsToDataPool(dataPoolFrame,fac,appl);
        }

        // wenn RECENT IMPORT >> MenuObject enthaelt ImportFactory
        //                    >> 1) Checken, ob XuluObject erzeugt wird
        //                       2) Neues Objekt erzeugen und DP hinzufuegen
        if ( item.compareActions(menuAction, DataPoolMenu.ACTION_IMPORT) &&
             item.compareActions(menuAction, DataPoolMenu.ACTION_RECENTIO)) {
          final IODefinition.ImportDefinition impDef = (IODefinition.ImportDefinition) menuObject;
          final ImportFactory                 fac    = impDef.getFactory();
          if ( !XuluObject.class.isAssignableFrom(fac.getImportType()) )
            throw new XuluDataException("Factory does not create an instance of ".concat(XuluObject.class.getName()));
          SwingWorker.Work work = new SwingWorker.Work() {
            public Object execute() throws Exception {
              Object impObj = fac.importObject( XuluGUIUtil.convertImportSourceAsNeeded(fac,impDef.getIOObject()), appl.getRegistry() );
              XuluGUIUtil.insertObjectToDataPool(dataPoolFrame,impObj,appl.getDataPool(),false,false);
              // Import wieder an die erste Stelle der Recent-Imports setzen
              appl.getRegistry().getRecentImports().remove( impDef );
              appl.getRegistry().getRecentImports().add( impDef );
              return null;
            }
          };
          new SwingWorker(work,appl.getMainFrame(),XULUGUI_RES.getString("ImportProgressMess"),0.2,0.2).start();
        }

        // wenn NEU durch COPY >> MenuObject enthaelt ImportFactory
        //                     >> 1) Checken, ob XuluObject erzeugt wird
        //                        2) Dateiname abfragen
        //                        3) Neues Objekt erzeugen und DP hinzufuegen
        if (item.compareActions(menuAction, DataPoolMenu.ACTION_NEW) &&
            item.compareActions(menuAction, DataPoolMenu.ACTION_COPY)) {
          Object selObj = dataPoolFrame.getSelectedObject();
          if ( selObj != null ) {
            // Zur Zeit koennen nur komplette Xulu-Objekte kopiert werden
            if ( !(selObj instanceof XuluObject) )
              throw new XuluDataException("Zur Zeit koennen nur ganze Xulu-Objekte kopiert werden. Keine einzelnen Eigenschaften!");
            XuluObject copyFromObj   = (XuluObject)selObj;
            InstantiationFactory fac = appl.getRegistry().getDefaultFactory(copyFromObj.getClass());
            newDataPoolObject        = fac.newInstance(true,copyFromObj);
            // Bezeichnung von kopiertem Objekt uebernehmen
            XuluObject newXuluObj = (XuluObject)newDataPoolObject;
            newXuluObj.setDescription("Kopie von ".concat(copyFromObj.getDescription()));
            // Objekt in Datenpool aufnehmen
            appl.getDataPool().addObject( newXuluObj );
          }
        }
      } catch (FactoryCanceledException err) {
        // manueller/gewollter Abbruch der Factory
      } catch (XuluDataException err) {
        XuluGUIMessages.showError(dataPoolFrame, err, "Objekt kann nicht erzeugt werden!");
      } catch (ClassCastException err) {
        XuluGUIMessages.showError(dataPoolFrame, err,
            "Der Xulu-Datenpool kann nur XuluObject-Instanzen aufnehmen!");
      } catch (AlreadyHandledException err) {
        // Exception wurde bereits verarbeitet
      } catch (Exception err) {
        XuluGUIMessages.showError(dataPoolFrame, err);
      }


      // wenn LOESCHEN >> 1) Alle selektieren Xulu-Objekte loeschen
      if ( item.compareActions(menuAction, DataPoolMenu.ACTION_DELETE) ) {
        try {
            // alle selektierten Objekte aus dem Datenpool entfernen und
            // zerstoeren
            Object[] obj = dataPoolFrame.getSelectedObjects();
            for (int i = 0; i < obj.length; i++)
              if ( obj[i] instanceof XuluObject ) {
                XuluObject xuluObject = (XuluObject)obj[i];
                // Wenn zur Zeit noch Zugriffe auf das Objekt bestehen, kann es
                // nicht geloescht werden
                if ( xuluObject.isAccessed() )
                  XuluGUIMessages.showError(dataPoolFrame,"Das Objekt '"+xuluObject.getDescription()+"' kann nicht geloescht werden. Es wird noch darauf zugegriffen!");
                else {
                  // Objekt aus Datenpool entfernen
                  appl.getDataPool().removeObject(xuluObject);
                  // Objekt aus Visualisierungen loeschen
                  for (int j = 0; j<appl.getVisualisationManager().getCount(); j++)
                    ((VisualisationTool)appl.getVisualisationManager().get(j)).remove(xuluObject);
                  // Objekt zerstoeren
                  xuluObject.dispose();
                }
              }
            dataPoolFrame.refresh();
        } catch ( Exception err ){
          XuluGUIMessages.showError(dataPoolFrame, err);
        }
      }

      // wenn EXPORT >> MenuObject enthaelt ExportFactory
      //             >> 1) Checken, ob Factory die ausgewaehlten XuluObjekte
      //                   exportieren kann
      //                2) Dateinamen abfragen
      //                3) DP-Objekte exportieren
      if (item.compareActions(menuAction, DataPoolMenu.ACTION_EXPORT)) {
        ExportFactory fac = (ExportFactory) menuObject;
        // versuchen, alle selektierten Objekte zu exportieren
        Object[] obj = dataPoolFrame.getSelectedObjects();
        for (int i = 0; i < obj.length; i++)
          try {
            XuluGUIUtil.exportObjects(dataPoolFrame,obj[i],fac,appl);
          } catch (XuluDataException err) {
            String name = obj[i] instanceof XuluObject ? ((XuluObject)obj[i]).getDescription() : ((Property)obj[i]).getName();
            XuluGUIMessages.showError(dataPoolFrame, err, "Objekt '".concat(name).concat("' konnte nicht exportiert werden werden!"));
          } catch (AlreadyHandledException err) {
            // Exception wurde bereits verarbeitet
          } catch (AccessViolationException err) {
            XuluGUIMessages.showAccessError(dataPoolFrame, err);
          } catch (Exception err) {
            XuluGUIMessages.showError(dataPoolFrame, err);
          }
      }

      // wenn DISPAY, aber nicht AKTUALISIEREN
      //             >> 1) Visualisierungstool bestimmen oder erzeugen
      //                   MenuObject enthaelt VisualisationTool oder entspr. Klasse
      //                2) Checken, ob Tool die das ausgewaehlten Objekte
      //                   darstellen kann
      //                3) Objekte darstellen
      if ( item.compareActions(menuAction, DataPoolMenu.ACTION_DISPLAY) &&
          !item.compareActions(menuAction, DataPoolMenu.ACTION_ACTUALIZE) ) {
        try {
          VisualisationTool visTool = null;
          // VisualisationTool bestimmen oder erzeugen und dem Manager hinzufuegen
          if (!item.compareActions(menuAction, DataPoolMenu.ACTION_NEW))
            visTool = (VisualisationTool) menuObject;
          else {
            visTool = (VisualisationTool) ( (Class) menuObject).newInstance();
            visTool.initialize(appl);
            appl.getVisualisationManager().add(visTool,true);
          }
          // versuchen, alle selektierten Objekte darzustellen
          Object[] obj = dataPoolFrame.getSelectedObjects();
          for (int i = 0; i < obj.length; i++)
            try {
              // Objekte, die nicht darstellbar sind, ignorieren
//          if ( !visTool.isVisualisable(obj[i]) )
//            throw new XuluDataException(visTool.getClass().getSimpleName().concat(" can not visualise an instance of ").concat(obj[i].getClass().getName()));
              String desc = "neues Objekt";
              if ( obj[i] instanceof XuluObject )
                desc =  ((XuluObject)obj[i]).getDescription();
              visTool.add(obj[i], desc);
// add-Methode liefert keinen UpdateListener mehr!!
//              VisualisationUpdateListener updateListener = visTool.add(obj[i], desc);
//              // wenn es sich um ein "verfolgbares" Objekt handelt,
//              // den Listener zuordnen
//              if ( obj[i] instanceof ObjectTraceable )
//                ((ObjectTraceable)obj[i]).addObjectListener(updateListener);

            }
            catch (AlreadyHandledException err) {
              // Exception wurde bereits verarbeitet
            }
            catch (AccessViolationException err) {
              XuluGUIMessages.showAccessError(dataPoolFrame, err);
            }
            catch (Exception err) {
              XuluGUIMessages.showError(dataPoolFrame, err);
            }
        } catch ( Exception err ){
            XuluGUIMessages.showError(dataPoolFrame, err);
        }
      }

      // wenn DISPAY und AKTUALISIEREN
      //             >> 1) Visualisierungstool bestimmen oder erzeugen
      //                   MenuObject enthaelt VisualisationTool oder entspr. Klasse
      //                2) Checken, ob Tool die das ausgewaehlten Objekte
      //                   darstellen kann
      //                3) Objekte darstellen
      if ( item.compareActions(menuAction, DataPoolMenu.ACTION_DISPLAY) &&
           item.compareActions(menuAction, DataPoolMenu.ACTION_ACTUALIZE) ) {
        try {
            // versuchen, alle selektierten Objekte zu aktualisieren
            Object[] obj = dataPoolFrame.getSelectedObjects();
            for (int i = 0; i < obj.length; i++)
              for (int j = 0; j<appl.getVisualisationManager().getCount(); j++)
                ((VisualisationTool)appl.getVisualisationManager().get(j)).update(obj[i]);
//              if ( obj[i] instanceof ObjectTraceable ) {
//                ObjectListener[] visListener = ( (ObjectTraceable) obj[i]).getObjectListener(VisualisationUpdateListener.class);
//                for (int j=0; j<visListener.length; j++)
//                  ((VisualisationUpdateListener)visListener[j]).refresh();
//              }
        } catch ( Exception err ){
            XuluGUIMessages.showError(dataPoolFrame, err);
        }
      }
    }
  }

}
