/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.gui;

import javax.swing.JFrame;

import edu.bonn.xulu.model.XuluModel;
import edu.bonn.xulu.appl.AbstractManager;


// nur fuer Doku
import edu.bonn.xulu.gui.XuluInternalFrame;
import edu.bonn.xulu.gui.DataPoolFrame;
import edu.bonn.xulu.gui.RegistryFrame;

/**
 * Diese Klasse bildet eine abstrakte Oberklasse fuer alle Implementierungen des
 * Modell-Kontroll-Fensters. Die Modellfenster werden (im Gegensatz z.B. zum
 * {@linkplain DataPoolFrame Datenpool} oder der {@linkplain RegistryFrame Registry})
 * nicht in das Hauptfenster der {@linkplain XuluMainFrame Xulu-Applikation}
 * integriert, sondern stellen unabhaengige Fenster da. Deshalb sind sie keine
 * Unterklassen von {@link XuluInternalFrame}, sondern von {@link JFrame}.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public abstract class ModelControlFrame extends JFrame implements AbstractManager.ContentType {
  /** Speichert die zugehoerige Xulu-Modell-Instanz */
  protected XuluModel model = null;

  /**
   * Erzeugt einen neue Instanz des grafischen Modell-Konroll-Fensters. Dieses
   * erhaelt als Namen die Bezeichunng des Modells.
   */
  public ModelControlFrame(XuluModel model) {
    super(model.getName());
    this.model = model;
  }

  /**
   * Liefert das dem Kontrollfenster zugeordnete Modell.
   */
  public XuluModel getModel() {
    return this.model;
  }

  /**
   * Aktualisiert die grafische Darstellung des internen Kontrollfensters.
   */
  public abstract void refresh();

  /**
   * Prueft, ob zwei Modell-Kontrollfenster gleich sind. Da im ModellManager keine
   * zwei Modelle den gleichen Namen haben sollen, gelten zwei Modell-Kontrollfenster
   * bereits als gleich, wenn <b>sie die gleiche Bezeichnung besitzen</b>!
   * @param o anderes Objekt
   * @see #getTitle()
   */
  public boolean equals(Object o) {
    if ( !(o instanceof ModelControlFrame) )
      return false;
    ModelControlFrame otherFrame = (ModelControlFrame)o;
    return o != null &&
         ( this.getTitle() == null && otherFrame.getTitle() == null ||
           this.getTitle().equals( otherFrame.getTitle() ) );
  }

}
