/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.gui;

import java.awt.Component;
import javax.swing.JInternalFrame;

import edu.bonn.xulu.appl.XuluConstants;

/**
 * Diese Klasse bildet die abstrakte Oberklasse fuer ein Fenster, das im
 * {@link edu.bonn.xulu.gui.XuluMainFrame} dargestellt werden soll.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public abstract class XuluInternalFrame extends JInternalFrame implements XuluConstants {
  /** Wird benoetigt, um in internen Subklassen Zugriff auf den Frame (als Parent) zu haben */
  protected Component GUI_COMPONENT = this;

  /**
   * Erzeugt einen neue Instanz des internen Fensters. Dieses ist
   * vergroesserbar, maxi- und minimierbar, nicht aber schliessbar.
   * Implementiert das Hauptfenster Methoden, um das Fenster wieder sichtbar
   * zu machen, kann diese Option mittels {@link JInternalFrame#setClosable(boolean)}
   * geaendert werden. Gleichzeitig muss aber auch die
   * {@link JInternalFrame#setDefaultCloseOperation(int) DefaultCloseOperation}
   * auf <code>HIDE_ON_CLOSE</code> gesetzt werden, da sonst das Fenster beim
   * Schliessen zerstoert wird!
   * @param name Bezeichnung fuer das Fenster
   */
  public XuluInternalFrame(String name) {
    super(name, true, false, true, true);
    this.setFrameIcon( this.XULU_ICON );
  }

  /**
   * Aktualisiert die grafische Darstellung des internen Fensters.
   */
  public abstract void refresh();
}
