/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.gui;

import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.AWTEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import javax.swing.JDialog;
import javax.swing.JButton;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.table.AbstractTableModel;
import java.io.File;
import java.util.Locale;
import java.util.Set;
import java.util.TreeSet;
import java.util.ResourceBundle;

import schmitzm.swing.ExceptionDialog;
import schmitzm.swing.JPanel;
import schmitzm.swing.MultipleOptionPane;
import schmitzm.swing.InputOption;
import schmitzm.swing.SelectionInputOption;
import schmitzm.swing.ManualInputOption;
import schmitzm.swing.event.InputOptionAdapter;
import schmitzm.lang.ResourceProvider;
import schmitzm.lang.LocaleComparator;

import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.XuluComponent;


/**
 * Diese Klasse stellt einen Dialog zum Verwalten der Xulu-Spachpakete dar.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class XuluLanguageManagerFrame extends JDialog implements ActionListener, XuluComponent {
  private boolean disposed     = false;

  private JTextPane infoText      = null;
  private JButton   cancelButton  = null;
  private JButton   newLangButton = null;
  private JTable    bundleTable   = null;
  private JPanel    controlPanel  = null;

  private SelectionInputOption newLangDialog_LangDesc = null;
  private ManualInputOption    newLangDialog_LangCode = null;

  /**
   * Erzeugt einen neuen Dialog.
   * @param parent aufrufendes Fenster
   * @param modal  Flag, ob des Fenster model ist
   */
  public XuluLanguageManagerFrame(Frame parent, boolean modal) {
    super(parent,modal);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try {
      initGUI();
      initNewLanguageDialog();
    }
    catch(Exception e) {
      ExceptionDialog.show(e);
    }
  }

  /**
   * Initialisiert die GUI.
   */
  private void initGUI() throws Exception  {
    //imageLabel.setIcon(new ImageIcon(XuluMainFrame_Infodialog.class.getResource("[Ihre Grafik]")));
    this.setTitle(XULUGUI_RES.getString("LangFrame.Title"));
    this.setSize(500,400);
    this.getContentPane().setLayout(new BorderLayout());
    
    infoText = new JTextPane();
    infoText.setText( XULUGUI_RES.getString("LangFrame.InfoText") );
    infoText.setBackground( this.getContentPane().getBackground() );
    
    bundleTable = new JTable( new BundleTableModel() );

    cancelButton = new JButton(SWING_RES.getString("Cancel"));
    cancelButton.addActionListener(this);
    newLangButton = new JButton(XULUGUI_RES.getString("LangFrame.NewLang"));
    newLangButton.addActionListener(this);

    controlPanel = new JPanel();
    controlPanel.setLayout( new FlowLayout(FlowLayout.CENTER) );
    controlPanel.add( cancelButton );
    controlPanel.add( newLangButton );

    this.getContentPane().add(infoText, BorderLayout.NORTH);
    this.getContentPane().add(controlPanel, BorderLayout.SOUTH);
    this.getContentPane().add(new JScrollPane(bundleTable), BorderLayout.CENTER);

//    this.pack();
  }

  /**
   * Initalisiert die Eingabe-Optionen fuer den Dialog zum Ergeugen
   * eines neuen Sprach-Bundles.
   */
  private void initNewLanguageDialog() {
    // Verfuegbare Sprachen nach der Bezeichnung sortieren
    TreeSet<Locale> sortedLocales = new TreeSet<Locale>(new LocaleComparator(true));
    for (Locale l : Locale.getAvailableLocales())
      sortedLocales.add(l);

    // Bezeichnungen und Sprachcodes in Array einfuegen
    String[] avLocaleDesc = new String[sortedLocales.size()+1];
    String[] avLocaleCode = new String[sortedLocales.size()+1];
    int i = 0;
    for (Locale l : sortedLocales) {
      avLocaleDesc[i] = l.getDisplayName();
      avLocaleCode[i] = l.toString();
      i++;
    }
    // Zusaetzliche Auswahl "andere Sprache" hinzufuegen
    avLocaleDesc[i] = XULUGUI_RES.getString("LangFrame.OtherLang");
    avLocaleCode[i] = "";

    // Eingabe-Option fuer den Sprach-Code erstellen
    newLangDialog_LangCode = new ManualInputOption.Text(
      XULUGUI_RES.getString("LangFrame.LanguageCode"),
      true
    );
    // Auswahl-Option mit den vordefinierten Sprachen erstellen
    newLangDialog_LangDesc = new SelectionInputOption.Combo(
      XULUGUI_RES.getString("LangFrame.Language"),
      true,
      avLocaleCode,
      "",
      avLocaleDesc
    );
    // Wenn sich die Auswahl aendert, soll der Code im Text-Feld
    // angezeigt werden
    newLangDialog_LangDesc.addInputOptionListener(new InputOptionAdapter() {
      public void optionChanged(InputOption option, Object oldValue, Object newValue) {
        newLangDialog_LangCode.setValue(newValue);
        newLangDialog_LangCode.transferFocus();
        newLangDialog_LangCode.setEnabled( "".equals(newValue));
      }
    });
  }

  /**
   * Schliesst das Fenster, wenn es sich um ein <code>WindowEvent.WINDOW_CLOSING<code>-Event
   * handelt.
   */
  protected void processWindowEvent(WindowEvent e) {
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      cancel();
    }
    super.processWindowEvent(e);
  }

  /**
   * Zerstoert das Fenster. Ruft <code>super.dispose()</code> auf.
   */
  public void dispose() {
    disposed = true;
    super.dispose();
  }

  /**
   * Prueft, ob das Fenster zerstoert ist.
   */
  public boolean isDisposed() {
    return disposed;
  }

  /**
   * Schliesst das Fenster.
   */
  private void cancel() {
    dispose();
  }

  /**
   * Fragt eine neue Sprache vom Anwender ab und legt die entsprechenden
   * Resource-Bundles an.
   */
  private void createNewLanguage() {
    // Auswahl auf die erste Sprache setzen
    newLangDialog_LangDesc.setSelectedIndex(0);
    // Dialog anzeigen
    Object[] input = MultipleOptionPane.showMultipleInputDialog(
        this,
        XULUGUI_RES.getString("LangFrame.NewLang"),
        newLangDialog_LangDesc,
        newLangDialog_LangCode
    );
    if ( input == null )
      return;

    // Property-Daten fuer alle Sprach-Bundles erstellen
    Locale newLocale = new Locale((String)input[1]);
    try {
      // Pruefen, ob irgendeine Datei bereits existiert
      for ( ResourceProvider rp : XuluModellingPlatform.RESOURCE_BUNDLES ) {
        File file = ResourceProvider.getPropertyFile(rp,newLocale);
        if ( file.exists() ) {
          XuluGUIMessages.FileOverrideOption answer = XuluGUIMessages.approveFileOverwrite(this,file.getName(),true);
          if ( answer.equals( XuluGUIMessages.FileOverrideOption.CANCEL ) )
            return;
          if ( answer.equals( XuluGUIMessages.FileOverrideOption.OVERWRITE_ALL ) )
            break;
          }
      }
      // Wenn alle Dateien zum Ueberschreiben bestaetigt wurden, dann die
      // Dateien loeschen
      for ( ResourceProvider rp : XuluModellingPlatform.RESOURCE_BUNDLES ) {
        File file = ResourceProvider.getPropertyFile(rp,newLocale);
        if ( file.exists() )
          file.delete();
      }
      // Dateien neu erstellen (mit APPEND-Funktion, damit erweiterte
      // Bundles nicht direkt wieder ueberschrieben werden)
      for ( ResourceProvider rp : XuluModellingPlatform.RESOURCE_BUNDLES )
        ResourceProvider.createPropertyFile(rp, newLocale, true);

      // Tabelle aktuelisieren
      ResourceBundle.clearCache();
      ((AbstractTableModel)bundleTable.getModel()).fireTableDataChanged();
      // Meldung anzeigen
      XuluGUIMessages.showInfoMessage(this,XULUGUI_RES.getString("LangFrame.FinishMess"));
    } catch (Exception err) {
      XuluGUIMessages.showError(this,err);
    }
  }


  /**
   * Initiiert die Aktion der Buttons.
   */
  public void actionPerformed(ActionEvent e) {
    if (e.getSource() == cancelButton)
      cancel();
    if (e.getSource() == newLangButton)
      createNewLanguage();
  }

  /**
   * Stellt ein Tabellen-Modell fuer die Sprachpakete dar.
   * @author <a href="mailto:Martin.Schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  private class BundleTableModel extends AbstractTableModel {
    public BundleTableModel() {
      super();
      fireTableDataChanged();
    }


    public int getColumnCount() {
      return 4;
    }

    public String getColumnName(int col) {
      switch ( col ) {
        case 0: return XULUGUI_RES.getString("LangFrame.Bundle");
        case 1: return XULUGUI_RES.getString("LangFrame.ExtentionOf");
        case 2: return XULUGUI_RES.getString("LangFrame.RootLang");
        case 3: return XULUGUI_RES.getString("LangFrame.AdditionalLang");
      }
      return null;
    }

    public int getRowCount() {
      return XuluModellingPlatform.RESOURCE_BUNDLES.size();
    }

    public Object getValueAt(int row, int col) {
      ResourceProvider rp = XuluModellingPlatform.RESOURCE_BUNDLES.elementAt(row);
      switch ( col ) {
        // Name des Bundles
        case 0: return rp.getBundleName();
        // Name des Original-Bundles (falls es existiert)
        case 1: String defName    = rp.getDefaultBundleName();
                String bundleName = rp.getBundleName();
                return !defName.equals(bundleName) ? defName : "";
        // Root-Sprache (Fall-Back-Locale)
        case 2: return rp.getRootLocale().getDisplayLanguage()+ " (" + rp.getRootLocale().toString() + ")";
        // weitere zur Verfuegung stehende Sprachen (durch Komma getrennt)
        case 3: return getLanguageString( rp.getAvailableLocales(false) );
      }
      return null;
    }

    private String getLanguageString(Set<Locale> locales) {
      String langStr = "";
      for (Locale l : locales) {
        if ( !langStr.equals("") )
          langStr += ", ";
        langStr += l.getDisplayLanguage() + " (" + l.toString() + ")";
      }
      return langStr;
    }

    public void fireTableDataChanged() {
      XuluModellingPlatform.RESOURCE_BUNDLES.sort();
      super.fireTableDataChanged();
    }
  }
}
