/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.border.*;
import schmitzm.swing.ExceptionDialog;
import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.XuluComponent;

/**
 * Diese Klasse stellt einen Info-Dialog zur Xulu-Modelling-Platform dar,
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class XuluMainFrameInfo extends JDialog implements ActionListener, XuluComponent {
  private boolean disposed     = false;

  private JPanel  infoPane     = new JPanel();
  private JPanel  controlPanel = new JPanel();
  private JPanel  iconPanel    = new JPanel();
  private JPanel  textPanel    = new JPanel();
  private JButton okButton     = new JButton();
  private JLabel  iconLabel    = new JLabel();

  // Inhalt des Text-Bereichs
  private String  product      = "Xulu Modelling Platform";
  private String  version      = "Version "+VERSION+", "+VERSION_DATE;
  private String  author       = "Developed by Martin Schmitz";
  private String  organisation = "University of Bonn/Germany";
  private String  contact      = "martin.schmitz@koeln.de";
  private String  copyright    = "Copyright (c) 2007";

  // Definiert die Reihenfolge der Textzeilen
  private String[] textLines = new String[] {
      product,
      version,
      author,
      organisation,
      contact,
      copyright
  };

  /**
   * Erzeugt einen neuen Info-Dialog.
   * @param parent aufrufendes Fenster
   */
  public XuluMainFrameInfo(Frame parent, boolean modal) {
    super(parent,modal);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try {
      jbInit();
    }
    catch(Exception e) {
      ExceptionDialog.show(e);
    }
  }

  /**
   * Initialisiert die GUI.
   */
  private void jbInit() throws Exception  {
    //imageLabel.setIcon(new ImageIcon(XuluMainFrame_Infodialog.class.getResource("[Ihre Grafik]")));
    this.setTitle(XULUGUI_RES.getString("Menu_Help_Info"));
    this.getContentPane().setLayout(new BorderLayout());
    this.getContentPane().add(controlPanel, BorderLayout.SOUTH);
    this.getContentPane().add(infoPane, BorderLayout.NORTH);
    this.setResizable(false);

    // oberes Pane, in dem Icon und Text angezeigt werden
    infoPane.setLayout(new BorderLayout());
    infoPane.add(iconPanel, BorderLayout.WEST);
    infoPane.add(textPanel, BorderLayout.CENTER);

    iconPanel.setLayout(new GridBagLayout());
    iconPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
    iconLabel.setIcon( this.XULU_INFOICON );
    iconPanel.add(iconLabel, new GridBagConstraints(0,0,1,1,1,1,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(0,0,0,0),0,0));
//    iconPanel.add(createStyledLabel(), new GridBagConstraints(0,1,1,1,0,0,GridBagConstraints.NORTH,GridBagConstraints.HORIZONTAL,new Insets(0,5,0,0),0,0));

    textPanel.setLayout(new GridLayout(textLines.length,1));
    textPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
    for (int i=0; i<textLines.length; i++) {
      JLabel label = new JLabel(textLines[i]);
      // die ersten beiden Zeile erscheint FETT
      if ( i<2 )
        label.setFont( label.getFont().deriveFont(Font.BOLD) );
      textPanel.add(label, null);
    }

    // unteres Pane, in dem sich der Kontroll-Button befindet
    controlPanel.setLayout(new FlowLayout());
    okButton.setText(SWING_RES.getString("Close"));
    okButton.addActionListener(this);
    controlPanel.add(okButton, null);

    this.pack();
  }

  private Component createStyledLabel() {
    JTextPane textPane = new JTextPane();
    StyledDocument doc = textPane.getStyledDocument();
    textPane.setEditable(false);
    textPane.setBackground( iconPanel.getBackground() );

    // Default-Style erstellen
    Style def     = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);
    Style regular = doc.addStyle("regular", def);
    StyleConstants.setFontFamily(def, "SansSerif");
    // kursiven Style ableiten
    Style s = doc.addStyle("italic", regular);
    StyleConstants.setItalic(s, true);
    // fett-kursiven Style ableiten
    s = doc.addStyle("bolditalic", regular);
    StyleConstants.setItalic(s, true);
    StyleConstants.setBold(s, true);

    // formatierten Text zusammensetzen
    try {
      doc.insertString(doc.getLength(), "M", doc.getStyle("bolditalic"));
      doc.insertString(doc.getLength(), "odellers, ", doc.getStyle("italic"));
      doc.insertString(doc.getLength(), "A", doc.getStyle("bolditalic"));
      doc.insertString(doc.getLength(), "vail ", doc.getStyle("italic"));
      doc.insertString(doc.getLength(), "X", doc.getStyle("bolditalic"));
      doc.insertString(doc.getLength(), "ulu !!", doc.getStyle("italic"));
    } catch (Exception err) {
    }
    return textPane;
  }

  /**
   * Schliesst das Fenster, wenn es sich um ein <code>WindowEvent.WINDOW_CLOSING<code>-Event
   * handelt.
   */
  protected void processWindowEvent(WindowEvent e) {
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      cancel();
    }
    super.processWindowEvent(e);
  }

  /**
   * Zerstoert das Fenster. Ruft <code>super.dispose()</code> auf.
   */
  public void dispose() {
    disposed = true;
    super.dispose();
  }

  /**
   * Prueft, ob das Fenster zerstoert ist.
   */
  public boolean isDisposed() {
    return disposed;
  }

  /**
   * Schliesst das Fenster.
   */
  private void cancel() {
    dispose();
  }

  /**
   * Aktion des OK-Buttons. Schlisst das Fenster.
   */
  public void actionPerformed(ActionEvent e) {
    if (e.getSource() == okButton) {
      cancel();
    }
  }
}
