/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.io;

import edu.bonn.xulu.appl.XuluRegistry;
// nur fuer Doku
import edu.bonn.xulu.io.AbstractFactory;
/**
 * Dieses Interface bildet den Obertyp fuer alle Import-Factorys. Diese haben
 * die Aufgabe, ein Objekt aus einer Import-Quelle (z.B. einer Datei) zu
 * erzeugen. Das Format, das dieser Stream haben muss, haengt von der jeweiligen
 * Factory-Implementierung ab.
 * <b>Bemerke:</b><br>
 * Die (entscheidende) Methode {@link #importObject(Object,XuluRegistry)} erhaelt
 * als Parameter fuer die Eingabe-Quelle ein <code>Object</code> (und nicht
 * z.B. einen <code>File</code> oder <code>OutputStream</code>), um die
 * Moeglichkeiten fuer die Implementierungen flexibler zu gestalten.
 * Was fuer ein Eingabe-Objekt uebergeben wird, haengt von der aufrufenden
 * Anwendung ab. Die Methode
 * {@link AbstractFactory#checkImportSourceObject(ImportFactory,Object,Class,boolean)}
 * bietet eine Hilfsmethode, um zu pruefen, ob die richtige Eingabe-Klasse
 * uebergeben wurde.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public interface ImportFactory extends Factory {
  /**
   * Liefert den Objekt-Typ, der von der Factory erzeugt wird.
   */
  public Class getImportType();

  /**
   * Liefert den Objekt-Typ, den die Factory als Quelle zum Importieren
   * benoetigt (z.B. {@link java.io.File} oder {@link java.io.InputStream}).<br>
   * <b>Bemerke:</b><br>
   * An dieser Stelle koennen auch Arrays angegeben werden,
   * wenn die Factory ein Objekt z.B. aus mehreren Dateien erzeugt!
   */
  public Class getImportSourceType();

  /**
   * Importiert ein Objekt aus einen Eingabe-Stream.
   * @param in  Import-Quelle (z.B. Stream oder Datei), aus dem das Objekt gelesen wird
   * @param reg bietet Zugriff auf andere registrierte Factories (um z.B. eine
   *            Standard-Instanz zu erzeugen, in die das Objekt importiert wird)
   * @exception java.lang.Exception wenn beim Importieren ein Fehler auftritt
   */
  public Object importObject(Object in, XuluRegistry reg) throws Exception;
}
