/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.model;

import schmitzm.swing.CaptionsChangeable;
import edu.bonn.xulu.model.ModelResource;
import edu.bonn.xulu.data.XuluDataException;

/**
 * Der ModelContentManager spezifiziert die Datenbasis, die ein Modell fuer
 * seine Ausfuehrung benoetigt. Diese besteht aus einer Reihe von
 * {@link ModelResource}-Objekten, welche vor allem den Datentyp des jeweils
 * benoetigten Objekts festlegen.<br>
 * Die Implementierung ist eng mit dem jeweiligen Modell verbunden und in der
 * Regel von Modell zu Modell unterschiedlich (aufgrund der unterschiedlichen
 * Anforderungen). Deshalb wird der ModelContentManager am besten als innere
 * Klasse des jeweiligen Modells implementiert.
 * @see edu.bonn.xulu.model.XuluModel
 * @see edu.bonn.xulu.model.AbstractModelContentManager
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public interface ModelContentManager extends CaptionsChangeable {
  /**
   * Liefert eine Ressource, die ein Modell benoetigt.
   * @param idx Nummer der Ressource
   */
  public ModelResource getResource(int idx);

  /**
   * Liefert die Anzahl benoetigter Ressourcen.
   */
  public int getResourceCount();

  /**
   * Ueberprueft alle Resourcen auf Korrektheit.
   * @see ModelResource#check()
   * @return <code>false</code> falls eine Ressource nicht korret gefuellt ist
   */
  public boolean check();

  /**
   * Ueberprueft alle Ressourcen auf Korrektheit.
   * @exception XuluDataException falls eine Ressource nicht korret gefuellt ist
   * @see ModelResource#checkAndError()
   */
  public void checkAndError() throws XuluDataException;
}
