/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.model.event;

import edu.bonn.xulu.model.XuluModel;

/**
 * Diese Klasse repraesentiert ein Ereignis, das fuer einen Modell-Schnitt
 * ausgeloest wird.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class AbstractStepModelEvent extends AbstractModelEvent {
  /** Speichert den Zeitschritt fuer den das Ereignis ausgeloest wurde. */
  protected int modelStep = 0;

  /**
   * Erzeugt ein neues Ereignis.
   * @param model  ausloesendes Modell
   * @param stepNo Modell-Schritt des Ereignisses
   */
  public AbstractStepModelEvent(XuluModel model, int stepNo) {
    super(model);
    this.modelStep = stepNo;
  }

  /**
   * Liefert den Modell-Schritt fuer den das Ereignis ausgeloest wurde.
   */
  public int getStepNo() {
    return this.modelStep;
  }
}
