/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.model.event;

import edu.bonn.xulu.model.StepModel;
import edu.bonn.xulu.model.event.ModelAdapter;
import edu.bonn.xulu.model.XuluModel;

/**
 * Erweitert den normalen {@link ModelAdapter} um die Ereignisse...
 * <ul>
 *   <li>{@link ModelStepStartedEvent}</li>
 *   <li>{@link ModelStepFinishedEvent}</li>
 * </ul>
 * @see ModelAdapter
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public abstract class StepModelAdapter extends ModelAdapter {
    /**
     * Wird fuer jedes Modell-Ereignis ausgeloest. Je nach Event-Typ wird die
     * entsprechende Adapter-Methode aufgerufen.
     * @param e Modell-Ereignis
     * @see #modelInitialised(XuluModel)
     * @see #modelStarted(XuluModel)
     * @see #modelStopped(XuluModel)
     * @see #modelDisposed(XuluModel)
     * @see #modelStepStarted(StepModel,int)
     * @see #modelStepFinished(StepModel,int)
     * @see #performOtherEvent(ModelEvent e)
     */
    public void performModelEvent(ModelEvent e) {
      if ( e instanceof ModelStepStartedEvent )
        this.modelStepStarted(((ModelStepStartedEvent)e).getSourceModel(),((ModelStepStartedEvent)e).getStepNo());
      else if ( e instanceof ModelStepFinishedEvent )
        this.modelStepFinished(((ModelStepFinishedEvent)e).getSourceModel(),((ModelStepFinishedEvent)e).getStepNo());
      else
        super.performModelEvent(e);
    }

  /**
   * Wird ausgeloest, bevor <code>Steppable</code>-Modell einen
   * Modellierungsschritt beginnt
   * @param model Xulu-Modell das des Schritt ausfuehrt
   * @param stepNo Nummer des gestarteten Modellierungsschritts
   */
  public void modelStepStarted(XuluModel model, int stepNo) {
  }

  /**
   * Wird ausgeloest, nachdem <code>Steppable</code>-Modell einen
   * Modellierungsschritt beendet hat
   * @param model Xulu-Modell das des Schritt ausgefuehrt hat
   * @param stepNo Nummer des beendeten Modellierungsschritts
   */
  public void modelStepFinished(XuluModel model, int stepNo) {
  }
}
