/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.appl;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;

import java.util.Vector;
import java.util.StringTokenizer;

import schmitzm.io.IOUtil;
import schmitzm.lang.LangUtil;

import edu.bonn.xulu.data.XuluObject;
import edu.bonn.xulu.data.XuluDataException;
import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.AbstractCommandInterpreter;
import edu.bonn.xulu.appl.ScriptablePlugin;
import edu.bonn.xulu.appl.XuluPlugin;
import edu.bonn.xulu.gui.XuluGUIUtil;
import edu.bonn.xulu.io.IODefinition;
import edu.bonn.xulu.io.InstantiationFactory;
import edu.bonn.xulu.io.ImportFactory;
import edu.bonn.xulu.io.ExportFactory;
import edu.bonn.xulu.io.TypeMappingException;
import schmitzm.lang.OperationCanceledException;
import schmitzm.lang.AlreadyHandledException;

/**
 * Dieser Interpreter fuehrt IO-Befehle des Datenpools aus. Einleitender Tag und
 * die einzelnen Parameter stehen jeweils in eigenen Datei-Zeilen. Leerzeilen
 * und {@linkplain IOUtil#isCommentLine(String) Kommentarteilen} werden ignoriert.
 * <table align=center border=2 cellpadding=5>
 *    <tr><th>Befehl</th>
 *        <th>einleitende Tag</th>
 *        <th>Parameter</th>
 *        </tr>
 *    <tr><td valign="top">Xulu-Objekt neu erzeugen</td>
 *        <td valign="top"><code>[Create]</code></td>
 *        <td valign="top"><code>type = ...</code><br>
 *            Klasse des Datentyps (passende Factory wird aus der Registry ermittelt)<br>
 *            <code>name = ...</code><br>
 *            Name fuer das Objekt<br>
 *            <code>sample = ...</code> (optional)<br>
 *            Name des Objekts, das als Vorlage dient<br>
 *            <code>dialog = true|false</code> (optional)<br>
 *            Flag, ob ein Dialog von der Factory angezeigt werden darf</td>
 *        </tr>
 *    <tr><td valign="top">Xulu-Objekt aus Datei importieren</td>
 *        <td valign="top"><code>[Import]</code></td>
 *        <td valign="top"><code>factory = ...</code><br>
 *            Bezeichnung (oder Klasse) einer registrierten Import-Factory<br>
 *            <code>name = ...</code><br>
 *            Name fuer das Objekt<br>
 *            <code>source = ...</code><br>
 *            Datei aus der das Objekt gelesen wird (Parameter kann mehrfach angegeben werden!)</td>
 *        </tr>
 *    <tr><td valign="top">Struktur eines Xulu-Objekt kopieren</td>
 *        <td valign="top"><code>[CopyStructure]</code></td>
 *        <td valign="top"><code>source = ...</code><br>
 *            Name des Objekts, das kopiert werden soll<br>
 *            <code>dest = ...</code><br>
 *            Name fuer das neue Objekt<br>
 *            <code>dialog = true|false</code> (optional)<br>
 *            Flag, ob ein Dialog von der Factory angezeigt werden darf</td>
 *        </tr>
 *    <tr><td valign="top">Xulu-Objekt umbenennen</td>
 *        <td valign="top"><code>[Rename]</code></td>
 *        <td valign="top"><code>source = ...</code><br>
 *            Name des Objekts, das umbenannt werden soll<br>
 *            <code>dest = ...</code><br>
 *            Neuer Name fuer das Objekt</td>
 *        </tr>
 *
 *          <tr><td valign="top">Neues Model erzeugen</td>
 *        <td valign="top"><code>[CreateModel]</code></td>
 *        <td valign="top"><code>modelClass = ...</code><br>
 *            Klasse des Modells, dass geladen werden soll<br>
 *            <code>name = ...</code><br>
 *            Name fuer das Modell</td>
 *        </tr>
 *
 *          <tr><td valign="top">Modell-Ressource-Mapping laden</td>
 *        <td valign="top"><code>[LoadMapping]</code></td>
 *        <td valign="top"><code>path = ...</code><br>
 *            Pfad zum Skript, dass das Mapping durchf�hrt<br>
 *            <code>name = ...</code><br>
 *            Das Mapping wird f�r alle Modelle mit diesem Namen durchgef�hrt</td>
 *        </tr>
 * </table>
 * <br>
 * Darueber hinaus kann zwischen den einzelnen Befehlen ueber das Kommando
 * <code>[BaseDir]</code> (gefolgt von eine Zeile mit Pfad-Angabe) das Basis-Verzeichnis
 * gewechselt werden, relativ zu dem Pfade beim Importieren interpretiert werden.
 * Dies verhindert lange und unuebersichtliche Datei-Angaben.
 * </ul>
 * @see edu.bonn.xulu.plugin.appl.ModelResourceMappingScriptInterpreter_Basic
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class DataScriptInterpreter_Basic extends AbstractCommandInterpreter {
  // speichert die eingelesenen Befehle
  private Vector commands = new Vector();
  // speichert die aktuelle Zeilennummer
  private int     lineNo  = 0;
  // speichert den aktuell eingelesenen Befehl
  private Command command = null;
  // speichert, ob Einfuege-Operationen bei einem Duplikat-Fehler autom.
  // uebergangen werden sollen
  private static boolean autoSkipDupl = false;

  private void showError(String errString) {
    System.out.println(errString);
    errorCount++;
  }

  /**
   * Fuehrt ein Script aus.
   * @param in   Eingabe-Stream in dem das Skript zu finden ist.
   * @param appl Applikation, in der das Skript auszufuehren ist
   */
  public void execute(InputStream in, XuluModellingPlatform appl) {
    // Waehrend der Einfuegung von mehreren Objekten kann der Anwender im
    // Konfliktfall auswaehlen, wie automatisch mit folgenden Konflikten
    // umgegangen werden soll
    // --> Entsprechend werden Einstellungen am Datenpool vorgenommen
    // --> Nach der kollektiven Einfuegung sollen die alten Einstellungen
    //     wieder hergestellt werden
    boolean oldOverwriteOption = appl.getDataPool().getAutoOverwrideDuplicateNames();
    boolean oldDuplicateOption = appl.getDataPool().getDuplicateNamesAllowed();
    // Bei Duplikat-Fehlern soll die Operation nicht autom.
    // uebergangen werden, sondern ein Dialog erscheinen
    this.autoSkipDupl = false;
    // Operationen ausfuehren
    super.execute(in,appl);
    // alte Datenpool-Einstellungen wieder herstellen
    appl.getDataPool().setAutoOverwriteDuplicateNames(oldOverwriteOption);
    appl.getDataPool().setDuplicateNamesAllowed(oldDuplicateOption);
  }

  /**
   * Interpretiert einen Eingabestream, wie oben beschrieben und erzeugt
   * daraus einzelne Kommandos.
   * @param in Stream auf das Skript
   */
  protected Command[] readCommands(InputStream in) {
    commands.clear(); // alte Befehlsliste loeschen
    command = null;   // letzten eingelesenene Befehl initialisieren
    // Stream parsen
    BufferedReader input = new BufferedReader( new InputStreamReader(in) );
    String  line    = null;
    File    baseDir = appl.getWorkingDirectory(); //new File(".");
    try {
      // Zeilenweise einlesen
      for(lineNo=1; (line = input.readLine()) != null; lineNo++) {
        line = line.trim();
        // Leerzeilen und Kommentare ignorieren
        if ( line.equals("") || IOUtil.isCommentLine(line) )
          continue;

        // Befehl-Zeile interpretieren
        if ( line.startsWith("[") && line.endsWith("]") ) {
          // alten Befehl abschliessen
          checkAndAddCommand("line "+lineNo);
          // neuen Befehl erzeugen
          command = null;
          if ( line.equalsIgnoreCase("[BaseDir]") )
            baseDir = new File(input.readLine());
          if ( line.equalsIgnoreCase("[Import]") )
            command = new Command_ImportFromFile();
          if ( line.equalsIgnoreCase("[Rename]") )
            command = new Command_Rename();
          if ( line.equalsIgnoreCase("[Create]") )
            command = new Command_New();
          if ( line.equalsIgnoreCase("[CopyStructure]") )
            command = new Command_CopyStructure();
          if ( line.equalsIgnoreCase("[CreateModel]") )
              command = new Command_CreateModel();
          if ( line.equalsIgnoreCase("[LoadMapping]") )
              command = new Command_LoadMapping();
          if ( line.equalsIgnoreCase("[Plugin]") )
              command = new Command_Plugin();
          // zur naechsten Zeile uebergehen
          continue;
        }

        // Eine Parameter-Zeile kann nur interpretiert werden,
        // wenn vorher ein Befehl definiert wurde
        if ( command == null )
          showError("error line " + lineNo + " > Command in outside section!");

        // Parameter-Zeile intepretieren
        try {
          if ( command instanceof Command_New )
            parseLineForCommand_New((Command_New)command,line);
          else if ( command instanceof Command_Rename )
            parseLineForCommand_Rename((Command_Rename)command,line);
          else if ( command instanceof Command_CopyStructure )
            parseLineForCommand_CopyStructure((Command_CopyStructure)command,line);
          else if ( command instanceof Command_ImportFromFile )
            parseLineForCommand_ImportFromFile((Command_ImportFromFile)command,line,baseDir);
          else if ( command instanceof Command_CreateModel )
              parseLineForCommand_CreateModel((Command_CreateModel)command,line);
          else if ( command instanceof Command_LoadMapping )
              parseLineForCommand_LoadMapping((Command_LoadMapping)command,line,baseDir);
          else if ( command instanceof Command_Plugin)
              parseLineForCommand_Plugin((Command_Plugin)command,line);
        } catch (Exception err) {
          showError("error line "+lineNo+" > "+err.getMessage());
        }
      }
      // letzten Befehl abschliessen
      checkAndAddCommand("at end of file");

      // Stream schliessen
      in.close();
    } catch ( Exception err ) {
      showError("Unknown error while parsing line "+lineNo);
//      err.printStackTrace();
    }
    return (Command[])commands.toArray(new Command[0]);
  }

  /**
   * Prueft ein Command auf Korrektheit und fuegt es der Kommandoliste hinzu.
   * @param lineDesc Bezeichnung der aktuell eingelesenen Zeile (Hinweis fuer
   *                 etwaige Fehlermeldung)
   */
  private void checkAndAddCommand(String lineDesc) {
    // alten Befehl checken und der Liste hinzufuegen
    if (command != null) {
      try {
        command.check();
        commands.add(command);
      }
      catch (Exception err) {
        showError("error "+lineDesc+" > Creation of last command failed: " + err.getMessage());
      }
    }
  }

  /**
   * Teilt eine Parameter-Zeile ein in Tag (vor Gleichheitszeichen) und
   * Wert (alles nach Gleichheitszeichen). Liefert in Array-Position 0 den
   * Tag-Namen und in Position 1 den Wert.
   * @param line Parameter-Zeile
   * @return 2-dim. Array
   */
  private String[] parseLineInTagAndValue(String line) {
    int pos = line.indexOf("=");
    if ( pos < 0 )
      throw new UnsupportedOperationException("Illegal line format");
    return new String[] {
        line.substring(0,pos).trim(),
        line.substring(pos+1).trim()
    };
  }

  /**
   * Interpretiert eine Zeile als Parameter fuer den Befehl
   * "Neues Objekt".
   * @param command  aktueller Befehl
   * @param line     Zeile
   */
  private void parseLineForCommand_New(Command_New command, String line) {
    String[] tok = parseLineInTagAndValue(line);
    if ( tok[0].equalsIgnoreCase("type") )
      try {
        command.objectType = Class.forName(tok[1]);
      } catch (ClassNotFoundException err) {
        throw new UnsupportedOperationException("class of type '"+tok[1]+"' not found");
      }
    else if ( tok[0].equalsIgnoreCase("sample") )
      command.sampleName = tok[1];
    else if ( tok[0].equalsIgnoreCase("name") )
      command.objectName = tok[1];
    else if ( tok[0].equalsIgnoreCase("dialog") )
      command.dialog = new Boolean(tok[1]);
    else
      throw new UnsupportedOperationException("Unknown tag '"+tok[0]+"'");
  }

  /**
   * Interpretiert eine Zeile als Parameter fuer den Befehl
   * "Objekt umbenennen".
   * @param command  aktueller Befehl
   * @param line     Zeile
   */
  private void parseLineForCommand_Rename(Command_Rename command, String line) {
    String[] tok = parseLineInTagAndValue(line);
    if ( tok[0].equalsIgnoreCase("source") )
      command.objectName = tok[1];
    else if ( tok[0].equalsIgnoreCase("dest") )
      command.newObjectName = tok[1];
    else
      throw new UnsupportedOperationException("Unknown tag '"+tok[0]+"'");
  }

  /**
   * Interpretiert eine Zeile als Parameter fuer den Befehl
   * "Objektstruktur kopieren".
   * @param command  aktueller Befehl
   * @param line     Zeile
   */
  private void parseLineForCommand_CopyStructure(Command_CopyStructure command, String line) {
    String[] tok = parseLineInTagAndValue(line);
    if ( tok[0].equalsIgnoreCase("source") )
      command.sampleName = tok[1];
    else if ( tok[0].equalsIgnoreCase("dest") )
      command.objectName = tok[1];
    else if ( tok[0].equalsIgnoreCase("dialog") )
      command.dialog = new Boolean(tok[1]);
    else
      throw new UnsupportedOperationException("Unknown tag '"+tok[0]+"'");
  }

  /**
   * Interpretiert eine Zeile als Parameter fuer den Befehl
   * "Objekt aus Datei importieren".
   * @param command  aktueller Befehl
   * @param line     Zeile
   * @param baseDir  aktuell gesetztes Basisverzeichnis fuer die Eingabedateien
   */
  private void parseLineForCommand_ImportFromFile(Command_ImportFromFile command, String line, File baseDir) {
    String[] tok = parseLineInTagAndValue(line);
    if ( tok[0].equalsIgnoreCase("factory") )
      command.facName = tok[1];
    else if ( tok[0].equalsIgnoreCase("name") )
      command.objectName = tok[1];
    else if ( tok[0].equalsIgnoreCase("source") )
      command.sourceVec.add( IOUtil.createRelativeFile(baseDir, tok[1]) );
    else
      throw new UnsupportedOperationException("Unknown tag '"+tok[0]+"'");
  }

  /**
   * Interpretiert eine Zeile als Parameter fuer den Befehl
   * "Erzeuge neues Modell".
   * @param command  aktueller Befehl
   * @param line     Zeile
   */
  private void parseLineForCommand_CreateModel(Command_CreateModel command, String line) {
    String[] tok = parseLineInTagAndValue(line);
    if ( tok[0].equalsIgnoreCase("modelClass") )
      command.modelClass = tok[1];
    else if ( tok[0].equalsIgnoreCase("name") )
      command.modelName = tok[1];
    else
      throw new UnsupportedOperationException("Unknown tag '"+tok[0]+"'");
  }


  /**
   * Interpretiert eine Zeile als Parameter fuer den Befehl
   * "Lade ein Modell - Ressource - Mapping".
   * @param command  aktueller Befehl
   * @param line     Zeile
   * @param baseDir  aktuell gesetztes Basisverzeichnis fuer die Eingabedateien
   */
  private void parseLineForCommand_LoadMapping(Command_LoadMapping command, String line, File baseDir) {
    String[] tok = parseLineInTagAndValue(line);
    if (tok[0].equalsIgnoreCase("name"))
      command.modelName = tok[1];
    else if (tok[0].equalsIgnoreCase("path"))
      command.scriptPath = IOUtil.createRelativeFile(baseDir, tok[1]).
          getAbsolutePath();
    else
      throw new UnsupportedOperationException("Unknown tag '" + tok[0] + "'");
  }
  /**
   * Interprets a line as command for a plugin. The "pluginname"-tag should
   * the fully qualified name of an existing and loaded {@link XuluPlugin}. All other lines
   * will be forwarded to the specified plugin, which is responsible for
   * correct interpretation.
   *
   * @author Dominik Appl
   * @param command actual command
   * @param line line to interpret by the the plugin
   *
   */
  private void parseLineForCommand_Plugin(Command_Plugin command, String line) {
    String[] tok = parseLineInTagAndValue(line);
    if (tok[0].equalsIgnoreCase("pluginname"))
      command.pluginName = tok[1];
    else
      command.pluginCommands.add(line);
  }

  /**
   * Dieser Befehl erzeugt ein neues Xulu-Objekts im Datenpool.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  protected class Command_New implements Command {
    private Class   objectType = null;
    private boolean dialog     = false;
    private String  objectName = null;
    private String  sampleName = null;

    private Command_New() {
    }

    /**
     * Erzeugt einen neuen Befehl.
     * @param objectType Typ des zu erzeugenden Objekts
     * @param objName    Bezeichnung des Objekts im Datenpool
     * @param sampleName Name des Objekts, welches als (strukturelle) Vorlage
     *                   fuer das neue Objekt verwendet wird (kann {@code null} sein)
     * @param dialog     bestimmt, ob ein Anwender-Dialog bei der Erzeugung
     *                   des Objekts angezeigt werden darf
     */
    public Command_New(Class objectType, String objName, String sampleName, boolean dialog) {
      this.objectType  = objectType;
      this.objectName  = objName;
      this.dialog      = dialog;
      this.sampleName  = sampleName;
    }

    /**
     * Liefert eine Beschreibung des Kommandos.
     */
    public String getDescription() {
      return "Create new object '"+objectName+"' ("+objectType.getSimpleName()+")";
    }

    /**
     * Fuehrt den Befehl aus.
     * @param appl Applikation unter der der Befehl ausgefuehrt wird
     * @exception TypeMappingException falls keine InstantiationFactory fuer den
     *            Datentyp registriert ist.
     * @exception XuluDataException die gefundenen Factory kein XuluObject instanziiert
     * @exception Exception falls die Instanziierung des Objekts fehl schlaegt
     */
    public void execute(XuluModellingPlatform appl) throws Exception, TypeMappingException, XuluDataException {
      XuluObject sample = (sampleName != null) ? findObjectInDataPool(sampleName,appl) : null;
      createNewObject(objectType,objectName,sample,dialog,appl);
    }

    /**
     * Prueft, ob alle Parameter korrekt gesetzt sind. Ggf. wird eine
     * Exception geworfen.
     */
    public void check() {
      if ( objectType == null )
        throw new UnsupportedOperationException("No object type specified!");
      if ( objectName == null || objectName.equals("") )
        throw new UnsupportedOperationException("No object name specified!");
    }
  }

  /**
   * Dieser Befehl importiert ein neues Xulu-Objekts in den Datenpool.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  protected class Command_ImportFromFile implements Command {
    private String   facName    = null;
    private boolean  dialog     = false;
    private String   objectName = null;
    private Vector   sourceVec  = new Vector<File>();

    /**
     * Erzeugt einen neuen Befehl.
     * @param facName    Bezeichnung der zu verwendenden ImportFactory
     * @param source     Eingabe-Dateien
     * @param objName    Bezeichnung des Objekts im Datenpool
     * @param dialog     bestimmt, ob ein Anwender-Dialog bei der Erzeugung
     *                   des Objekts angezeigt werden darf
     */
    public Command_ImportFromFile(String facName, File[] source, String objName, boolean dialog) {
      this.facName     = facName;
      this.objectName  = objName;
      this.dialog      = dialog;
      for (int i=0; i<source.length; i++)
        sourceVec.add(source[i]);
    }

    private Command_ImportFromFile() {
    }

    /**
     * Liefert eine Beschreibung des Kommandos.
     */
    public String getDescription() {
      return "Import object '"+objectName+"' by "+facName;
    }

    /**
     * Fuehrt den Befehl aus.
     * @param appl Applikation unter der der Befehl ausgefuehrt wird
     * @exception TypeMappingException falls keine InstantiationFactory fuer den
     *            Datentyp registriert ist.
     * @exception XuluDataException die gefundenen Factory kein XuluObject instanziiert
     * @exception Exception falls die Instanziierung des Objekts fehl schlaegt
     */
    public void execute(XuluModellingPlatform appl) throws Exception, TypeMappingException, XuluDataException {
      importObjectFromFile(facName,(File[])sourceVec.toArray(new File[0]),objectName,dialog,appl);
    }

    /**
     * Prueft, ob alle Parameter korrekt gesetzt sind. Ggf. wird eine
     * Exception geworfen.
     */
    public void check() {
      if ( facName == null || facName.equals("") )
        throw new UnsupportedOperationException("No factory specified!");
      if ( objectName == null || objectName.equals("") )
        throw new UnsupportedOperationException("No object name specified!");
      if ( sourceVec == null || sourceVec.size() == 0 )
        throw new UnsupportedOperationException("No source files specified!");
    }
  }

  /**
   * Dieser Befehl erzeugt ein neues Xulu-Objekts im Datenpool auf Basis eines
   * bestehenden Objekts.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  protected class Command_CopyStructure implements Command {
    private boolean dialog     = false;
    private String  sampleName = null;
    private String  objectName = null;

    private Command_CopyStructure() {
    }

    /**
     * Erzeugt einen neuen Befehl.
     * @param sampleName  Name des zu kopierenden Objekts
     * @param objectName  Bezeichnung des neuen Objekts im Datenpool
     * @param dialog      bestimmt, ob ein Anwender-Dialog beim Kopieren
     *                    des Objekts angezeigt werden darf
     */
    public Command_CopyStructure(String sampleName, String objectName, boolean dialog) {
      this.sampleName = sampleName;
      this.objectName = objectName;
      this.dialog     = dialog;
    }

    /**
     * Liefert eine Beschreibung des Kommandos.
     */
    public String getDescription() {
      return "Create new object '"+objectName+"' from '"+sampleName+"'";
    }

    /**
     * Fuehrt den Befehl aus.
     * @param appl Applikation unter der der Befehl ausgefuehrt wird
     * @exception TypeMappingException falls keine InstantiationFactory fuer den
     *            Datentyp registriert ist.
     * @exception XuluDataException die gefundenen Factory kein XuluObject instanziiert
     * @exception Exception falls die Instanziierung des Objekts fehl schlaegt
     */
    public void execute(XuluModellingPlatform appl) throws Exception, TypeMappingException, XuluDataException {
      XuluObject sample = findObjectInDataPool(sampleName,appl);
      createNewObject(sample.getClass(),objectName,sample,dialog,appl);
    }

    /**
     * Prueft, ob alle Parameter korrekt gesetzt sind. Ggf. wird eine
     * Exception geworfen.
     */
    public void check() {
      if ( sampleName == null || sampleName.equals("") )
        throw new UnsupportedOperationException("No sample object specified!");
      if ( objectName == null || objectName.equals("") )
        throw new UnsupportedOperationException("No object name specified!");
    }
  }

  /**
   * Dieser Befehl aendert den Namen eines Xulu-Objekts im Datenpool.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  protected class Command_Rename implements Command {
    private String  objectName = null;
    private String  newObjectName = null;

    private Command_Rename() {
    }

    /**
     * Erzeugt einen neuen Befehl.
     * @param objectName    Name des umzubenennenden Objekts
     * @param newObjectName neuer Name
     */
    public Command_Rename(String objectName, String newObjectName) {
      this.objectName    = objectName;
      this.newObjectName = newObjectName;
    }

    /**
     * Liefert eine Beschreibung des Kommandos.
     */
    public String getDescription() {
      return "Rename object '" + objectName + "' to '"+newObjectName+"'";
    }

    /**
     * Fuehrt den Befehl aus.
     * @param appl Applikation unter der der Befehl ausgefuehrt wird
     * @exception XuluDataException falls es kein oder mehr als ein Objekt
     *            mit dem Namen im Datenpool gibt
     */
    public void execute(XuluModellingPlatform appl) throws Exception {
      findObjectInDataPool(objectName,appl).setDescription(newObjectName);
    }

    /**
     * Prueft, ob alle Parameter korrekt gesetzt sind. Ggf. wird eine
     * Exception geworfen.
     */
    public void check() {
      if ( objectName == null || objectName.equals("") )
        throw new UnsupportedOperationException("No object name specified!");
      if ( newObjectName == null || newObjectName.equals("") )
        throw new UnsupportedOperationException("No new name specified!");
    }
  }

  /**
   * Erzeugt ein neues Modell
   * @author Dominik Appl - University of Bonn/Germany
   * @version 1.0
   */
  protected class Command_CreateModel implements Command {
    private String  modelClass = null;
    private String  modelName = null;

    private Command_CreateModel() {
    }

    /**
     * Erzeugt einen neuen Befehl.
     * @param modelClass Klasse des zu ladenen Modells
     * @param modelName Name fuer die neue Modell-Instanz
     */
    public Command_CreateModel(String modelClass, String modelName) {
      this.modelClass = modelClass;
      this.modelName = modelName;
    }

    /**
     * Liefert eine Beschreibung des Kommandos.
     */
    public String getDescription() {
      return "Create Model \"" + modelName + "\" from class " + modelClass;
    }

    /**
     * Fuehrt den Befehl aus.
     * @param appl Applikation unter der der Befehl ausgefuehrt wird
     */
    public void execute(XuluModellingPlatform appl) throws Exception {
      try{
        Class model = LangUtil.loadClass(modelClass,appl.getDynamicClassRootDirectory().toURI().toURL());
        XuluGUIUtil.createNewModel(appl.getMainFrame(), model, modelName, appl);
      } catch (Exception e){
        System.err.println("Unable to create instance of " + modelName);
        e.printStackTrace();
        throw (e);
      }
    }
    /**
     * Prueft, ob alle Parameter korrekt gesetzt sind. Ggf. wird eine
     * Exception geworfen.
     */
    public void check() {
        if ( modelName == null || modelName.equals("") )
            throw new UnsupportedOperationException("No Modelname specified!");
        if ( modelClass == null || modelClass.equals("") )
            throw new UnsupportedOperationException("No Model Classname specified!");

      }
    }

    /**
     * F�hrt mittels eines Skripts ein Mapping von Ressourcen
     * auf ein Modell durch
     * @author Dominik Appl - University of Bonn/Germany
     * @version 1.0
     */
    protected class Command_LoadMapping implements Command {
      private String  scriptPath = null;
      private String  modelName = null;

      private Command_LoadMapping() {
      }

      /**
       * Erzeugt einen neuen Befehl.
       * @param modelName Name des (geladenen) Modells
       * @param scriptPath Pfad zu dem Skript das das Mapping durch f�hrt
       *
       * @see ModelResourceMappingScriptInterpreter_Basic
       */

      public Command_LoadMapping(String scriptPath, String modelName) {
        this.scriptPath = scriptPath;
        this.modelName = modelName;
      }

      /**
       * Liefert eine Beschreibung des Kommandos.
       */
      public String getDescription() {
        return "Loading Mapping for Model :\"" + modelName + "\" from path " + scriptPath;
      }

      /**
       * Fuehrt den Befehl aus.
       * @param appl Applikation unter der der Befehl ausgefuehrt wird
       */
      public void execute(XuluModellingPlatform appl) throws Exception {
        try{
              InputStream in = new FileInputStream(scriptPath);
              new ModelResourceMappingScriptInterpreter_Basic().execute(in ,modelName,appl);
        }
          catch (Exception e){
            System.err.println("Unable to load mapping: " + e.getMessage());
        }
      }

    /**
     * Prueft, ob alle Parameter korrekt gesetzt sind. Ggf. wird eine
     * Exception geworfen.
     */
    public void check() {
        if ( modelName == null || modelName.equals("") )
            throw new UnsupportedOperationException("No Modelname specified!");
        if ( scriptPath == null || scriptPath.equals("") )
            throw new UnsupportedOperationException("No ScriptPath specified!");
      }
    }


    /**
     * Forwards scripting to a method in a plugin
     *
     * @author Dominik Appl - University of Bonn/Germany
     *      */
    protected class Command_Plugin implements Command {
      private String  pluginName = null;
      private Vector pluginCommands = new Vector<String>();

      private Command_Plugin() {
      }

      /**
         * Creates a new plugin command. The command will be forwarded to the
         * specified plugin, which is responsible for interpreting it correctly.
         *
         * @param pluginname
         *            the exact java-name to the class of the plugin
         * @param pluginCommands
         *            Vector of tokenized commands which are forwared to the
         *            plugin. Each command is a String array with position 0
         *            containing the tagname and further positions containing
         *            the values for this tag.
         *
         * @see ModelResourceMappingScriptInterpreter_Basic
         */

      public Command_Plugin(String pluginname, Vector pluginCommands) {
        this.pluginName = pluginname;
        this.pluginCommands = pluginCommands;
      }

      /**
       * returns a description of the command
       */
      public String getDescription() {
        return "Forwarding Commands to plugin :\""  + pluginName + "\"";
      }

      /**
         * Executes the commands
         *
         * @param appl
         *            Application which executes the command
         */
        public void execute(XuluModellingPlatform appl) throws Exception {
            ScriptablePlugin plugin = (ScriptablePlugin) getPluginForName(pluginName);
            if (plugin == null)
                return;
            else
                plugin.executeScript((String[]) pluginCommands.toArray());
        }

      /**
         * Returns a {@link XuluPlugin} for a fully qualified name.
         *
         * @return the plugin, if one was found. Null if no plugin with the
         *         given name is loaded
         */
        public XuluPlugin getPluginForName(String name) {
            XuluPlugin[] plugins = appl.getRegistry().getPlugins();
            for (XuluPlugin xp : plugins)
                if (xp.getName().equals(name))
                   return xp;
            return null;
        }

    /**
     * Checks if the parameters are set correctly
     */
    public void check() {
        if ( pluginName == null || pluginName.equals("") )
            throw new UnsupportedOperationException("No plugin specified!");
        if ( pluginCommands.isEmpty() )
            throw new UnsupportedOperationException("No commands specified!");
        XuluPlugin plugin = getPluginForName(pluginName);
        //check if a plugin with the name is loaded
        if(plugin==null) throw new UnsupportedOperationException("Specified plugin with name: " + pluginName + " is not loaded!");
        if(!(plugin instanceof ScriptablePlugin))
            throw  new UnsupportedOperationException("Plugin keine Instanz von ScriptablePlugin");
      }
    }



  ////////////////////////////////////////////////////////////////////////
  ///////////////////  statische Hilfsmethoden  //////////////////////////
  ////////////////////////////////////////////////////////////////////////
  private static XuluObject findObjectInDataPool(String name, XuluModellingPlatform appl) throws Exception {
    // Objekt im Datenpool suchen
    XuluObject[] o = appl.getDataPool().getObjects(name);
    if ( o.length == 0 )
      throw new XuluDataException("No XuluObject with name '"+name+"' found in the data pool!");
    if ( o.length > 1 )
      throw new XuluDataException(o.length+" XuluObjects with name '"+name+"' found in the data pool!");
    return o[0];
  }

  private static InstantiationFactory findInstantiationFactoryInRegistry(Class type, boolean dialog, XuluModellingPlatform appl) throws Exception {
    // Factory zum Datentyp in Registry suchen
    InstantiationFactory fac = appl.getRegistry().getDefaultFactory(type);
    if ( fac == null )
      throw new TypeMappingException("No factory found to instantiate "+type.getName());
    return fac;
  }

  private static ImportFactory findImportFactoryInRegistry(String facName, boolean dialog, XuluModellingPlatform appl) throws Exception {
//    return (ImportFactory)Class.forName(facName).newInstance();
    // Factory zum Datentyp in Registry suchen
    ImportFactory[] fac = appl.getRegistry().getImportFactories(facName);
    if ( fac == null || fac.length == 0 )
      throw new TypeMappingException("No factory with name '"+facName+"' found in registry!");
    if ( fac.length > 1 )
      throw new TypeMappingException(fac.length+" factories with name '"+facName+"' found in registry!");
    return fac[0];
  }

  private static void createNewObject(Class type, String name, Object sample, boolean dialog, XuluModellingPlatform appl) throws Exception {
    InstantiationFactory fac = findInstantiationFactoryInRegistry(type,dialog,appl);
    if ( !XuluObject.class.isAssignableFrom( fac.getInstanceType() ) )
      throw new XuluDataException("Factory does not create a XuluObject (only XuluObjects can be handled by the data pool!)");
    // Objekt erzeugen
    Object o = (sample==null) ? fac.newInstance(dialog) : fac.newInstance(dialog,sample);
    // Objekt in Datenpool einfuegen
    ((XuluObject)o).setDescription(name);
    try {
      XuluGUIUtil.insertObjectToDataPool( appl.getMainFrame(), o, appl.getDataPool(), true, autoSkipDupl );
    } catch ( AlreadyHandledException err ) {
      // Einfuegung wurde (ueber Anwender-Dialog) fuer das Objekt uebergangen
      // und soll auch fuer alle alle Folge-Konflikte uebergangen werden
      // --> Schleife fortsetzen, aber mit autoSkip
      autoSkipDupl = true;
    }
  }

  private static void importObjectFromFile(String facName, File[] sourceFile, String name, boolean dialog, XuluModellingPlatform appl) throws Exception {
    ImportFactory fac = findImportFactoryInRegistry(facName,dialog,appl);
    if ( !XuluObject.class.isAssignableFrom( fac.getImportType() ) )
      throw new XuluDataException("Factory does not create a XuluObject (only XuluObjects can be handled by the data pool!)");

    // Eingabe-Daten umformatieren
    Object facSource = sourceFile;
    if ( !Object[].class.isAssignableFrom( fac.getImportSourceType() ) )
      // nur die erste gegebene Datei beruecksichtigen
      facSource = sourceFile[0];

    // Import in Recent-Imports aufnehmen
    appl.getRegistry().getRecentImports().add( new IODefinition.ImportDefinition(facSource,fac,appl.getRegistry()) );
    // Objekt erzeugen
    Object o = fac.importObject( XuluGUIUtil.convertImportSourceAsNeeded(fac,facSource),appl.getRegistry());
    // Objekt in Datenpool einfuegen
    ((XuluObject)o).setDescription(name);
    try {
      XuluGUIUtil.insertObjectToDataPool( appl.getMainFrame(), o, appl.getDataPool(), true, autoSkipDupl );
    } catch ( AlreadyHandledException err ) {
      // Einfuegung wurde (ueber Anwender-Dialog) fuer das Objekt uebergangen
      // und soll auch fuer alle alle Folge-Konflikte uebergangen werden
      // --> Schleife fortsetzen, aber mit autoSkip
      autoSkipDupl = true;
    }
  }

}
