/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.appl;

import schmitzm.swing.MultipleOptionPane;
import schmitzm.swing.InputOption;
import schmitzm.swing.SelectionInputOption;

import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.Handler;
import edu.bonn.xulu.appl.HandlerFactory;
import edu.bonn.xulu.appl.VisualisationTool;
import edu.bonn.xulu.appl.XuluConstants;
import edu.bonn.xulu.io.Factory;
import edu.bonn.xulu.io.AbstractFactory;
import edu.bonn.xulu.gui.DataPoolInputOption;

/**
 * Mit dieser Factory werden Instanzen des {@link VisualisationHandler}
 * erstellt und modifiziert. Die benoetigten Informationen (darzustellendes
 * Datenpool-Objekt und Visualisierungstool) werden interaktiv ueber Dialoge vom
 * Anwender abgefragt.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of  Bonn/Germany)
 * @version 1.0
 */
public class VisualisationHandlerFactory extends AbstractFactory implements HandlerFactory {
  /**
   * Erzeugt eine neue Handler-Factory.
   */
  public VisualisationHandlerFactory() {
    this("");
  }

  /**
   * Erzeugt eine neue Handler-Factory.
   * @param name Beschreibung der Factory
   */
  public VisualisationHandlerFactory(String name) {
    super(name);
  }

  /**
   * Erzeugt einen neuen {@link VisualisationHandler}.
   * @param appl Instanz der Xulu-Applikation
   */
  public VisualisationHandler createHandler(XuluModellingPlatform appl) {
    Object[] input = showDialog(appl,null,null);
    if ( input == null || input.length < 2 )
      return null;
    return input[1] instanceof VisualisationTool ? new VisualisationHandler(input[0],(VisualisationTool)input[1],this,appl) : new VisualisationHandler(input[0],(Class)input[1],this,appl);
  }

  /**
   * Veraendert einen bestehenden {@link VisualisationHandler}.
   * @param appl Instanz der Xulu-Applikation
   * @param handler Handler, der veraendert werden soll
   * @exception UnsupportedOperationException falls der uebergebene Handler
   *            kein {@link VisualisationHandler} ist
   */
  public void modifyHandler(XuluModellingPlatform appl, Handler handler) {
    if ( !(handler instanceof VisualisationHandler) )
      throw new UnsupportedOperationException("VisualisationHandlerFactory can only modify instances of VisualisationHandler.");
    VisualisationHandler visHandler = (VisualisationHandler)handler;
    Object[] input = showDialog(appl,visHandler.getObject(), visHandler.getVisualisationTool() != null ? visHandler.getVisualisationTool() : visHandler.getVisualisationToolClass());
    if ( input != null && input.length >= 2 ) {
      visHandler.setObject(input[0]);
      if ( input[1] instanceof Class ) {
        // Wenn eine Klasse angewaehlt wurde, soll ein neues VisTool-Fenster
        // erzeugt werden. Das bisher gesetzte Tool muss erst geloescht werden,
        // da ansonsten das Setzen der Klasse keine Auswirkungen hat
        visHandler.setVisualisationTool( null );
        visHandler.setVisualisationToolClass( (Class) input[1]);
      }
      else
        visHandler.setVisualisationTool( (VisualisationTool)input[1] );
    }
  }

  private Object[] showDialog(XuluModellingPlatform appl, Object object, Object visTool) {
    // existierende Visualisierungstools und Tool-Klassen
    VisualisationTool[] visTools         = appl.getVisualisationManager().getAll();
    Class[]             visToolClasses   = appl.getRegistry().getVisualisationTools();
    Object[]            visToolItems     = new Object[visTools.length+visToolClasses.length];
    String[]            visToolItemDescs = new String[visToolItems.length];
    for (int i=0; i<visToolItemDescs.length; i++) {
      visToolItems[i]     = (i<visTools.length) ? visTools[i] : visToolClasses[i-visTools.length];
      visToolItemDescs[i] = (visToolItems[i] instanceof Class) ? ((Class)visToolItems[i]).getSimpleName()+" ("+XuluConstants.XULUGUI_RES.getString("new")+")" : ((VisualisationTool)visToolItems[i]).getTitle();
    }

    // Dialog erzeugen
    return MultipleOptionPane.showMultipleInputDialog(null,XuluConstants.XULUGUI_RES.getString("Handler"), new InputOption[] {
        new DataPoolInputOption(appl.getDataPool(),XuluConstants.XULUGUI_RES.getString("UpdateObject"),true,object),
        new SelectionInputOption.Combo(XuluConstants.XULUGUI_RES.getString("VisTool")+":",true,visToolItems,visTool,visToolItemDescs),
    });
  }
}
