/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.gui;

import java.awt.BorderLayout;
import javax.swing.table.AbstractTableModel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.TableModel;
import javax.swing.ListSelectionModel;

import schmitzm.swing.ObjectDisplayContainer;
import schmitzm.data.property.Property;
import schmitzm.data.property.MatrixProperty;
import schmitzm.data.property.PropertyWriteAccess;
import schmitzm.data.property.AccessViolationException;

import edu.bonn.xulu.gui.XuluGUIMessages;

import schmitzm.temp.BaseTypeUtil;

/**
 * Diese Klasse stellt eine abstrakte Oberklasse fuer die Darstellung
 * von tabellarischen <code>ObjectDisplayContainer</code> dar.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public abstract class DisplayContainer_Table extends ObjectDisplayContainer {
  protected JTable                        table      = null;
  protected JScrollPane                   scrollPane = null;
  protected DisplayContainerTableModel    tableModel = null;

  /**
   * Erzeugt eine neue Darstellungskomponente.
   */
  public DisplayContainer_Table() {
    this(null);
  }

  /**
   * Erzeugt eine neue Darstellungskomponente und zeigt sofort ein
   * Objekt an.
   * @param data anzuzeigendes Objekt
   */
  public DisplayContainer_Table(Object data) {
    super();
    this.setLayout( new BorderLayout() );
    this.tableModel = createTableModel(null); // Objekt wird erst gesetzt, wenn Tabelle erzeugt ist!
    this.table      = new JTable(tableModel);
    this.scrollPane = new JScrollPane( table );
    this.add( scrollPane );
    setObject(data);
  }

  /**
   * Erzeugt das TableModel fuer die Tabelle
   * @param o darzustellendes Objekt (kann <code>null</code> sein!)
   */
  protected abstract DisplayContainerTableModel createTableModel(Object o);

  /**
   * Aktualisiert die Darstellung des aktuell angezeigten Objekts.
   */
  public void refresh() {
    if (tableModel != null) {
      tableModel.setData(getObject());
      tableModel.fireTableStructureChanged();
      scrollPane.getVerticalScrollBar().setValue(0);
      scrollPane.getHorizontalScrollBar().setValue(0);
    }
  }

  /**
   * Liefert alle in der Tabelle selektierten Tabellenelemente.
   * @return immer leerer <code>Object</code>-Array
   */
  public Object[] getSelectedElements() {
    return new Object[0];
  }

  ///////////////////////////////////////////////////////////////////
  // TableModel fuer die MatrixProperty-Tabelle
  ///////////////////////////////////////////////////////////////////
  /**
   * Diese Klasse stellt ein TableModel fuer MatrixProperty-Instanzen dar.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  protected abstract class DisplayContainerTableModel extends AbstractTableModel {
    /**
     * Setzt das Datenobjekt, das in der Tabelle dargestellt wird.
     * @param obj Objekt (kann <code>null</code> sein!)
     */
    public abstract void setData(Object obj);
  }

}
