/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.gui;

import java.awt.Dimension;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;

import schmitzm.swing.TreeSelectionDialog;
import schmitzm.swing.MultipleOptionPane;
import schmitzm.swing.InputOption;
import schmitzm.swing.SelectionInputOption;
import schmitzm.swing.event.InputOptionAdapter;
import schmitzm.swing.table.MutableTable;
import schmitzm.swing.tree.EmptyNode;

import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.DataPool;
import edu.bonn.xulu.appl.ModelControlManager;
import edu.bonn.xulu.appl.Handler;
import edu.bonn.xulu.appl.HandlerFactory;
import edu.bonn.xulu.appl.EventHandler;
import edu.bonn.xulu.appl.EventManager;

import edu.bonn.xulu.appl.XuluConstants;
import edu.bonn.xulu.gui.DataPoolTreeModel;
import edu.bonn.xulu.gui.DataPoolInputOption;
import edu.bonn.xulu.gui.ObjectOrModelInputOption;
import edu.bonn.xulu.gui.ModelManagerTreeModel;
import edu.bonn.xulu.gui.XuluGUIMessages;

import edu.bonn.xulu.plugin.gui.AbstractManagerFrame_BasicTable;

// nur fuer Doku
import javax.swing.JInternalFrame;


/**
 * Diese Klasse stellt eine GUI fuer den {@linkplain EventManager Xulu-EventManager}
 * dar. Diese stellt (in einem {@link JInternalFrame}) tabellarisch alle {@link EventHandler}
 * in 3 Spalten dar:
 * <ol>
 *   <li>Objekt, das beobachtet wird</li>
 *   <li>Ereignistyp, auf den reagiert wird</li>
 *   <li>Handler, der ausgefuehrt wird, wenn das Objekt das Ereignis ausgeloest hat</li>
 * </ol>
 * Ueber ein Popup-Menue koennen dem Event-Manager Event-Handler hinzugefuegt oder
 * entfernt werden. Durch einen Doppelklick auf eine Tabellenzeile wird der
 * jeweilige Event-Handler editiert.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class EventManagerFrame_BasicTable extends AbstractManagerFrame_BasicTable {
  private XuluModellingPlatform appl = null;

  /**
   * Erzeugt ein neues EventManager-Fenster.
   * @param appl  Instanz der XuluModellingPlatform
   * @param title Titel fuer das Fenster
   */
  public EventManagerFrame_BasicTable(XuluModellingPlatform appl, String title) {
    this(appl,title, new EventManagerTableModel(appl));
  }

  /**
   * Erzeugt ein neues EventManager-Fenster.
   * @param appl  Instanz der XuluModellingPlatform
   * @param title Titel fuer das Fenster
   * @param size  Groesse des Fensters
   */
  public EventManagerFrame_BasicTable(XuluModellingPlatform appl, String title, Dimension size) {
    this(appl,title, new EventManagerTableModel(appl), size);
  }

  /**
   * Erzeugt ein neues EventManager-Fenster.
   * @param appl  Instanz der XuluModellingPlatform
   * @param title Titel fuer das Fenster
   * @param managerTableModel TableModel fuer den Event-Manager
   */
  private EventManagerFrame_BasicTable(XuluModellingPlatform appl, String title, EventManagerTableModel managerTableModel) {
    this(appl,title, managerTableModel, new Dimension(300,350));
  }

  /**
   * Erzeugt ein neues EventManager-Fenster.
   * @param appl  Instanz der XuluModellingPlatform
   * @param title Titel fuer das Fenster
   * @param managerTableModel TableModel fuer den Event-Manager
   * @param size  Groesse des Fensters
   */
  private EventManagerFrame_BasicTable(XuluModellingPlatform appl, String title, EventManagerTableModel managerTableModel, Dimension size) {
    super(title, managerTableModel, size,MutableTable.ITEM_ADD | MutableTable.ITEM_REMOVE);
    this.appl = appl;
  }

  /**
   * Diese Klasse stellt ein tabellarisches Daten-Modell fuer den Xulu-EventManager dar.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  private static class EventManagerTableModel extends AbstractManagerFrame_BasicTable.AbstractManagerTableModel {
    private XuluModellingPlatform appl = null;

    /**
     * Erzeugt ein neues Daten-Modell fuer einen <code>EventManager</code>.
     * @param appl Instanz der Xulu-Anwendung
     */
    public EventManagerTableModel(XuluModellingPlatform appl) {
      super(appl.getEventManager());
      this.appl = appl;
    }

    /**
     * Liefert die Spaltennamen der Tabelle.
     */
    @Override
    public String[] createColumnNames() {
      return new String[] {
          XULUGUI_RES.getString("Object"),
          XULUGUI_RES.getString("Event"),
          XULUGUI_RES.getString("Handler")
      };
    }
    
    /**
     * Liefert einen Wert der Tabelle. Fuer Spalte 0 wird der jeweilige Titel
     * der Komponente geliefert, fuer Spalte 1 der Klassenname des Ereignisses
     * und fuer Spalte 3 eine Beschreibung des Handlers.
     * @param row Tabellen-Zeile
     * @param col Tabellen-Spalte
     */
    public Object getValueAt(int row, int col) {
      switch (col) {
        case 0: // erste Spalte: Objekt
          return manager.get(row).getTitle();
        case 1: // zweite Spalte: Event
          return ( (EventHandler) manager.get(row)).getEventType().getSimpleName();
        case 2: // dritte Spalte: Handler
          Handler handler = ( (EventHandler) manager.get(row)).getHandler();
          return handler.getFactory() == null ? handler.getClass().getSimpleName() : handler.getFactory().getName();
      }
      return null;
    }

    /**
     * Prueft, ob eine Tabellen-Zelle editierbar ist. Dies ist fuer <b>keine</b>
     * @param row Tabellen-Zeile
     * @param col Tabellen-Spalte
     */
    public boolean isCellEditable(int row, int col) {
      return false;
    }

    /**
     * Entfernt einen Eintrag aus dem <code>AbstractManager</code> und
     * somit aus der Tabelle. Wird aufgerufen, wenn der Menuepunkt
     * "Entfernen" aus dem Kontextmenue gewaehlt wird.
     * @param row Tabellenzeile
     * @see AbstractManager#remove(int)
     */
    public void performRemoveRow(int row) {
      manager.remove(row);
    }

    /**
     * Fragt ueber Anwender-Dialoge ein Objekt, ein Ereignis und einen
     * Handler ab und fuegt dem Event-Manager einen entsprechenden
     * Event-Handler hinzu.
     */
    public void performAddRow() {
      try {
        EventHandler eh = createEventHandler(appl);
        if (eh != null)
          manager.add(eh);
      } catch(Exception err) {
        XuluGUIMessages.showError(null,err);
      }
    }

    /**
     * Fragt ein Objekt, einen moeglichen Ereignis-Typ, sowie einen Handler
     * ab und erzeugt daraus einen Event-Handler.
     * @return <code>null</code> falls einer der Anwender-Dialoge abgebrochen wurde
     */
    private EventHandler createEventHandler(XuluModellingPlatform appl) {
      Object object = null;
      Class eventType = null;
      Handler handler = null;

      final XuluModellingPlatform    xulu            = appl;
      final ObjectOrModelInputOption objectOption    = new ObjectOrModelInputOption(appl,XULUGUI_RES.getString("EventObject"),true,null);
      final SelectionInputOption     eventTypeOption = new SelectionInputOption.Combo(XULUGUI_RES.getString("Event")+":",true);
      final SelectionInputOption     handlerOption   = new SelectionInputOption.Combo(XULUGUI_RES.getString("Handler")+":",true);

      // wenn sich das zu beobachtende Objekt aendert, muss die Auswahl der
      // moeglichen Ereignis-Typen und Handler entsprechend aktualisiert werden
      objectOption.addInputOptionListener( new InputOptionAdapter() {
        public void optionChanged(InputOption option, Object oldValue, Object newValue) {
          // moegliche Ereignis-Typen fuer Objekt setzen
          Class[]  eventTypesForObject = xulu.getEventManager().getEventTypes(objectOption.getValue());
          if ( eventTypesForObject.length < 1 )
            XuluGUIMessages.showError(null,XuluConstants.XULUERR_RES.getString("NoEvents"));
          String[] eventTypeNamesForObject = new String[eventTypesForObject.length];
          for (int i=0; i<eventTypeNamesForObject.length; i++)
            eventTypeNamesForObject[i] = eventTypesForObject[i].getSimpleName();
          eventTypeOption.setSelectionObjects(eventTypesForObject,eventTypeNamesForObject);

          // moegliche Handler fuer Objekt setzen
          HandlerFactory[] handlerFac = xulu.getEventManager().getHandlerFactories();
          if (handlerFac.length < 1)
            XuluGUIMessages.showError(null,XuluConstants.XULUERR_RES.getString("NoHandler"));
          String[] handlerFacDesc = new String[handlerFac.length];
          for (int i = 0; i < handlerFacDesc.length; i++)
            handlerFacDesc[i] = handlerFac[i].getName();
          handlerOption.setSelectionObjects(handlerFac,handlerFacDesc);
        }
      });

      while( handler == null ) {
        // Dialog erzeugen
        Object[] input = MultipleOptionPane.showMultipleInputDialog(null,
            XULUGUI_RES.getString("Event"), new InputOption[] {
            objectOption,
            eventTypeOption,
            handlerOption
        });
        if (input == null || input.length < 1)
          return null; // Dialog abgebrochen

        object = input[0];
        eventType = (Class) input[1];
        handler = ( (HandlerFactory) input[2]).createHandler(xulu);
      }

      // Event-Handler erzeugen
      return new EventHandler(object,eventType,handler,appl);
    }
  }
}

