/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.io.feature.gt;

import edu.bonn.xulu.io.InstantiationFactory;
import edu.bonn.xulu.io.AbstractFactory;
import edu.bonn.xulu.plugin.data.feature.SingleFeatureCollection;
import org.geotools.feature.FeatureCollections;
// nur fuer Doku
import org.geotools.feature.FeatureCollection;

/**
 * Diese Factory erzeugt Standard-Instanzen von {@link SingleFeatureCollection}.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class SingleFeatureCollectionFactory extends AbstractFactory implements InstantiationFactory {
  /**
   * Liefert den Datentyp, der von der Factory erzeugt wird.
   * @return immer {@link SingleFeatureCollection SingleFeatureCollection.class}
   */
  public Class getInstanceType() {
    return SingleFeatureCollection.class;
  }

  /**
   * Erzeugt eine {@link SingleFeatureCollection}, die eine leerer
   * {@link FeatureCollection} beinhaltet. Diese wird ueber die Methode
   * {@link FeatureCollections#newCollection()} erzeugt.
   * @param dialog wird nicht benoetigt, da kein Anwenderdialog benoetigt wird
   *               um eine leere FeatureCollection zu erzeugen
   */
  public SingleFeatureCollection newInstance(boolean dialog) {
    return new SingleFeatureCollection( FeatureCollections.newCollection() );
  }

  /**
   * <b>Noch nicht implementiert!!</b><br>
   * Erzeugt eine (strukturelle) Kopie einer {@link SingleFeatureCollection}.
   * @todo newInstance(Object sample) implementieren
   * @param dialog bestimmt, ob die Vorlagen-Daten sofort uebernommen werden, oder
   *               ueber einen Anwender-Dialog abgeaendert werden koennen
   * @param sample Vorlage
   * @exception IllegalArgumentException falls es sich bei der Vorlage nicht
   *            um eine <code>SingleFeatureCollection</code> handelt
   */
  public SingleFeatureCollection newInstance(boolean dialog, Object sample) {
    if ( sample == null )
      throw new IllegalArgumentException(this.getClass().getSimpleName().concat(" can not create an instance of ").concat(getInstanceType().getSimpleName()).concat(" from null") );
    if ( !getInstanceType().isAssignableFrom(sample.getClass()) )
      throw new IllegalArgumentException(this.getClass().getSimpleName().concat(" can not create an instance of ").concat(getInstanceType().getSimpleName()).concat(" from ").concat(sample.getClass().getName()) );
    throw new UnsupportedOperationException("newInstance(Object) not yet implemented!");
//    SingleFeatureCollection sfc = ...
//    // Style von der Vorlage uebernehmen
//      edu.bonn.xulu.plugin.io.IOUtil.setStyleToXuluObject(
//        sfc,
//        edu.bonn.xulu.plugin.io.IOUtil.getStyleFromXuluObject( (SingleFeatureCollection) sample )
//      );
//    return sfc;
  }

}
