/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.vis;

import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JMenuItem;
import javax.swing.JFrame;
import javax.swing.JToggleButton;
import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import java.util.Vector;
import java.util.HashSet;
import java.util.Set;

import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.DefaultTableXYDataset;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.xy.XYAreaRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.chart.renderer.xy.XYBarRenderer;
import org.jfree.chart.plot.DefaultDrawingSupplier;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.urls.StandardXYURLGenerator;
import org.jfree.chart.plot.XYPlot;

import org.apache.log4j.Logger;

import schmitzm.swing.JPanel;
import schmitzm.swing.ButtonGroup;
import schmitzm.swing.MultiSplitPane;
import schmitzm.swing.table.MutableTable;
import schmitzm.swing.table.AbstractMutableTableModel;
import schmitzm.data.property.Property;
import schmitzm.data.property.ValueProperty;
import schmitzm.data.property.ListProperty;
import schmitzm.data.property.ListPropertyReadAccess;
import schmitzm.data.property.MatrixProperty;
import schmitzm.data.property.PropertyReadAccess;

import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.VisualisationTool;
import edu.bonn.xulu.appl.VisualisationUpdateListener;
import edu.bonn.xulu.appl.XuluConstants;

import static schmitzm.jfree.JFreeChartUtil.RESOURCE;
import static edu.bonn.xulu.appl.XuluConstants.XULU_VIS_RES;

/**
 * Diese Klasse stellt ein Visualisierungstool dar, in dem Zahlenreihen in Form
 * von Charts angezeigt werden koennen. Folgende Datentypen werden unterstuetzt:
 * <ul>
 *   <li></li>
 * </ul>
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class JFreeChartVisualisationTool extends JFrame implements VisualisationTool {
  private static final Logger LOGGER = Logger.getLogger(JFreeChartVisualisationTool.class.getSimpleName());

  /** Zaehlt die Instanzen (nur fuer den Titel) */
  private static int instanceCount = 1;
  /** Speichert die verteilten UpdateListener */
  protected Vector<VisualisationUpdateListener> updateListener = new Vector<VisualisationUpdateListener>();

  protected XYItemRenderer  AREA_RENDERER  = new XYAreaRenderer(XYAreaRenderer.AREA);
  protected XYItemRenderer  LINE_RENDERER  = new XYLineAndShapeRenderer(true,false); //new XYAreaRenderer(XYAreaRenderer.SHAPES_AND_LINES);
  protected XYItemRenderer  BAR_RENDERER   = new XYBarRenderer(0.5d);
  protected XYItemRenderer  POINT_RENDERER = new XYLineAndShapeRenderer(false,true); //new XYAreaRenderer(XYAreaRenderer.SHAPES);

  /** Speichert das Panel, in dem die einzelnen Charts angezeigt werden. */
  protected ChartPanel chartPanel = null;
  /** Enthaelt die im Diagramm angezeigten Daten. */
//  protected DefaultCategoryDataset dataSet = null;
  protected DefaultTableXYDataset dataSet = null;
  /** Enthaelt den {@link Plot} des {@linkplain JFreeChart Chart}. */
  protected XYPlot chartPlot = null;

  /** Button um das Chart als Punkt-Diagramm einzustellen. */
  protected JToggleButton pointButton = null;
  /** Button um das Chart als Linien-Diagramm einzustellen. */
  protected JToggleButton lineButton  = null;
  /** Button um das Chart als Balken-Diagramm einzustellen. */
  protected JToggleButton barButton   = null;
  /** Button um das Chart als Flaechen-Diagramm einzustellen. */
  protected JToggleButton areaButton   = null;
  /** Speichert das Panel, in dem die Button angezeigt werden, mit denen
   *  zwischen den verschiedenen Styles gewechselt werden kann. */
  protected JPanel buttonPanel = null;
  /** Buttongroup, die Steuert, das im {@link buttonPanel} immer nur ein
   *  Toggle-Button aktiviert ist */
  protected ButtonGroup buttonGroup = null;
  /** Tabelle, in der die Datenobjekte angezeigt werden. */
  protected MutableTable objectTable = null;
  /** Split-Pane, in dem Daten-Liste und Chart angezeigt werden. */
  protected MultiSplitPane horSplitPane = null;

  /**
   * Erzeugt ein neues {@code JFreeChartVisualisationTool}-Fenster.
   */
  public JFreeChartVisualisationTool() {
    super();
    this.setIconImage( XuluConstants.XULU_ICON.getImage() );
    this.setTitle(XULU_VIS_RES.getString("JFreeChartVisualiationTool.FrameTitle").concat(" (").concat(String.valueOf(instanceCount++)).concat(")"));

    // Panel, in dem die Charts angezeigt werden
    this.chartPanel = new ChartPanel(null);

//    this.dataSet    = new DefaultCategoryDataset();
    this.dataSet    = new DefaultTableXYDataset();

    // Button, mit denen zwischen den Anzeigeformen gewechselt werden kann
    this.buttonPanel = new JPanel();
    this.buttonPanel.setLayout( new FlowLayout(FlowLayout.CENTER, 5, 5) );
    this.buttonGroup = new ButtonGroup();
    this.pointButton = createButton( RESOURCE.getString("diagram.points") );
    this.lineButton  = createButton( RESOURCE.getString("diagram.lines") );
    this.barButton   = createButton( RESOURCE.getString("diagram.bars") );
    this.areaButton  = createButton( RESOURCE.getString("diagram.areas") );
    this.pointButton.setSelected(true);
    // Tabelle, in der die Chart-Datensaetze angezeigt werden
    this.objectTable = new MutableTable( new ChartDataTableModel(), MutableTable.ITEM_REMOVE ) {
      public PopupMenu createPopupMenu(int mask) {
        PopupMenu menu = super.createPopupMenu(mask);
        JMenuItem rotateItem = new JMenuItem( XULU_VIS_RES.getString("JFreeChartVisualiationTool.RotateData") );
        rotateItem.addActionListener( new ActionListener() {
          public void actionPerformed(ActionEvent e) {
            performChartDataRotation();
          }
        });
        menu.add( rotateItem );
        return menu;
      }
    };
    this.objectTable.setTableHeader(null);

    // GUI zusammensetzen
    this.horSplitPane = new MultiSplitPane(2,MultiSplitPane.HORIZONTAL_SPLIT);
    this.horSplitPane.setResizeWeigth( new double[] {0.2,0.8} );
    this.horSplitPane.setDividerSize(5);
    this.horSplitPane.setInnerBorder(null);
    this.horSplitPane.getContainer(0).add(objectTable.createScrollPane(),BorderLayout.CENTER);
    this.horSplitPane.getContainer(1).add(chartPanel);

    this.setPreferredSize( new Dimension(600,400) );

    Container contentPane = this.getContentPane();
    contentPane.setLayout( new BorderLayout() );
    contentPane.add(horSplitPane, BorderLayout.CENTER);
    contentPane.add(buttonPanel, BorderLayout.SOUTH);
    pack();
    checkChartPlotExists();
  }

  private JToggleButton createButton(String caption) {
    JToggleButton button = new JToggleButton( new StyleButtonAction(caption) );
    this.buttonPanel.add(button);
    this.buttonGroup.add(button);

    return button;
  }

  /**
   * Macht nichts, da die Xulu-Applikation nicht fuer die Visualisierung
   * benoetigt wird.
   */
  public void initialize(XuluModellingPlatform appl) {
  }

  /**
   * Verbirgt das {@code JFreeChartVisualisationTool}-Fenster, entfernt alle (durch
   * {@code UpdateListener}) damit verbundenen Charts und "vernichtet"
   * anschliessend das Fenster mit {@link #dispose()}.
   */
  public void close() {
    // Fenster unsichtbar machen
    setVisible(false);
    // Alle Layer schliessen
    for (int i=0; i<updateListener.size(); i++)
      ((VisualisationUpdateListener)updateListener.elementAt(i)).close();
    // Frame "vernichten"
    dispose();
  }

  /**
   * Setzt den Titel des Fensters und des Charts.
   * @param title neue Bezeichnung
   */
  public void setTitle(String title) {
    super.setTitle(title);
    if ( chartPanel != null && chartPanel.getChart() != null )
      chartPanel.getChart().setTitle(title);
  }

  /**
   * Prueft, ob Instanzen einer bestimmten Klasse visualisiert werden koennen.
   * {@code JFreeChartVisualisationTool} kann Instanzen folgende Klassen
   * darstellen:
   * <ul>
   *   <li>{@link ListProperty} mit {@linkplain Number numerischen} Elementen</li>
   *   <li>{@link MatrixProperty} mit {@linkplain Number numerischen} Elementen</li>
   * </ul>
   */
  public boolean isVisualisable(Class c) {
    return ListProperty.class.isAssignableFrom( c ) ||
           MatrixProperty.class.isAssignableFrom( c );
  }

  /**
   * Prueft, ob ein Objekt visualisiert werden kann.
   * {@code JFreeChartVisualisationTool} kann folgende Objekt-Instanzen darstellen:
   * <ul>
   *   <li>{@link ListProperty} mit {@linkplain Number numerischen} Elementen</li>
   *   <li>{@link MatrixProperty} mit {@linkplain Number numerischen} Elementen</li>
   * </ul>
   * @param obj ein Objekt
   */
  public boolean isVisualisable(Object obj) {
    if ( obj==null )
      return false;
    if ( !isVisualisable( obj.getClass() ) )
      return false;

    // Pruefen, ob ValueProperty mit numerischem Inhalt
    if ( !(obj instanceof ValueProperty) ||
         !Number.class.isAssignableFrom( ((ValueProperty)obj).getType() ) )
      return false;

    // Wenn ListProperty, dann darstellbar
    if ( obj instanceof ListProperty )
      return true;
    // Wenn MatrixProperty, dann darf diese maximal 2-dimensional sein
    if ( obj instanceof MatrixProperty )
      return ((MatrixProperty)obj).getDimension() <= 2;

    return false;
  }

  /**
   * Visualisiert ein Objekt.
   * @param obj  Objekt
   * @param desc Beschreibung zu dem Objekt, die in der Layer-Uebersicht
   *             angezeigt wird (kann <code>null</code> sein)
   * @exception java.lang.UnsupportedOperationException falls das Objekt nicht
   *            visualisiert werden kann
   * @return immer {@code true}
   */
  public boolean add(Object obj, String desc) {
    // Pruefen, ob Objekt dargestellt werden kann
    if ( !isVisualisable(obj) )
      if ( obj==null )
        throw new UnsupportedOperationException( getClass().getSimpleName().concat(" can not visualise null-Objects!") );
      else
        throw new UnsupportedOperationException( getClass().getSimpleName().concat(" can not visualise instances of ").concat(obj.getClass().getName()) );

    VisualisationUpdateListener newListener = null;
    if ( obj instanceof ListProperty )
      newListener = new ListPropertyUpdateListener(this, (ListProperty)obj);
    else if ( obj instanceof MatrixProperty )
      newListener = new MatrixPropertyUpdateListener(this, (MatrixProperty)obj);
    else
      // Kann eigentlich nicht vorkommen!
      throw new UnsupportedOperationException( getClass().getSimpleName().concat(" can not visualise instances of ").concat(obj.getClass().getName()) );
    updateListener.add(newListener);
    newListener.refresh();
    ((ChartDataTableModel)objectTable.getModel()).fireTableDataChanged();
    return true;
  }

  /**
   * Schliesst alle Darstellungen eines Objekts.
   * @param obj zu entfernendes Objekt
   * @return <code>false</code> falls das Objekt nicht (mehr) visualisiert wird
   */
  public boolean remove(Object obj) {
    boolean objRemoved = false;
    for (int i=updateListener.size()-1; i>=0; i--) {
      VisualisationUpdateListener l = (VisualisationUpdateListener)updateListener.elementAt(i);
      if ( l.getSourceObject().equals(obj) ) {
        l.close();
        updateListener.remove(l);
        objRemoved = true;
      }
    }
    if ( objRemoved )
      ((ChartDataTableModel)objectTable.getModel()).fireTableDataChanged();

    return objRemoved;
  }

  /**
   * Aktualisiert alle Visualisierungen eines Objekts.
   * @param obj zu aktualisierendes Objekt
   * @return <code>false</code> falls das Objekt nicht (mehr) visualisiert wird
   */
  public boolean update(Object obj) {
    boolean objUpdated = false;
    for (int i=updateListener.size()-1; i>=0; i--) {
      VisualisationUpdateListener l = updateListener.elementAt(i);
      if ( l.getSourceObject().equals(obj) ) {
        l.refresh();
        objUpdated = true;
      }
    }
    return objUpdated;
  }

  /**
   * Prueft, ob das ChartPanel bereits einen Plot besitzt. Wenn nicht,
   * wird ein Standard-Chart (ohne Daten) generiert.
   */
  private void checkChartPlotExists() {
    if ( chartPlot == null ) {
        // Standard-Chart verwenden (ohne Daten)
        this.chartPanel.setChart(
          ChartFactory.createScatterPlot(
            this.getTitle(),
            "",
            "",
            null,
            PlotOrientation.VERTICAL,
            true, // legend
            true, // tooltips
            true  // URL
          )
        );
        this.chartPlot = chartPanel.getChart().getXYPlot();
    }
  }

  /**
   * Aktualisiert die Datenbasis des Charts, so dass die Auswirkung
   * geaenderter oder neuer Daten sichtbar wird.
   */
  protected void updateChartData() {
//    checkChartPlotExists();
    chartPlot.setDataset( dataSet );
  }

  /**
   * Aktualisiert das Chart-Layout gemaess des in der GUI ausgewaehlten
   * Chart-Typ.
   */
  protected void updateChartType() {
//    checkChartPlotExists();
    if ( buttonGroup.getSelectedButton() == null )
      pointButton.setSelected(true);
    AbstractButton button = buttonGroup.getSelectedButton();

    XYItemRenderer newRenderer = null;
    if ( button == pointButton )
      newRenderer = POINT_RENDERER;
    // Linien-Diagramm
    if ( button == lineButton )
      newRenderer = LINE_RENDERER;
    // Balken-Diagramm
    if ( button == barButton )
      newRenderer = BAR_RENDERER;
    // Flaechen-Diagramm
    if ( button == areaButton )
      newRenderer = AREA_RENDERER;

    if ( newRenderer != null ) {
      newRenderer.setBaseToolTipGenerator( new StandardXYToolTipGenerator() );
      newRenderer.setURLGenerator( new StandardXYURLGenerator() );
      chartPlot.setDrawingSupplier( new DefaultDrawingSupplier() );
      chartPlot.setRenderer(newRenderer);
    }
  }

  private void performChartDataRotation() {
    int selRow[] = objectTable.getSelectedRows();
    for (int r=selRow.length-1; r>=0; r--)
      ((UpdateListener)updateListener.elementAt(r)).rotateVisualisationObject();
  }

//  /**
//   * Erzeugt eine neues Chart aus dem {@link #dataSet}.
//   */
//  protected JFreeChart createChart() {
//    if ( buttonGroup.getSelectedButton() == null )
//      pointButton.setSelected(true);
//    AbstractButton button = buttonGroup.getSelectedButton();
//
//    // Punkt-Diagramm
//    if ( button == pointButton )
//      return ChartFactory.createScatterPlot(
//        this.getTitle(),
//        "",
//        "",
//        dataSet,
//        PlotOrientation.VERTICAL,
//        true, // legend
//        true, // tooltips
//        true  // URL
//      );
//    // Linien-Diagramm
//    if ( button == lineButton )
//      return ChartFactory.createXYLineChart(
//        this.getTitle(),
//        "",
//        "",
//        dataSet,
//        PlotOrientation.VERTICAL,
//        true, // legend
//        true, // tooltips
//        true  // URL
//      );
//    // Balken-Diagramm
//    if ( button == barButton )
//      return ChartFactory.createXYBarChart(
//        this.getTitle(),
//        "",
//        false,
//        "",
//        new XYBarDataset( (XYDataset)dataSet, 0.5d),
////        (IntervalXYDataset)(dataSet instanceof TableXYDataset ? (TableXYDataset)dataSet : new XYBarDataset( (XYDataset) dataSet, 0.5d)),
////        dataSet,
//        PlotOrientation.VERTICAL,
//        true, // legend
//        true, // tooltips
//        true // URL
//      );
//    // Flaechen-Diagramm
//    if ( button == areaButton )
//      return ChartFactory.createXYAreaChart(
//        this.getTitle(),
//        "",
//        "",
//        dataSet,
//        PlotOrientation.VERTICAL,
//        true, // legend
//        true, // tooltips
//        true // URL
//      );
//
//
//    return null;
//  }

  /**
   * Diese Klasse stellt die Aktion fuer alle Style-Button des
   * {@link JFreeChartVisualisationTool} dar.
   * @author <a href="mailto:Martin.Schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  protected class StyleButtonAction extends AbstractAction {

    /**
     * Erzeugt einen neue Aktion
     */
    public StyleButtonAction(String caption) {
      super();
//      Icon icon = SwingUtil.createImageIconFromResourcePath("resource/icons/small/info.gif","Info");
//      this.putValue( SHORT_DESCRIPTION, ... );
//      this.putValue( SMALL_ICON, ... );
//      if ( icon == null )
      this.putValue( NAME, caption );
    }

    /**
     * Fuehrt die Style-Aktion aus. Fuehrt {@link updateChart()} aus.
     * @param e das ActionEvent
     */
    public void actionPerformed(ActionEvent e) {
      updateChartType();
    }
  }

  /**
   * Daten-Modell fuer die im Chart dargestellten Objekte. Dieses enthaelt
   * nur eine Spalte in der die Bezeichnung des jeweiligen Objekts angezeigt
   * wird.
   * @see {@link UpdateListener#getObjectDesc()}
   * @author <a href="mailto:Martin.Schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  protected class ChartDataTableModel extends AbstractMutableTableModel {
    /**
     * Erzeugt ein neues Tabellen-Modell.
     */
    public ChartDataTableModel() {
      super();
    }

    /**
     * Liefert die Spaltennamen der Tabelle.
     */
    @Override
    public String[] createColumnNames() {
      return new String[] {XULU_VIS_RES.getString("JFreeChartVisualiationTool.ChartObjects")};
    }
    
    /**
     * Liefert die Anzahl an Zeilen, also die Anzahl an visualisierten
     * Datenobjekten.
     */
    public int getRowCount() {
      return updateListener.size();
    }

    /**
     * Liefert die Objekt-Beschreibung des jeweiligen {@link UpdateListener}
     * @param row Zeilenindex (beginnend bei 0)
     * @param col Spaltenindex (beginnend bei 0, wird nicht verwendet)
     */
    public Object getValueAt(int row, int col) {
      return ((UpdateListener)updateListener.elementAt(row)).getObjectDesc();
    }

    /**
     * Entfernt ein Datenobjekt aus dem Chart.
     * @see UpdateListener#close()
     * @param row Zeilenindex (beginnend bei 0)
     */
    public void performRemoveRow(int row) {
      updateListener.elementAt(row).close();
      updateListener.remove(row);
    }

    /**
     * Macht nichts, da die Eintraege in der Chart-Datentabelle nicht
     * veraendert werden koennen.
     * @param row Zeilenindex (beginnend bei 0)
     * @param col Spaltenindex (beginnend bei 0)
     */
    public void performChangeData(int row, int col) {
    }

    /**
     * Macht nichts, da (manuell) keine Eintraege in der Chart-Datentabelle
     * eingefuegt werden koennen.
     */
    public void performAddRow() {
    }
  }

  /**
   * Dieser UpdateListener verknuepft ein Objekt mit einem oder mehreren
   * Charts in einer {@link JFreeChartVisualisationTool}-Instanz. Er kann unmittelbar als
   * {@link schmitzm.data.event.ObjectListener} eingesetzt werden, um
   * mit der Objekt-Aenderung automatisch die Visualisierung zu aktualisieren.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  protected abstract class UpdateListener extends VisualisationUpdateListener {
    /** Enthaelt alle Daten-Serien, die zu dem Objekt gehoeren. */
    protected Set<XYSeries> objectSeries = new HashSet<XYSeries>();

    /**
     * Erzeugt einen neuen <code>UpdateListener</code>.
     * @param tool  Visualisierungstool, mit dem der Listener verbunden ist
     * @param obj   Objekt welches visualisiert wird
     * @param chart Chart in dem das Objekt dargestellt ist
     */
    public UpdateListener(JFreeChartVisualisationTool tool, Object obj) {
      super(tool,obj);
    }

    /**
     * Liefert eine Beschreibung des visualisierten Objekts.
     */
    public abstract String getObjectDesc();

    /**
     * Entfernt das Charts, das zum Objekt gehoert aus dem {@link JFreeChartVisualisationTool}.
     */
    public void close() {
      // Die alten Daten aus dem DataSet entfernen
      removeVisualisationObjectFromDataSet();
      // Chart aktualisieren
      updateChartData();
    }

    /**
     * Aktualisiert das Chart im {@link JFreeChartVisualisationTool}.
     */
    public void refresh() {
      // Die alten Daten aus dem DataSet entfernen
      removeVisualisationObjectFromDataSet();
      // Die Daten des Objekts neu im DataSet einfuegen
      putVisualisationObjectToDataSet();
      // Chart aktualisieren
      updateChartData();
    }

    /**
     * Erzeugt eine {@link XYSeries} fuer die Werte einer Liste und
     * fuegt die Series dem DataSet des {@link JFreeChartVisualisationTool}
     * hinzu.
     * @param listProp eine Liste von Werten
     */
    protected abstract void putVisualisationObjectToDataSet();

    /**
     * Erzeugt eine leere {@link XYSeries} und fuegt diese in die Series-Liste
     * des Objekts, sowie in das DataSet des {@link JFreeChartVisualisationTool}
     * ein.
     */
    protected XYSeries createXYSeries(Comparable rowKey) {
      XYSeries newSeries = new XYSeries(rowKey,true,false);
      objectSeries.add( newSeries );
      dataSet.addSeries( newSeries );
      return newSeries;
    }

    /**
     * Entfernt alle zu dem Objekt gehoerenden Series aus dem DataSet des
     * {@link JFreeChartVisualisationTool}.
     */
    protected void removeVisualisationObjectFromDataSet() {
      for (XYSeries series : objectSeries) {
        dataSet.removeSeries(series);
        series.clear();
      }
      objectSeries.clear();
    }

    /**
     * Verdreht die X- und Y-Achse des Daten-Objekts. Diese Standard-Implementierung
     * macht nichts.
     */
    public void rotateVisualisationObject() {
    }
  }

  /**
   * Dieser UpdateListener verknuepft eine {@link ListProperty} mit einem oder mehreren
   * Charts in einer {@link JFreeChartVisualisationTool}-Instanz. Er kann unmittelbar als
   * {@link schmitzm.data.event.ObjectListener} eingesetzt werden, um
   * mit der Objekt-Aenderung automatisch die Visualisierung zu aktualisieren.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  protected class ListPropertyUpdateListener extends UpdateListener {
    /**
     * Erzeugt einen neuen <code>UpdateListener</code>.
     * @param tool  Visualisierungstool, mit dem der Listener verbunden ist
     * @param obj   Objekt welches visualisiert wird
     * @param chart Chart in dem das Objekt dargestellt ist
     */
    public ListPropertyUpdateListener(JFreeChartVisualisationTool tool, ListProperty obj) {
      super(tool,obj);
    }

    /**
     * Liefert eine Beschreibung des visualisierten Objekts.
     */
    public String getObjectDesc() {
      return ((Property)visObject).getName();
    }

    /**
     * Erzeugt eine {@link XYSeries} fuer die Werte einer Liste und
     * fuegt die Series dem DataSet des {@link JFreeChartVisualisationTool}
     * hinzu.
     * @param listProp eine Liste von Werten
     */
    protected void putVisualisationObjectToDataSet() {
      ListProperty listProp = (ListProperty)visObject;
      XYSeries newSeries = createXYSeries(listProp.getName());
      ListPropertyReadAccess list = listProp.getReadAccess(this);
      for (int i=0; i<list.getCount(); i++) {
        Number value  = (Number)list.getValue(i);
        Number colKey = i;
        newSeries.addOrUpdate(colKey, value);
      }
      list.release();
    }
  }

  /**
   * Dieser UpdateListener verknuepft eine {@link MatrixProperty} mit einem oder mehreren
   * Charts in einer {@link JFreeChartVisualisationTool}-Instanz. Er kann unmittelbar als
   * {@link schmitzm.data.event.ObjectListener} eingesetzt werden, um
   * mit der Objekt-Aenderung automatisch die Visualisierung zu aktualisieren.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  protected class MatrixPropertyUpdateListener extends UpdateListener {
    private boolean horSeries = true;

    /**
     * Erzeugt einen neuen <code>UpdateListener</code>.
     * @param tool  Visualisierungstool, mit dem der Listener verbunden ist
     * @param obj   Objekt welches visualisiert wird
     * @param chart Chart in dem das Objekt dargestellt ist
     */
    public MatrixPropertyUpdateListener(JFreeChartVisualisationTool tool, MatrixProperty obj) {
      super(tool,obj);
    }

    /**
     * Liefert eine Beschreibung des visualisierten Objekts.
     */
    public String getObjectDesc() {
      return ((Property)visObject).getName();
    }

    /**
     * Erzeugt eine {@link XYSeries} fuer jede Zeile einer Matrix und
     * fuegt die Series dem DataSet des {@link JFreeChartVisualisationTool}
     * hinzu.
     * @param matrixProp eine 1- oder 2-dim. Matrix von Werten
     */
    protected void putVisualisationObjectToDataSet() {
      MatrixProperty     matrixProp = (MatrixProperty)visObject;
      String             propName   = matrixProp.getName();
      PropertyReadAccess matrix     = matrixProp.getReadAccess(this);

      // 1-dimensionale Matrix
      if ( matrixProp.getDimension() == 1 ) {
        XYSeries newSeries = createXYSeries(propName);
        for (int i=0; i<matrixProp.getSize(0); i++) {
          Number colKey = i;
          Number value  = (Number)matrix.getValue(i);
          newSeries.addOrUpdate(colKey,value);
        }
      }
      // 2-dimensionale Matrix
      int rowDim = horSeries ? 1 : 0;
      int colDim = horSeries ? 0 : 1;
      if ( matrixProp.getDimension() == 2 ) {
        for (int row=0; row<matrixProp.getSize(rowDim); row++) {
          XYSeries newSeries = createXYSeries(propName + "["+row+"]");
          for (int col=0; col<matrixProp.getSize(colDim); col++) {
            Number colKey = col;
            Number value  = (Number)matrix.getValue( horSeries ? new int[] {col,row} : new int[] {row,col});
            newSeries.addOrUpdate(colKey,value);
          }
        }
      }
      matrix.release();
    }

    /**
     * Verdreht die X- und Y-Achse des Daten-Objekts. Macht nichts, wenn die
     * Matrix 1-dimensional ist.
     */
    public void rotateVisualisationObject() {
      MatrixProperty matrixProp = (MatrixProperty)visObject;
      if ( matrixProp.getDimension() <= 1 )
        return;
      // Alte Daten aus Dataset entfernen
      removeVisualisationObjectFromDataSet();
      // Rotations-Flag verdrehen
      horSeries = !horSeries;
      // Daten neu in das Dataset einfuegen
      putVisualisationObjectToDataSet();
    }

  }


}
